@@ -17,38 +17,40 @@
 package apps
 
 import (
+	"errors"
 	"fmt"
+	"strings"
 
 	"github.com/appc/spec/schema"
 	"github.com/appc/spec/schema/types"
 )
 
-// AppImageType describes a type of an image reference. The reference
-// can either be guessed or be a hash, a URL, a path, or a name. The
-// first option means that the application will have to deduce the
-// actual type (one of the last four).
-type AppImageType int
-
-const (
-	AppImageGuess AppImageType = iota // image type to be guessed
-	AppImageHash                      // image hash
-	AppImageURL                       // image URL with a scheme
-	AppImagePath                      // absolute or relative path
-	AppImageName                      // image name
+var (
+	ErrInvalidSeccompMode     = errors.New("invalid seccomp mode command-line override")
+	ErrInvalidSeccompOverride = errors.New("invalid seccomp command-line override")
 )
 
 type App struct {
-	Image       string                            // the image reference as supplied by the user on the cli
-	ImType      AppImageType                      // the type of the image reference (to be guessed, url, path or hash)
-	Args        []string                          // any arguments the user supplied for this app
-	Asc         string                            // signature file override for image verification (if fetching occurs)
-	Exec        string                            // exec override for image
-	Mounts      []schema.Mount                    // mounts for this app (superseding any mounts in rktApps.mounts of same MountPoint)
-	MemoryLimit *types.ResourceMemory             // memory isolator override
-	CPULimit    *types.ResourceCPU                // cpu isolator override
-	User, Group string                            // user, group overrides
-	CapsRetain  *types.LinuxCapabilitiesRetainSet // os/linux/capabilities-retain-set overrides
-	CapsRemove  *types.LinuxCapabilitiesRevokeSet // os/linux/capabilities-remove-set overrides
+	Name              string                            // the name of the app. If not set, the the image's name will be used.
+	Image             string                            // the image reference as supplied by the user on the cli
+	Args              []string                          // any arguments the user supplied for this app
+	Asc               string                            // signature file override for image verification (if fetching occurs)
+	Exec              string                            // exec override for image
+	WorkingDir        string                            // working directory override for image
+	ReadOnlyRootFS    bool                              // read-only rootfs override.
+	Mounts            []schema.Mount                    // mounts for this app (superseding any mounts in rktApps.mounts of same MountPoint)
+	MemoryLimit       *types.ResourceMemory             // memory isolator override
+	CPULimit          *types.ResourceCPU                // cpu isolator override
+	CPUShares         *types.LinuxCPUShares             // cpu-shares isolator override
+	User, Group       string                            // user, group overrides
+	SupplementaryGIDs []int                             // supplementary gids override
+	CapsRetain        *types.LinuxCapabilitiesRetainSet // os/linux/capabilities-retain-set overrides
+	CapsRemove        *types.LinuxCapabilitiesRevokeSet // os/linux/capabilities-remove-set overrides
+	SeccompFilter     string                            // seccomp CLI overrides
+	OOMScoreAdj       *types.LinuxOOMScoreAdj           // oom-score-adj isolator override
+	UserAnnotations   map[string]string                 // the user annotations of the app.
+	UserLabels        map[string]string                 // the user labels of the app.
+	Environments      map[string]string                 // the environments of the app.
 
 	// TODO(jonboulle): These images are partially-populated hashes, this should be clarified.
 	ImageID types.Hash // resolved image identifier
@@ -60,6 +62,33 @@ type Apps struct {
 	Volumes []types.Volume // volumes available to all apps
 }
 
+// SeccompFilter returns type, filter set and optional errno
+// for a seccomp filter override specified via CLI
+func (a *App) SeccompOverride() (mode string, errno string, set []string, e error) {
+	if a.SeccompFilter == "" {
+		return
+	}
+	for _, i := range strings.Split(a.SeccompFilter, ",") {
+		kv := strings.Split(i, "=")
+		if len(kv) == 2 {
+			switch kv[0] {
+			case "mode":
+				if kv[1] != "remove" && kv[1] != "retain" {
+					e = ErrInvalidSeccompMode
+				}
+				mode = kv[1]
+			case "errno":
+				errno = kv[1]
+			default:
+				e = ErrInvalidSeccompOverride
+			}
+		} else {
+			set = append(set, i)
+		}
+	}
+	return
+}
+
 // Reset creates a new slice for al.apps, needed by tests
 func (al *Apps) Reset() {
 	al.apps = make([]App, 0)
@@ -92,6 +121,9 @@ func (al *Apps) Validate() error {
 
 	f := func(mnts []schema.Mount) error {
 		for _, m := range mnts {
+			if m.AppVolume != nil { // allow app-specific volumes
+				continue
+			}
 			if _, ok := vs[m.Volume]; !ok {
 				return fmt.Errorf("dangling mount point %q: volume %q not found", m.Path, m.Volume)
 			}
@@ -145,13 +177,3 @@ func (al *Apps) GetArgs() [][]string {
 	}
 	return aal
 }
-
-// GetImageIDs returns a list of the imageIDs in al, one per app.
-// The order reflects the app order in al.
-func (al *Apps) GetImageIDs() []types.Hash {
-	var hl []types.Hash
-	for _, a := range al.apps {
-		hl = append(hl, a.ImageID)
-	}
-	return hl
-}
