@@ -18,7 +18,6 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
-	"net"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -50,23 +49,27 @@ func pluginErr(err error, output []byte) error {
 	return err
 }
 
-func (e *podEnv) netPluginAdd(n *activeNet, netns string) (ip, hostIP net.IP, err error) {
+// Executes a given network plugin. If successful, mutates n.runtime with
+// the runtime information
+func (e *podEnv) netPluginAdd(n *activeNet, netns string) error {
 	output, err := e.execNetPlugin("ADD", n, netns)
 	if err != nil {
-		return nil, nil, pluginErr(err, output)
+		return pluginErr(err, output)
 	}
 
 	pr := cnitypes.Result{}
 	if err = json.Unmarshal(output, &pr); err != nil {
 		err = errwrap.Wrap(fmt.Errorf("parsing %q", string(output)), err)
-		return nil, nil, errwrap.Wrap(fmt.Errorf("error parsing %q result", n.conf.Name), err)
+		return errwrap.Wrap(fmt.Errorf("error parsing %q result", n.conf.Name), err)
 	}
 
 	if pr.IP4 == nil {
-		return nil, nil, nil
+		return nil // TODO(casey) should this be an error?
 	}
 
-	return pr.IP4.IP.IP, pr.IP4.Gateway, nil
+	// All is well - mutate the runtime
+	n.runtime.MergeCNIResult(pr)
+	return nil
 }
 
 func (e *podEnv) netPluginDel(n *activeNet, netns string) error {
@@ -116,6 +119,7 @@ func (e *podEnv) execNetPlugin(cmd string, n *activeNet, netns string) ([]byte,
 	}
 
 	vars := [][2]string{
+		{"CNI_VERSION", "0.1.0"},
 		{"CNI_COMMAND", cmd},
 		{"CNI_CONTAINERID", e.podID.String()},
 		{"CNI_NETNS", netns},
