@@ -24,6 +24,9 @@ import (
 	"path/filepath"
 	"sort"
 	"strings"
+	"syscall"
+
+	"golang.org/x/sys/unix"
 
 	"github.com/appc/spec/schema/types"
 	"github.com/hashicorp/errwrap"
@@ -91,9 +94,30 @@ func (e *podEnv) loadNets() ([]activeNet, error) {
 		return nil, fmt.Errorf("networks not found: %v", strings.Join(missing, ", "))
 	}
 
+	// Add the runtime args to the network instances.
+	// We don't do this earlier because we also load networks in other contexts
+	for _, n := range nets {
+		n.runtime.Args = e.netsLoadList.SpecificArgs(n.conf.Name)
+	}
 	return nets, nil
 }
 
+// podNSCreate creates the network namespace and saves a reference to its path.
+// NewNS will bind-mount the namespace in /run/netns, so we write that filename
+// to disk.
+func (e *podEnv) podNSCreate() error {
+	podNS, err := ns.NewNS()
+	if err != nil {
+		return err
+	}
+	e.podNS = podNS
+
+	if err := e.podNSPathSave(); err != nil {
+		return err
+	}
+	return nil
+}
+
 func (e *podEnv) podNSFilePath() string {
 	return filepath.Join(e.podRoot, "netns")
 }
@@ -122,16 +146,17 @@ func podNSerrorOK(podNSPath string, err error) bool {
 	}
 }
 
-func (e *podEnv) podNSLoad() (ns.NetNS, error) {
+func (e *podEnv) podNSLoad() error {
 	podNSPath, err := e.podNSPathLoad()
 	if err != nil && !podNSerrorOK(podNSPath, err) {
-		return nil, err
+		return err
 	} else {
 		podNS, err := ns.GetNS(podNSPath)
 		if err != nil && !podNSerrorOK(podNSPath, err) {
-			return nil, err
+			return err
 		}
-		return podNS, nil
+		e.podNS = podNS
+		return nil
 	}
 }
 
@@ -149,11 +174,38 @@ func (e *podEnv) podNSPathSave() error {
 	return nil
 }
 
+func (e *podEnv) podNSDestroy() error {
+	if e.podNS == nil {
+		return nil
+	}
+
+	// Close the namespace handle
+	// If this handle also *created* the namespace, it will delete it for us.
+	_ = e.podNS.Close()
+
+	// We still need to try and delete the namespace ourselves - no way to know
+	// if podNS.Close() did it for us.
+	// Unmount the ns bind-mount, and delete the mountpoint if successful
+
+	if err := syscall.Unmount(e.podNS.Path(), unix.MNT_DETACH); err != nil {
+		// if already unmounted, umount(2) returns EINVAL - continue
+		if !os.IsNotExist(err) && err != syscall.EINVAL {
+			return errwrap.Wrap(fmt.Errorf("error unmounting netns %q", e.podNS.Path()), err)
+		}
+	}
+	if err := os.RemoveAll(e.podNS.Path()); err != nil {
+		if !os.IsNotExist(err) {
+			return errwrap.Wrap(fmt.Errorf("failed to remove netns %s", e.podNS.Path()), err)
+		}
+	}
+	return nil
+}
+
 func (e *podEnv) netDir() string {
 	return filepath.Join(e.podRoot, "net")
 }
 
-func (e *podEnv) setupNets(nets []activeNet) error {
+func (e *podEnv) setupNets(nets []activeNet, noDNS bool) error {
 	err := os.MkdirAll(e.netDir(), 0755)
 	if err != nil {
 		return err
@@ -167,18 +219,48 @@ func (e *podEnv) setupNets(nets []activeNet) error {
 	}()
 
 	n := activeNet{}
+
+	// did stage0 already make /etc/rkt-resolv.conf (i.e. --dns passed)
+	resolvPath := filepath.Join(common.Stage1RootfsPath(e.podRoot), "etc/rkt-resolv.conf")
+	_, err = os.Stat(resolvPath)
+	if err != nil && !os.IsNotExist(err) {
+		return errwrap.Wrap(fmt.Errorf("error statting /etc/rkt-resolv.conf"), err)
+	}
+	podHasResolvConf := err == nil
+
 	for i, n = range nets {
-		stderr.Printf("loading network %v with type %v", n.conf.Name, n.conf.Type)
+		if debuglog {
+			stderr.Printf("loading network %v with type %v", n.conf.Name, n.conf.Type)
+		}
 
 		n.runtime.IfName = fmt.Sprintf(IfNamePattern, i)
 		if n.runtime.ConfPath, err = copyFileToDir(n.runtime.ConfPath, e.netDir()); err != nil {
 			return errwrap.Wrap(fmt.Errorf("error copying %q to %q", n.runtime.ConfPath, e.netDir()), err)
 		}
 
-		n.runtime.IP, n.runtime.HostIP, err = e.netPluginAdd(&n, e.podNS.Path())
+		// Actually shell out to the plugin
+		err = e.netPluginAdd(&n, e.podNS.Path())
 		if err != nil {
 			return errwrap.Wrap(fmt.Errorf("error adding network %q", n.conf.Name), err)
 		}
+
+		// Generate rkt-resolv.conf if it's not already there.
+		// The first network plugin that supplies a non-empty
+		// DNS response will win, unless noDNS is true (--dns passed to rkt run)
+		if !common.IsDNSZero(&n.runtime.DNS) && !noDNS {
+			if !podHasResolvConf {
+				err := ioutil.WriteFile(
+					resolvPath,
+					[]byte(common.MakeResolvConf(n.runtime.DNS, "Generated by rkt from network "+n.conf.Name)),
+					0644)
+				if err != nil {
+					return errwrap.Wrap(fmt.Errorf("error creating resolv.conf"), err)
+				}
+				podHasResolvConf = true
+			} else {
+				stderr.Printf("Warning: network %v plugin specified DNS configuration, but DNS already supplied", n.conf.Name)
+			}
+		}
 	}
 	return nil
 }
@@ -186,7 +268,9 @@ func (e *podEnv) setupNets(nets []activeNet) error {
 func (e *podEnv) teardownNets(nets []activeNet) {
 
 	for i := len(nets) - 1; i >= 0; i-- {
-		stderr.Printf("teardown - executing net-plugin %v", nets[i].conf.Type)
+		if debuglog {
+			stderr.Printf("teardown - executing net-plugin %v", nets[i].conf.Type)
+		}
 
 		podNSpath := ""
 		if e.podNS != nil {
@@ -277,6 +361,9 @@ func copyFileToDir(src, dstdir string) (string, error) {
 	return dst, err
 }
 
+// loadUserNets will load all network configuration files from the user-supplied
+// configuration directory (typically /etc/rkt/net.d). Do not do any mutation here -
+// we also load networks in a few other code paths.
 func loadUserNets(localConfig string, netsLoadList common.NetList) ([]activeNet, error) {
 	if netsLoadList.None() {
 		stderr.Printf("networking namespace with loopback only")
@@ -284,7 +371,9 @@ func loadUserNets(localConfig string, netsLoadList common.NetList) ([]activeNet,
 	}
 
 	userNetPath := filepath.Join(localConfig, UserNetPathSuffix)
-	stderr.Printf("loading networks from %v", userNetPath)
+	if debuglog {
+		stderr.Printf("loading networks from %v", userNetPath)
+	}
 
 	files, err := listFiles(userNetPath)
 	if err != nil {
@@ -319,8 +408,6 @@ func loadUserNets(localConfig string, netsLoadList common.NetList) ([]activeNet,
 			continue
 		}
 
-		n.runtime.Args = netsLoadList.SpecificArgs(n.conf.Name)
-
 		nets = append(nets, *n)
 	}
 
