@@ -30,10 +30,15 @@ import (
 // the container.  A list of options can be passed into this function to alter
 // the labels.  The labels returned will include a random MCS String, that is
 // guaranteed to be unique.
-func InitLabels(options []string) (string, string, error) {
+func InitLabels(mcsdir string, options []string) (string, string, error) {
 	if !selinux.SelinuxEnabled() {
 		return "", "", nil
 	}
+
+	if err := selinux.SetMCSDir(mcsdir); err != nil {
+		return "", "", errwrap.Wrap(errors.New("unable to configure MCS storage directory"), err)
+	}
+
 	processLabel, mountLabel, err := selinux.GetLxcContexts()
 	if err != nil {
 		return "", "", err
@@ -53,12 +58,6 @@ func InitLabels(options []string) (string, string, error) {
 			if con[0] == "level" || con[0] == "user" {
 				mcon[con[0]] = con[1]
 			}
-			if con[0] == "mcsdir" {
-				err := selinux.SetMCSDir(con[1])
-				if err != nil {
-					return "", "", errwrap.Wrap(errors.New("unable to configure MCS storage directory"), err)
-				}
-			}
 		}
 		processLabel = pcon.Get()
 		mountLabel = mcon.Get()
@@ -66,11 +65,6 @@ func InitLabels(options []string) (string, string, error) {
 	return processLabel, mountLabel, nil
 }
 
-// DEPRECATED: The GenLabels function is only to be used during the transition to the official API.
-func GenLabels(options string) (string, string, error) {
-	return InitLabels(strings.Fields(options))
-}
-
 // FormatMountLabel returns a string to be used by the mount command.
 // The format of this string will be used to alter the labeling of the mountpoint.
 // The string returned is suitable to be used as the options field of the mount command.
