@@ -23,34 +23,39 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"strings"
 	"syscall"
 
 	"github.com/appc/spec/pkg/device"
 	"github.com/coreos/rkt/pkg/fileutil"
 	"github.com/coreos/rkt/pkg/user"
-	"github.com/hashicorp/errwrap"
 )
 
 const DEFAULT_DIR_MODE os.FileMode = 0755
 
 var ErrNotSupportedPlatform = errors.New("platform and architecture is not supported")
 
+var xattrPrefixWhitelist = []string{
+	"security.capability",
+	"user",
+}
+
 // Map of paths that should be whitelisted. The paths should be relative to the
 // root of the tar file and should be cleaned (for example using filepath.Clean)
 type PathWhitelistMap map[string]struct{}
 
-type FilePermissionsEditor func(string, int, int, byte, os.FileInfo) error
+type FilePermissionsEditor func(path string, uid, gid int, typ byte, _ os.FileInfo, xattr map[string]string) error
 
 func NewUidShiftingFilePermEditor(uidRange *user.UidRange) (FilePermissionsEditor, error) {
 	if os.Geteuid() != 0 {
-		return func(_ string, _, _ int, _ byte, _ os.FileInfo) error {
+		return func(_ string, _, _ int, _ byte, _ os.FileInfo, _ map[string]string) error {
 			// The files are owned by the current user on creation.
 			// If we do nothing, they will remain so.
 			return nil
 		}, nil
 	}
 
-	return func(path string, uid, gid int, typ byte, fi os.FileInfo) error {
+	return func(path string, uid, gid int, typ byte, fi os.FileInfo, xattr map[string]string) error {
 		shiftedUid, shiftedGid, err := uidRange.ShiftRange(uint32(uid), uint32(gid))
 		if err != nil {
 			return err
@@ -67,6 +72,20 @@ func NewUidShiftingFilePermEditor(uidRange *user.UidRange) (FilePermissionsEdito
 				return err
 			}
 		}
+
+		if typ == tar.TypeReg || typ == tar.TypeRegA {
+			for k, v := range xattr {
+				if !isXattrAllowed(k) {
+					continue
+				}
+
+				err := syscall.Setxattr(path, k, []byte(v), 0)
+				if err != nil {
+					return err
+				}
+			}
+		}
+
 		return nil
 	}, nil
 }
@@ -94,13 +113,13 @@ Tar:
 			}
 			err = extractFile(tr, target, hdr, overwrite, editor)
 			if err != nil {
-				return errwrap.Wrap(errors.New("error extracting tarball"), err)
+				return fmt.Errorf("could not extract file %q in %q: %v", hdr.Name, target, err)
 			}
 			if hdr.Typeflag == tar.TypeDir {
 				dirhdrs = append(dirhdrs, hdr)
 			}
 		default:
-			return errwrap.Wrap(errors.New("error extracting tarball"), err)
+			return err
 		}
 	}
 
@@ -109,7 +128,7 @@ Tar:
 	for _, hdr := range dirhdrs {
 		p := filepath.Join(target, hdr.Name)
 		if err := syscall.UtimesNano(p, HdrToTimespec(hdr)); err != nil {
-			return err
+			return fmt.Errorf("UtimesNano failed on %q: %v", p, err)
 		}
 	}
 	return nil
@@ -194,13 +213,15 @@ func extractFile(tr *tar.Reader, target string, hdr *tar.Header, overwrite bool,
 		if err := syscall.Mkfifo(p, uint32(fi.Mode())); err != nil {
 			return err
 		}
+	case typ == tar.TypeXGlobalHeader:
+		return nil
 	// TODO(jonboulle): implement other modes
 	default:
 		return fmt.Errorf("unsupported type: %v", typ)
 	}
 
 	if editor != nil {
-		if err := editor(p, hdr.Uid, hdr.Gid, hdr.Typeflag, fi); err != nil {
+		if err := editor(p, hdr.Uid, hdr.Gid, hdr.Typeflag, fi, hdr.Xattrs); err != nil {
 			return err
 		}
 	}
@@ -242,11 +263,11 @@ func extractFileFromTar(tr *tar.Reader, file string) ([]byte, error) {
 			}
 			buf, err := ioutil.ReadAll(tr)
 			if err != nil {
-				return nil, errwrap.Wrap(errors.New("error extracting tarball"), err)
+				return nil, err
 			}
 			return buf, nil
 		default:
-			return nil, errwrap.Wrap(errors.New("error extracting tarball"), err)
+			return nil, err
 		}
 	}
 }
@@ -254,3 +275,13 @@ func extractFileFromTar(tr *tar.Reader, file string) ([]byte, error) {
 func HdrToTimespec(hdr *tar.Header) []syscall.Timespec {
 	return []syscall.Timespec{fileutil.TimeToTimespec(hdr.AccessTime), fileutil.TimeToTimespec(hdr.ModTime)}
 }
+
+func isXattrAllowed(xattr string) bool {
+	for _, v := range xattrPrefixWhitelist {
+		if strings.HasPrefix(xattr, v) {
+			return true
+		}
+	}
+
+	return false
+}
