@@ -17,6 +17,7 @@ package main
 import (
 	"encoding/json"
 
+	pkgPod "github.com/coreos/rkt/pkg/pod"
 	"github.com/spf13/cobra"
 )
 
@@ -38,19 +39,19 @@ func init() {
 func runCatManifest(cmd *cobra.Command, args []string) (exit int) {
 	if len(args) != 1 {
 		cmd.Usage()
-		return 1
+		return 254
 	}
 
-	pod, err := getPodFromUUIDString(args[0])
+	pod, err := pkgPod.PodFromUUIDString(getDataDir(), args[0])
 	if err != nil {
 		stderr.PrintE("problem retrieving pod", err)
-		return 1
+		return 254
 	}
 	defer pod.Close()
 
-	manifest, err := pod.getManifest()
+	_, manifest, err := pod.PodManifest()
 	if err != nil {
-		return 1
+		return 254
 	}
 
 	var b []byte
@@ -61,7 +62,7 @@ func runCatManifest(cmd *cobra.Command, args []string) (exit int) {
 	}
 	if err != nil {
 		stderr.PrintE("cannot read the pod manifest", err)
-		return 1
+		return 254
 	}
 
 	stdout.Print(string(b))
