@@ -16,29 +16,40 @@ package image
 
 import (
 	"fmt"
+	"io/ioutil"
 	"net/url"
 	"os"
 	"path/filepath"
-	"reflect"
 	"strings"
 	"time"
 
-	"github.com/coreos/rkt/common/apps"
+	dist "github.com/coreos/rkt/pkg/distribution"
 	"github.com/coreos/rkt/pkg/keystore"
 	rktlog "github.com/coreos/rkt/pkg/log"
 	"github.com/coreos/rkt/rkt/config"
 	rktflag "github.com/coreos/rkt/rkt/flag"
-	"github.com/coreos/rkt/store"
+	"github.com/coreos/rkt/store/imagestore"
+	"github.com/coreos/rkt/store/treestore"
+	"github.com/hashicorp/errwrap"
 
+	"github.com/appc/spec/discovery"
 	"github.com/appc/spec/schema"
-	"github.com/appc/spec/schema/types"
 	"golang.org/x/crypto/openpgp"
 )
 
+type imageStringType int
+
+const (
+	imageStringName imageStringType = iota // image type to be guessed
+	imageStringPath                        // absolute or relative path
+)
+
 // action is a common type for Finder and Fetcher
 type action struct {
-	// S is a store where images will be looked for or stored.
-	S *store.Store
+	// S is an aci store where images will be looked for or stored.
+	S *imagestore.Store
+	// Ts is an aci tree store.
+	Ts *treestore.Store
 	// Ks is a keystore used for verification of the image
 	Ks *keystore.Keystore
 	// Headers is a map of headers which might be used for
@@ -62,10 +73,10 @@ type action struct {
 	// local filesystem or a remote location.
 	StoreOnly bool
 	// NoStore tells whether to avoid getting images from the
-	// store. Note that the store can be still used as a cache.
+	// store. Note that transport caching (like http Etags) can be still
+	// used to avoid refetching.
 	NoStore bool
-	// NoCache tells to avoid getting images from the store
-	// completely.
+	// NoCache tells to ignore transport caching.
 	NoCache bool
 	// WithDeps tells whether image dependencies should be
 	// downloaded too.
@@ -74,43 +85,17 @@ type action struct {
 
 var (
 	log    *rktlog.Logger
-	stdout *rktlog.Logger = rktlog.New(os.Stdout, "", false)
+	diag   *rktlog.Logger
+	stdout *rktlog.Logger
 )
 
 func ensureLogger(debug bool) {
-	if log == nil {
-		log = rktlog.New(os.Stderr, "image", debug)
+	if log == nil || diag == nil || stdout == nil {
+		log, diag, stdout = rktlog.NewLogSet("image", debug)
 	}
-}
-
-// isReallyNil makes sure that the passed value is really really
-// nil. So it returns true if value is plain nil or if it is e.g. an
-// interface with non-nil type but nil-value (which normally is
-// different from nil itself).
-func isReallyNil(iface interface{}) bool {
-	// this catches the cases when you pass non-interface nil
-	// directly, like:
-	//
-	// isReallyNil(nil)
-	// var m map[string]string
-	// isReallyNil(m)
-	if iface == nil {
-		return true
+	if !debug {
+		diag.SetOutput(ioutil.Discard)
 	}
-	// get a reflect value
-	v := reflect.ValueOf(iface)
-	// only channels, functions, interfaces, maps, pointers and
-	// slices are nillable
-	switch v.Kind() {
-	case reflect.Chan, reflect.Func, reflect.Interface, reflect.Map, reflect.Ptr, reflect.Slice:
-		// this catches the cases when you pass some interface
-		// with nil value, like:
-		//
-		// var v io.Closer = func(){var f *os.File; return f}()
-		// isReallyNil(v)
-		return v.IsNil()
-	}
-	return false
 }
 
 // useCached checks if downloadTime plus maxAge is before/after the current time.
@@ -146,15 +131,80 @@ func printIdentities(entity *openpgp.Entity) {
 	log.Print(strings.Join(lines, "\n"))
 }
 
-func guessImageType(image string) apps.AppImageType {
-	if _, err := types.NewHash(image); err == nil {
-		return apps.AppImageHash
+// DistFromImageString return the distribution for the given input image string
+func DistFromImageString(is string) (dist.Distribution, error) {
+	u, err := url.Parse(is)
+	if err != nil {
+		return nil, errwrap.Wrap(fmt.Errorf("failed to parse image url %q", is), err)
 	}
-	if u, err := url.Parse(image); err == nil && u.Scheme != "" {
-		return apps.AppImageURL
+
+	// Convert user friendly image string names to internal distribution URIs
+	// file:///full/path/to/aci/file.aci -> archive:aci:file%3A%2F%2F%2Ffull%2Fpath%2Fto%2Faci%2Ffile.aci
+	switch u.Scheme {
+	case "":
+		// no scheme given, hence it is an appc image name or path
+		appImageType := guessAppcOrPath(is, []string{schema.ACIExtension})
+
+		switch appImageType {
+		case imageStringName:
+			app, err := discovery.NewAppFromString(is)
+			if err != nil {
+				return nil, fmt.Errorf("invalid appc image string %q: %v", is, err)
+			}
+			return dist.NewAppcFromApp(app), nil
+		case imageStringPath:
+			absPath, err := filepath.Abs(is)
+			if err != nil {
+				return nil, errwrap.Wrap(fmt.Errorf("failed to get an absolute path for %q", is), err)
+			}
+			is = "file://" + absPath
+
+			// given a file:// image string, call this function again to return an ACI distribution
+			return DistFromImageString(is)
+		default:
+			return nil, fmt.Errorf("invalid image string type %q", appImageType)
+		}
+	case "file", "http", "https":
+		// An ACI archive with any transport type (file, http, s3 etc...) and final aci extension
+		if filepath.Ext(u.Path) == schema.ACIExtension {
+			dist, err := dist.NewACIArchiveFromTransportURL(u)
+			if err != nil {
+				return nil, fmt.Errorf("archive distribution creation error: %v", err)
+			}
+			return dist, nil
+		}
+	case "docker":
+		// Accept both docker: and docker:// uri
+		dockerStr := is
+		if strings.HasPrefix(dockerStr, "docker://") {
+			dockerStr = strings.TrimPrefix(dockerStr, "docker://")
+		} else if strings.HasPrefix(dockerStr, "docker:") {
+			dockerStr = strings.TrimPrefix(dockerStr, "docker:")
+		}
+
+		dist, err := dist.NewDockerFromString(dockerStr)
+		if err != nil {
+			return nil, fmt.Errorf("docker distribution creation error: %v", err)
+		}
+		return dist, nil
+	case dist.Scheme: // cimd
+		return dist.Parse(is)
+	default:
+		// any other scheme is a an appc image name, i.e. "my-app:v1.0"
+		app, err := discovery.NewAppFromString(is)
+		if err != nil {
+			return nil, fmt.Errorf("invalid appc image string %q: %v", is, err)
+		}
+
+		return dist.NewAppcFromApp(app), nil
 	}
-	if filepath.IsAbs(image) {
-		return apps.AppImagePath
+
+	return nil, fmt.Errorf("invalid image string %q", is)
+}
+
+func guessAppcOrPath(is string, extensions []string) imageStringType {
+	if filepath.IsAbs(is) {
+		return imageStringPath
 	}
 
 	// Well, at this point is basically heuristics time. The image
@@ -162,9 +212,9 @@ func guessImageType(image string) apps.AppImageType {
 
 	// First, let's try to stat whatever file the URL would specify. If it
 	// exists, that's probably what the user wanted.
-	f, err := os.Stat(image)
+	f, err := os.Stat(is)
 	if err == nil && f.Mode().IsRegular() {
-		return apps.AppImagePath
+		return imageStringPath
 	}
 
 	// Second, let's check if there is a colon in the image
@@ -173,21 +223,23 @@ func guessImageType(image string) apps.AppImageType {
 	// highly unlikely that the image parameter is a path. Colon
 	// in this context is often used for specifying a version of
 	// an image, like in "example.com/reduce-worker:1.0.0".
-	if strings.ContainsRune(image, ':') {
-		return apps.AppImageName
+	if strings.ContainsRune(is, ':') {
+		return imageStringName
 	}
 
 	// Third, let's check if there is a dot followed by a slash
 	// (./) - if so, it is likely that the image parameter is path
 	// like ./aci-in-this-dir or ../aci-in-parent-dir
-	if strings.Contains(image, "./") {
-		return apps.AppImagePath
+	if strings.Contains(is, "./") {
+		return imageStringPath
 	}
 
 	// Fourth, let's check if the image parameter has an .aci
 	// extension. If so, likely a path like "stage1-coreos.aci".
-	if filepath.Ext(image) == schema.ACIExtension {
-		return apps.AppImagePath
+	for _, e := range extensions {
+		if filepath.Ext(is) == e {
+			return imageStringPath
+		}
 	}
 
 	// At this point, if the image parameter is something like
@@ -197,5 +249,36 @@ func guessImageType(image string) apps.AppImageType {
 	// "stage1-coreos" in this directory tree, then you better be
 	// off prepending the parameter with "./", because I'm gonna
 	// treat this as an image name otherwise.
-	return apps.AppImageName
+	return imageStringName
+}
+
+func eTag(rem *imagestore.Remote) string {
+	if rem != nil {
+		return rem.ETag
+	}
+	return ""
+}
+
+func maybeUseCached(rem *imagestore.Remote, cd *cacheData) string {
+	if rem == nil || cd == nil {
+		return ""
+	}
+	if cd.UseCached {
+		return rem.BlobKey
+	}
+	return ""
+}
+
+func remoteForURL(s *imagestore.Store, u *url.URL) (*imagestore.Remote, error) {
+	urlStr := u.String()
+	rem, err := s.GetRemote(urlStr)
+	if err != nil {
+		if err == imagestore.ErrRemoteNotFound {
+			return nil, nil
+		}
+
+		return nil, errwrap.Wrap(fmt.Errorf("failed to fetch remote for URL %q", urlStr), err)
+	}
+
+	return rem, nil
 }
