@@ -17,7 +17,7 @@ package main
 import (
 	"encoding/json"
 
-	"github.com/coreos/rkt/store"
+	"github.com/coreos/rkt/store/imagestore"
 
 	"github.com/spf13/cobra"
 )
@@ -26,7 +26,7 @@ var (
 	cmdImageCatManifest = &cobra.Command{
 		Use:   "cat-manifest IMAGE",
 		Short: "Inspect and print the image manifest",
-		Long:  `IMAGE should be a string referencing an image; either a hash or an image name.`,
+		Long:  `IMAGE should be a string referencing an image; either a ID or an image name.`,
 		Run:   runWrapper(runImageCatManifest),
 	}
 	flagPrettyPrint bool
@@ -40,25 +40,25 @@ func init() {
 func runImageCatManifest(cmd *cobra.Command, args []string) (exit int) {
 	if len(args) != 1 {
 		cmd.Usage()
-		return 1
+		return 254
 	}
 
-	s, err := store.NewStore(getDataDir())
+	s, err := imagestore.NewStore(storeDir())
 	if err != nil {
 		stderr.PrintE("cannot open store", err)
-		return 1
+		return 254
 	}
 
 	key, err := getStoreKeyFromAppOrHash(s, args[0])
 	if err != nil {
 		stderr.Error(err)
-		return 1
+		return 254
 	}
 
 	manifest, err := s.GetImageManifest(key)
 	if err != nil {
 		stderr.PrintE("cannot get image manifest", err)
-		return 1
+		return 254
 	}
 
 	var b []byte
@@ -69,7 +69,7 @@ func runImageCatManifest(cmd *cobra.Command, args []string) (exit int) {
 	}
 	if err != nil {
 		stderr.PrintE("cannot read the image manifest", err)
-		return 1
+		return 254
 	}
 
 	stdout.Print(string(b))
