@@ -22,7 +22,8 @@ import (
 
 	"github.com/coreos/rkt/pkg/fileutil"
 	"github.com/coreos/rkt/pkg/user"
-	"github.com/coreos/rkt/store"
+	"github.com/coreos/rkt/store/imagestore"
+	"github.com/coreos/rkt/store/treestore"
 
 	"github.com/spf13/cobra"
 )
@@ -31,7 +32,7 @@ var (
 	cmdImageRender = &cobra.Command{
 		Use:   "render IMAGE OUTPUT_DIR",
 		Short: "Render a stored image to a directory with all its dependencies",
-		Long: `IMAGE should be a string referencing an image: either a hash or an image name.
+		Long: `IMAGE should be a string referencing an image: either a ID or an image name.
 
 This differs from extract in that the rendered image is in the state the app
 would see when running in rkt, dependencies and all.
@@ -53,83 +54,89 @@ func init() {
 func runImageRender(cmd *cobra.Command, args []string) (exit int) {
 	if len(args) != 2 {
 		cmd.Usage()
-		return 1
+		return 254
 	}
 	outputDir := args[1]
 
-	s, err := store.NewStore(getDataDir())
+	s, err := imagestore.NewStore(storeDir())
 	if err != nil {
 		stderr.PrintE("cannot open store", err)
-		return 1
+		return 254
+	}
+
+	ts, err := treestore.NewStore(treeStoreDir(), s)
+	if err != nil {
+		stderr.PrintE("cannot open store", err)
+		return
 	}
 
 	key, err := getStoreKeyFromAppOrHash(s, args[0])
 	if err != nil {
 		stderr.Error(err)
-		return 1
+		return 254
 	}
 
-	id, _, err := s.RenderTreeStore(key, false)
+	id, _, err := ts.Render(key, false)
 	if err != nil {
 		stderr.PrintE("error rendering ACI", err)
-		return 1
+		return 254
 	}
-	if _, err := s.CheckTreeStore(id); err != nil {
+	if _, err := ts.Check(id); err != nil {
 		stderr.Print("warning: tree cache is in a bad state. Rebuilding...")
 		var err error
-		if id, _, err = s.RenderTreeStore(key, true); err != nil {
+		if id, _, err = ts.Render(key, true); err != nil {
 			stderr.PrintE("error rendering ACI", err)
-			return 1
+			return 254
 		}
 	}
 
 	if _, err := os.Stat(outputDir); err == nil {
 		if !flagRenderOverwrite {
 			stderr.Print("output directory exists (try --overwrite)")
-			return 1
+			return 254
 		}
 
 		// don't allow the user to delete the root filesystem by mistake
 		if outputDir == "/" {
 			stderr.Print("this would delete your root filesystem. Refusing.")
-			return 1
+			return 254
 		}
 
 		if err := os.RemoveAll(outputDir); err != nil {
 			stderr.PrintE("error removing existing output dir", err)
-			return 1
+			return 254
 		}
 	}
 	rootfsOutDir := outputDir
 	if !flagRenderRootfsOnly {
 		if err := os.MkdirAll(outputDir, 0755); err != nil {
 			stderr.PrintE("error creating output directory", err)
-			return 1
+			return 254
 		}
 		rootfsOutDir = filepath.Join(rootfsOutDir, "rootfs")
 
 		manifest, err := s.GetImageManifest(key)
 		if err != nil {
 			stderr.PrintE("error getting manifest", err)
-			return 1
+			return 254
 		}
 
 		mb, err := json.Marshal(manifest)
 		if err != nil {
 			stderr.PrintE("error marshalling image manifest", err)
-			return 1
+			return 254
 		}
 
 		if err := ioutil.WriteFile(filepath.Join(outputDir, "manifest"), mb, 0700); err != nil {
 			stderr.PrintE("error writing image manifest", err)
-			return 1
+			return 254
 		}
 	}
 
-	cachedTreePath := s.GetTreeStoreRootFS(id)
+	cachedTreePath := ts.GetRootFS(id)
 	if err := fileutil.CopyTree(cachedTreePath, rootfsOutDir, user.NewBlankUidRange()); err != nil {
 		stderr.PrintE("error copying ACI rootfs", err)
-		return 1
+		return 254
 	}
 
 	return 0
