@@ -22,10 +22,10 @@ import (
 	"strings"
 
 	"github.com/appc/spec/schema/types"
-	"github.com/coreos/rkt/common/apps"
 	"github.com/coreos/rkt/rkt/config"
 	"github.com/coreos/rkt/rkt/image"
-	"github.com/coreos/rkt/store"
+	"github.com/coreos/rkt/store/imagestore"
+	"github.com/coreos/rkt/store/treestore"
 	"github.com/hashicorp/errwrap"
 	"github.com/spf13/pflag"
 )
@@ -172,8 +172,25 @@ func addStage1ImageFlags(flags *pflag.FlagSet) {
 // If the user passed --stage1-url, --stage1-path, --stage1-name,
 // --stage1-hash, or --stage1-from-dir then we take what was passed
 // and try to load it. If it failed, we bail out. No second chances
-// and whatnot. The details about how each location type should be
-// handled are below.
+// and whatnot. The details about how each flag type should be handled
+// are below.
+//
+// For --stage1-url, we do discovery, and try to fetch it directly into
+// the store.
+//
+// For --stage1-path, we do no discovery and try to fetch the image
+// directly from the given file path into the store. If the file is not
+// found, then we try to fetch the file in the same directory as the
+// rkt binary itself into the store.
+//
+// For --stage1-from-dir, we do no discovery and try to fetch the image
+// from the stage1 images directory by the name.
+//
+// For --stage1-name, we do discovery and fetch the discovered image
+// into the store.
+//
+// For --stage1-hash, we do no discovery and try to fetch the image
+// from the store by the hash.
 //
 // If the user passed none of the above flags, we try to get the name,
 // the version and the location from the configuration. The name and
@@ -195,20 +212,17 @@ func addStage1ImageFlags(flags *pflag.FlagSet) {
 // have a second chance, we try to fetch the file in the same
 // directory as the rkt binary itself into the store.
 //
-// If location is a name, we do the discovery, fetch the discovered
-// image into the store.
-//
 // If location is an image hash then we make sure that it exists in
 // the store.
-func getStage1Hash(s *store.Store, c *config.Config) (*types.Hash, error) {
+func getStage1Hash(s *imagestore.Store, ts *treestore.Store, c *config.Config) (*types.Hash, error) {
 	imgDir := getStage1ImagesDirectory(c)
 	if overriddenStage1Location.kind != stage1ImageLocationUnset {
 		// we passed a --stage-{url,path,name,hash,from-dir} flag
-		return getStage1HashFromFlag(s, overriddenStage1Location, imgDir)
+		return getStage1HashFromFlag(s, ts, overriddenStage1Location, imgDir)
 	}
 
 	imgRef, imgLoc, imgFileName := getStage1DataFromConfig(c)
-	return getConfiguredStage1Hash(s, imgRef, imgLoc, imgFileName)
+	return getConfiguredStage1Hash(s, ts, imgRef, imgLoc, imgFileName)
 }
 
 func getStage1ImagesDirectory(c *config.Config) string {
@@ -218,7 +232,7 @@ func getStage1ImagesDirectory(c *config.Config) string {
 	return buildDefaultStage1ImagesDir
 }
 
-func getStage1HashFromFlag(s *store.Store, loc stage1ImageLocation, dir string) (*types.Hash, error) {
+func getStage1HashFromFlag(s *imagestore.Store, ts *treestore.Store, loc stage1ImageLocation, dir string) (*types.Hash, error) {
 	withKeystore := true
 	location := loc.location
 	if loc.kind == stage1ImageLocationFromDir {
@@ -229,31 +243,15 @@ func getStage1HashFromFlag(s *store.Store, loc stage1ImageLocation, dir string)
 		return nil, err
 	}
 
-	imgType := apps.AppImageGuess
 	switch loc.kind {
-	case stage1ImageLocationURL:
-		imgType = apps.AppImageURL
+	case stage1ImageLocationURL, stage1ImageLocationPath, stage1ImageLocationFromDir:
 		if trustedLocation {
 			withKeystore = false
 		}
-	case stage1ImageLocationPath:
-		imgType = apps.AppImagePath
-		if trustedLocation {
-			withKeystore = false
-		}
-	case stage1ImageLocationName:
-		imgType = apps.AppImageName
-	case stage1ImageLocationHash:
-		imgType = apps.AppImageHash
-	case stage1ImageLocationFromDir:
-		if trustedLocation {
-			withKeystore = false
-		}
-		imgType = apps.AppImagePath
 	}
 
-	fn := getStage1Finder(s, withKeystore)
-	return fn.FindImage(location, "", imgType)
+	fn := getStage1Finder(s, ts, withKeystore)
+	return fn.FindImage(location, "")
 }
 
 func getStage1DataFromConfig(c *config.Config) (string, string, string) {
@@ -297,15 +295,15 @@ func isTrustedLocation(location string) (bool, error) {
 	return false, nil
 }
 
-func getConfiguredStage1Hash(s *store.Store, imgRef, imgLoc, imgFileName string) (*types.Hash, error) {
+func getConfiguredStage1Hash(s *imagestore.Store, ts *treestore.Store, imgRef, imgLoc, imgFileName string) (*types.Hash, error) {
 	trusted, err := isTrustedLocation(imgLoc)
 	if err != nil {
 		return nil, err
 	}
-	fn := getStage1Finder(s, !trusted)
+	fn := getStage1Finder(s, ts, !trusted)
 	if !strings.HasSuffix(imgRef, "-dirty") {
 		fn.StoreOnly = true
-		if hash, err := fn.FindImage(imgRef, "", apps.AppImageName); err == nil {
+		if hash, err := fn.FindImage(imgRef, ""); err == nil {
 			return hash, nil
 		}
 		fn.StoreOnly = false
@@ -316,14 +314,16 @@ func getConfiguredStage1Hash(s *store.Store, imgRef, imgLoc, imgFileName string)
 	// If imgLoc is not an absolute path, then it is a URL
 	imgLocIsURL := imgLoc != "" && !filepath.IsAbs(imgLoc)
 	if imgLocIsURL {
-		return fn.FindImage(imgLoc, "", apps.AppImageURL)
+		return fn.FindImage(imgLoc, "")
 	}
 	return getStage1HashFromPath(fn, imgLoc, imgFileName)
 }
 
-func getStage1Finder(s *store.Store, withKeystore bool) *image.Finder {
+func getStage1Finder(s *imagestore.Store, ts *treestore.Store, withKeystore bool) *image.Finder {
 	fn := &image.Finder{
 		S:                  s,
+		Ts:                 ts,
+		Debug:              globalFlags.Debug,
 		InsecureFlags:      globalFlags.InsecureFlags,
 		TrustKeysFromHTTPS: globalFlags.TrustKeysFromHTTPS,
 
@@ -342,7 +342,7 @@ func getStage1HashFromPath(fn *image.Finder, imgLoc, imgFileName string) (*types
 	var fetchErr error
 	var fallbackErr error
 	if imgLoc != "" {
-		hash, err := fn.FindImage(imgLoc, "", apps.AppImagePath)
+		hash, err := fn.FindImage(imgLoc, "")
 		if err == nil {
 			return hash, nil
 		}
@@ -357,7 +357,7 @@ func getStage1HashFromPath(fn *image.Finder, imgLoc, imgFileName string) (*types
 			fn.Ks = nil
 			rktDir := filepath.Dir(exePath)
 			imgPath := filepath.Join(rktDir, imgFileName)
-			hash, err := fn.FindImage(imgPath, "", apps.AppImagePath)
+			hash, err := fn.FindImage(imgPath, "")
 			if err == nil {
 				return hash, nil
 			}
