@@ -15,11 +15,15 @@ if ! [[ $(protoc --version) =~ "3.0.0" ]]; then
 	exit 255
 fi
 
-export GOPATH=$(godep path)
 export PATH=.:${PATH}
+cd $GOPATH/src/github.com/coreos/rkt
 
 echo "building protoc-gen-go"
-go build github.com/golang/protobuf/protoc-gen-go
+pushd vendor/github.com/golang/protobuf/protoc-gen-go
+go build
+mv protoc-gen-go $(dirs -l +1)
+popd
+
 trap 'rm -f "protoc-gen-go"' EXIT
 
 echo "generating code"
