@@ -28,7 +28,6 @@ import (
 	"errors"
 	"fmt"
 	"io/ioutil"
-	"net"
 	"os"
 	"path"
 	"path/filepath"
@@ -40,72 +39,89 @@ import (
 
 	"github.com/appc/spec/schema"
 	"github.com/appc/spec/schema/types"
+	cnitypes "github.com/containernetworking/cni/pkg/types"
 	"github.com/coreos/rkt/common"
 	"github.com/coreos/rkt/common/apps"
+	commonnet "github.com/coreos/rkt/common/networking"
+	"github.com/coreos/rkt/common/overlay"
 	"github.com/coreos/rkt/pkg/aci"
 	"github.com/coreos/rkt/pkg/fileutil"
 	"github.com/coreos/rkt/pkg/label"
 	"github.com/coreos/rkt/pkg/sys"
 	"github.com/coreos/rkt/pkg/tpm"
 	"github.com/coreos/rkt/pkg/user"
-	"github.com/coreos/rkt/store"
+	"github.com/coreos/rkt/store/imagestore"
+	"github.com/coreos/rkt/store/treestore"
 	"github.com/coreos/rkt/version"
 	"github.com/hashicorp/errwrap"
 )
 
-const (
-	// Default perm bits for the regular files
-	// within the stage1 directory. (e.g. image manifest,
-	// pod manifest, stage1ID, etc).
-	defaultRegularFilePerm = os.FileMode(0640)
-
-	// Default perm bits for the regular directories
-	// within the stage1 directory.
-	defaultRegularDirPerm = os.FileMode(0750)
-)
-
 var debugEnabled bool
 
-// configuration parameters required by Prepare
+// PrepareConfig defines the configuration parameters required by Prepare
 type PrepareConfig struct {
 	*CommonConfig
-	Apps               *apps.Apps          // apps to prepare
-	InheritEnv         bool                // inherit parent environment into apps
-	ExplicitEnv        []string            // always set these environment variables for all the apps
-	EnvFromFile        []string            // environment variables loaded from files, set for all the apps
-	Ports              []types.ExposedPort // list of ports that rkt will expose on the host
-	UseOverlay         bool                // prepare pod with overlay fs
-	SkipTreeStoreCheck bool                // skip checking the treestore before rendering
-	PodManifest        string              // use the pod manifest specified by the user, this will ignore flags such as '--volume', '--port', etc.
-	PrivateUsers       *user.UidRange      // User namespaces
+	Apps               *apps.Apps            // apps to prepare
+	InheritEnv         bool                  // inherit parent environment into apps
+	ExplicitEnv        []string              // always set these environment variables for all the apps
+	EnvFromFile        []string              // environment variables loaded from files, set for all the apps
+	Ports              []types.ExposedPort   // list of ports that rkt will expose on the host
+	UseOverlay         bool                  // prepare pod with overlay fs
+	SkipTreeStoreCheck bool                  // skip checking the treestore before rendering
+	PodManifest        string                // use the pod manifest specified by the user, this will ignore flags such as '--volume', '--port', etc.
+	PrivateUsers       *user.UidRange        // user namespaces
+	UserAnnotations    types.UserAnnotations // user annotations for the pod.
+	UserLabels         types.UserLabels      // user labels for the pod.
 }
 
-// configuration parameters needed by Run
+// RunConfig defines the configuration parameters needed by Run
 type RunConfig struct {
 	*CommonConfig
-	Net         common.NetList // pod should have its own network stack
-	LockFd      int            // lock file descriptor
-	Interactive bool           // whether the pod is interactive or not
-	MDSRegister bool           // whether to register with metadata service or not
-	Apps        schema.AppList // applications (prepare gets them via Apps)
-	LocalConfig string         // Path to local configuration
-	Hostname    string         // hostname of the pod
-	RktGid      int            // group id of the 'rkt' group, -1 if there's no rkt group.
-	DNS         []string       // DNS name servers to write in /etc/resolv.conf
-	DNSSearch   []string       // DNS search domains to write in /etc/resolv.conf
-	DNSOpt      []string       // DNS options to write in /etc/resolv.conf
+	Net                  common.NetList // pod should have its own network stack
+	LockFd               int            // lock file descriptor
+	Interactive          bool           // whether the pod is interactive or not
+	MDSRegister          bool           // whether to register with metadata service or not
+	Apps                 schema.AppList // applications (prepare gets them via Apps)
+	LocalConfig          string         // Path to local configuration
+	Hostname             string         // hostname of the pod
+	RktGid               int            // group id of the 'rkt' group, -1 ere's no rkt group.
+	DNSConfMode          DNSConfMode    // dns configuration file mode - for stAage1
+	DNSConfig            cnitypes.DNS   // the DNS configuration (nameservers, search, options)
+	InsecureCapabilities bool           // Do not restrict capabilities
+	InsecurePaths        bool           // Do not restrict access to files in sysfs or procfs
+	InsecureSeccomp      bool           // Do not add seccomp restrictions
+	UseOverlay           bool           // run pod with overlay fs
+	HostsEntries         HostsEntries   // The entries in /etc/hosts
 }
 
-// configuration shared by both Run and Prepare
+// CommonConfig defines the configuration shared by both Run and Prepare
 type CommonConfig struct {
-	Store        *store.Store // store containing all of the configured application images
-	Stage1Image  types.Hash   // stage1 image containing usable /init and /enter entrypoints
-	UUID         *types.UUID  // UUID of the pod
-	RootHash     string       // Hash of the root filesystem
-	ManifestData string       // The pod manifest data
+	DataDir      string            // The path to the data directory, e.g. /var/lib/rkt/pods
+	Store        *imagestore.Store // store containing all of the configured application images
+	TreeStore    *treestore.Store  // store containing all of the configured application images
+	Stage1Image  types.Hash        // stage1 image containing usable /init and /enter entrypoints
+	UUID         *types.UUID       // UUID of the pod
+	RootHash     string            // Hash of the root filesystem
+	ManifestData string            // The pod manifest data
 	Debug        bool
 	MountLabel   string // selinux label to use for fs
-	ProcessLabel string // selinux label to use for process
+	ProcessLabel string // selinux label to use
+	Mutable      bool   // whether this pod is mutable
+}
+
+// HostsEntries encapsulates the entries in an etc-hosts file: mapping from IP
+// to arbitrary list of hostnames
+type HostsEntries map[string][]string
+
+// DNSConfMode indicates what the stage1 should do with dns config files
+// The values and meanings are:
+// 'host': bind-mount from host
+// 'stage0': the stage0 has generated it
+// 'none' : do not generate it
+// 'default' : do whatever was the default
+type DNSConfMode struct {
+	Resolv string // /etc/rkt-resolv.conf
+	Hosts  string // /etc/rkt-hosts
 }
 
 func init() {
@@ -115,6 +131,7 @@ func init() {
 	runtime.LockOSThread()
 }
 
+// InitDebug enables debugging
 func InitDebug() {
 	debugEnabled = true
 	log.SetDebug(true)
@@ -192,12 +209,22 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 		if err != nil {
 			return errwrap.Wrap(errors.New("error getting the manifest"), err)
 		}
-		appName, err := imageNameToAppName(am.Name)
-		if err != nil {
-			return errwrap.Wrap(errors.New("error converting image name to app name"), err)
+
+		var appName *types.ACName
+		if app.Name != "" {
+			appName, err = types.NewACName(app.Name)
+			if err != nil {
+				return errwrap.Wrap(errors.New("invalid app name format"), err)
+			}
+		} else {
+			appName, err = imageNameToAppName(am.Name)
+			if err != nil {
+				return errwrap.Wrap(errors.New("error converting image name to app name"), err)
+			}
 		}
-		if err := prepareAppImage(cfg, *appName, img, dir, cfg.UseOverlay); err != nil {
-			return errwrap.Wrap(fmt.Errorf("error setting up image %s", img), err)
+
+		if _, err := prepareAppImage(cfg, *appName, img, dir, cfg.UseOverlay); err != nil {
+			return errwrap.Wrap(fmt.Errorf("error preparing image %s", img), err)
 		}
 		if pm.Apps.Get(*appName) != nil {
 			return fmt.Errorf("error: multiple apps with name %s", am.Name)
@@ -214,11 +241,11 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 				ID:     img,
 				Labels: am.Labels,
 			},
-			Annotations: am.Annotations,
-			Mounts:      MergeMounts(cfg.Apps.Mounts, app.Mounts),
+			Mounts:         MergeMounts(cfg.Apps.Mounts, app.Mounts),
+			ReadOnlyRootFS: app.ReadOnlyRootFS,
 		}
 
-		if execOverride := app.Exec; execOverride != "" {
+		if app.Exec != "" {
 			// Create a minimal App section if not present
 			if am.App == nil {
 				ra.App = &types.App{
@@ -226,53 +253,39 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 					Group: strconv.Itoa(os.Getgid()),
 				}
 			}
-			ra.App.Exec = []string{execOverride}
+			ra.App.Exec = []string{app.Exec}
 		}
 
-		if execAppends := app.Args; execAppends != nil {
-			ra.App.Exec = append(ra.App.Exec, execAppends...)
+		if app.Args != nil {
+			ra.App.Exec = append(ra.App.Exec, app.Args...)
 		}
 
-		if memoryOverride := app.MemoryLimit; memoryOverride != nil {
-			isolator := memoryOverride.AsIsolator()
-			ra.App.Isolators = append(ra.App.Isolators, isolator)
+		if app.WorkingDir != "" {
+			ra.App.WorkingDirectory = app.WorkingDir
 		}
 
-		if cpuOverride := app.CPULimit; cpuOverride != nil {
-			isolator := cpuOverride.AsIsolator()
-			ra.App.Isolators = append(ra.App.Isolators, isolator)
+		if err := prepareIsolators(app, ra.App); err != nil {
+			return err
 		}
 
-		if app.CapsRetain != nil && app.CapsRemove != nil {
-			return fmt.Errorf("error: cannot use both --cap-retain and --cap-remove on the same image")
+		if app.User != "" {
+			ra.App.User = app.User
 		}
 
-		// Delete existing caps isolators if the user wants to override
-		// them with either --cap-retain or --cap-remove
-		if app.CapsRetain != nil || app.CapsRemove != nil {
-			for i := len(ra.App.Isolators) - 1; i >= 0; i-- {
-				isolator := ra.App.Isolators[i]
-				if _, ok := isolator.Value().(types.LinuxCapabilitiesSet); ok {
-					ra.App.Isolators = append(ra.App.Isolators[:i],
-						ra.App.Isolators[i+1:]...)
-				}
-			}
+		if app.Group != "" {
+			ra.App.Group = app.Group
 		}
 
-		if capsRetain := app.CapsRetain; capsRetain != nil {
-			isolator := capsRetain.AsIsolator()
-			ra.App.Isolators = append(ra.App.Isolators, isolator)
-		} else if capsRemove := app.CapsRemove; capsRemove != nil {
-			isolator := capsRemove.AsIsolator()
-			ra.App.Isolators = append(ra.App.Isolators, isolator)
+		if app.SupplementaryGIDs != nil {
+			ra.App.SupplementaryGIDs = app.SupplementaryGIDs
 		}
 
-		if user := app.User; user != "" {
-			ra.App.User = user
+		if app.UserAnnotations != nil {
+			ra.App.UserAnnotations = app.UserAnnotations
 		}
 
-		if group := app.Group; group != "" {
-			ra.App.Group = group
+		if app.UserLabels != nil {
+			ra.App.UserLabels = app.UserLabels
 		}
 
 		// loading the environment from the lowest priority to highest
@@ -283,6 +296,15 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 
 		mergeEnvs(&ra.App.Environment, cfg.EnvFromFile, true)
 		mergeEnvs(&ra.App.Environment, cfg.ExplicitEnv, true)
+
+		if app.Environments != nil {
+			envs := make([]string, 0, len(app.Environments))
+			for name, value := range app.Environments {
+				envs = append(envs, fmt.Sprintf("%s=%s", name, value))
+			}
+			mergeEnvs(&ra.App.Environment, envs, true)
+		}
+
 		pm.Apps = append(pm.Apps, ra)
 		return nil
 	}); err != nil {
@@ -292,7 +314,20 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 	// TODO(jonboulle): check that app mountpoint expectations are
 	// satisfied here, rather than waiting for stage1
 	pm.Volumes = cfg.Apps.Volumes
+
+	// Check to see if ports have any errors
 	pm.Ports = cfg.Ports
+	if _, err := commonnet.ForwardedPorts(&pm); err != nil {
+		return nil, err
+	}
+
+	pm.Annotations = append(pm.Annotations, types.Annotation{
+		Name:  "coreos.com/rkt/stage1/mutable",
+		Value: strconv.FormatBool(cfg.Mutable),
+	})
+
+	pm.UserAnnotations = cfg.UserAnnotations
+	pm.UserLabels = cfg.UserLabels
 
 	pmb, err := json.Marshal(pm)
 	if err != nil {
@@ -301,6 +336,89 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 	return pmb, nil
 }
 
+// prepareIsolators merges the CLI app parameters with the manifest's app
+func prepareIsolators(setup *apps.App, app *types.App) error {
+	if memoryOverride := setup.MemoryLimit; memoryOverride != nil {
+		isolator := memoryOverride.AsIsolator()
+		app.Isolators = append(app.Isolators, isolator)
+	}
+
+	if cpuOverride := setup.CPULimit; cpuOverride != nil {
+		isolator := cpuOverride.AsIsolator()
+		app.Isolators = append(app.Isolators, isolator)
+	}
+
+	if cpuSharesOverride := setup.CPUShares; cpuSharesOverride != nil {
+		isolator := cpuSharesOverride.AsIsolator()
+		app.Isolators.ReplaceIsolatorsByName(isolator, []types.ACIdentifier{types.LinuxCPUSharesName})
+	}
+
+	if oomAdjOverride := setup.OOMScoreAdj; oomAdjOverride != nil {
+		app.Isolators.ReplaceIsolatorsByName(oomAdjOverride.AsIsolator(), []types.ACIdentifier{types.LinuxOOMScoreAdjName})
+	}
+
+	if setup.CapsRetain != nil && setup.CapsRemove != nil {
+		return fmt.Errorf("error: cannot use both --caps-retain and --caps-remove on the same image")
+	}
+
+	// Delete existing caps isolators if the user wants to override
+	// them with either --caps-retain or --caps-remove
+	if setup.CapsRetain != nil || setup.CapsRemove != nil {
+		for i := len(app.Isolators) - 1; i >= 0; i-- {
+			isolator := app.Isolators[i]
+			if _, ok := isolator.Value().(types.LinuxCapabilitiesSet); ok {
+				app.Isolators = append(app.Isolators[:i],
+					app.Isolators[i+1:]...)
+			}
+		}
+	}
+
+	if capsRetain := setup.CapsRetain; capsRetain != nil {
+		isolator, err := capsRetain.AsIsolator()
+		if err != nil {
+			return err
+		}
+		app.Isolators = append(app.Isolators, *isolator)
+	} else if capsRemove := setup.CapsRemove; capsRemove != nil {
+		isolator, err := capsRemove.AsIsolator()
+		if err != nil {
+			return err
+		}
+		app.Isolators = append(app.Isolators, *isolator)
+	}
+
+	// Override seccomp isolators via --seccomp CLI switch
+	if setup.SeccompFilter != "" {
+		var is *types.Isolator
+		mode, errno, set, err := setup.SeccompOverride()
+		if err != nil {
+			return err
+		}
+		switch mode {
+		case "retain":
+			lss, err := types.NewLinuxSeccompRetainSet(errno, set...)
+			if err != nil {
+				return err
+			}
+			if is, err = lss.AsIsolator(); err != nil {
+				return err
+			}
+		case "remove":
+			lss, err := types.NewLinuxSeccompRemoveSet(errno, set...)
+			if err != nil {
+				return err
+			}
+			if is, err = lss.AsIsolator(); err != nil {
+				return err
+			}
+		default:
+			return apps.ErrInvalidSeccompMode
+		}
+		app.Isolators.ReplaceIsolatorsByName(*is, []types.ACIdentifier{types.LinuxSeccompRemoveSetName, types.LinuxSeccompRetainSetName})
+	}
+	return nil
+}
+
 // validatePodManifest reads the user-specified pod manifest, prepares the app images
 // and validates the pod manifest. If the pod manifest passes validation, it returns
 // the manifest as []byte.
@@ -326,8 +444,8 @@ func validatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 		if err != nil {
 			return nil, errwrap.Wrap(errors.New("error getting the image manifest from store"), err)
 		}
-		if err := prepareAppImage(cfg, ra.Name, img.ID, dir, cfg.UseOverlay); err != nil {
-			return nil, errwrap.Wrap(fmt.Errorf("error setting up image %s", img), err)
+		if _, err := prepareAppImage(cfg, ra.Name, img.ID, dir, cfg.UseOverlay); err != nil {
+			return nil, errwrap.Wrap(fmt.Errorf("error preparing image %s", img), err)
 		}
 		if _, ok := appNames[ra.Name]; ok {
 			return nil, fmt.Errorf("multiple apps with same name %s", ra.Name)
@@ -337,16 +455,21 @@ func validatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 			return nil, fmt.Errorf("no app section in the pod manifest or the image manifest")
 		}
 	}
+
+	// Validate forwarded ports
+	if _, err := commonnet.ForwardedPorts(&pm); err != nil {
+		return nil, err
+	}
 	return pmb, nil
 }
 
 // Prepare sets up a pod based on the given config.
 func Prepare(cfg PrepareConfig, dir string, uuid *types.UUID) error {
-	if err := os.MkdirAll(common.AppsInfoPath(dir), defaultRegularDirPerm); err != nil {
+	if err := os.MkdirAll(common.AppsInfoPath(dir), common.DefaultRegularDirPerm); err != nil {
 		return errwrap.Wrap(errors.New("error creating apps info directory"), err)
 	}
 	debug("Preparing stage1")
-	if err := prepareStage1Image(cfg, cfg.Stage1Image, dir, cfg.UseOverlay); err != nil {
+	if err := prepareStage1Image(cfg, dir); err != nil {
 		return errwrap.Wrap(errors.New("error preparing stage1"), err)
 	}
 
@@ -363,12 +486,25 @@ func Prepare(cfg PrepareConfig, dir string, uuid *types.UUID) error {
 
 	cfg.CommonConfig.ManifestData = string(pmb)
 
+	// create pod lock file for app add/rm operations.
+	f, err := os.OpenFile(common.PodManifestLockPath(dir), os.O_CREATE|os.O_RDWR, 0600)
+	if err != nil {
+		return err
+	}
+	f.Close()
+
 	debug("Writing pod manifest")
 	fn := common.PodManifestPath(dir)
-	if err := ioutil.WriteFile(fn, pmb, defaultRegularFilePerm); err != nil {
+	if err := ioutil.WriteFile(fn, pmb, common.DefaultRegularFilePerm); err != nil {
 		return errwrap.Wrap(errors.New("error writing pod manifest"), err)
 	}
 
+	f, err = os.OpenFile(common.PodCreatedPath(dir), os.O_CREATE|os.O_RDWR, common.DefaultRegularFilePerm)
+	if err != nil {
+		return err
+	}
+	f.Close()
+
 	if cfg.UseOverlay {
 		// mark the pod as prepared with overlay
 		f, err := os.Create(filepath.Join(dir, common.OverlayPreparedFilename))
@@ -382,7 +518,7 @@ func Prepare(cfg PrepareConfig, dir string, uuid *types.UUID) error {
 		// mark the pod as prepared for user namespaces
 		uidrangeBytes := cfg.PrivateUsers.Serialize()
 
-		if err := ioutil.WriteFile(filepath.Join(dir, common.PrivateUsersPreparedFilename), uidrangeBytes, defaultRegularFilePerm); err != nil {
+		if err := ioutil.WriteFile(filepath.Join(dir, common.PrivateUsersPreparedFilename), uidrangeBytes, common.DefaultRegularFilePerm); err != nil {
 			return errwrap.Wrap(errors.New("error writing userns marker file"), err)
 		}
 	}
@@ -390,22 +526,6 @@ func Prepare(cfg PrepareConfig, dir string, uuid *types.UUID) error {
 	return nil
 }
 
-func preparedWithOverlay(dir string) (bool, error) {
-	_, err := os.Stat(filepath.Join(dir, common.OverlayPreparedFilename))
-	if os.IsNotExist(err) {
-		return false, nil
-	}
-	if err != nil {
-		return false, err
-	}
-
-	if !common.SupportsOverlay() {
-		return false, fmt.Errorf("the pod was prepared with overlay but overlay is not supported")
-	}
-
-	return true, nil
-}
-
 func preparedWithPrivateUsers(dir string) (string, error) {
 	bytes, err := ioutil.ReadFile(filepath.Join(dir, common.PrivateUsersPreparedFilename))
 	if os.IsNotExist(err) {
@@ -418,68 +538,98 @@ func preparedWithPrivateUsers(dir string) (string, error) {
 	return string(bytes), nil
 }
 
-func addResolvConf(cfg RunConfig, rootfs string) {
-	content := "# Generated by rkt\n\n"
-	if len(cfg.DNSSearch) > 0 {
-		content += fmt.Sprintf("search %s\n", strings.Join(cfg.DNSSearch, " "))
-	}
-	for _, server := range cfg.DNS {
-		// skip empty entries
-		if server == "" {
-			continue
-		}
-		// comment invalid entries
-		if net.ParseIP(server) == nil {
-			content += "# "
-		}
-		content += "nameserver " + server + "\n"
-	}
-	if len(cfg.DNSOpt) > 0 {
-		content += fmt.Sprintf("options %s\n", strings.Join(cfg.DNSOpt, " "))
-	}
-	content += "\n"
+func writeDnsConfig(cfg *RunConfig, rootfs string) {
+	writeResolvConf(cfg, rootfs)
+	writeEtcHosts(cfg, rootfs)
+}
 
-	if err := os.Mkdir(filepath.Join(rootfs, "etc"), defaultRegularDirPerm); err != nil {
+// writeResolvConf will generate <stage1>/etc/rkt-resolv.conf if needed
+func writeResolvConf(cfg *RunConfig, rootfs string) {
+	if cfg.DNSConfMode.Resolv != "stage0" {
+		return
+	}
+
+	if err := os.Mkdir(filepath.Join(rootfs, "etc"), common.DefaultRegularDirPerm); err != nil {
 		if !os.IsExist(err) {
 			log.Fatalf("error creating dir %q: %v\n", "/etc", err)
 		}
 	}
+	resolvPath := filepath.Join(rootfs, "etc/rkt-resolv.conf")
+	f, err := os.Create(resolvPath)
+	if err != nil {
+		log.Fatalf("error writing etc/rkt-resolv.conf: %v\n", err)
+	}
+	defer f.Close()
 
-	if err := ioutil.WriteFile(filepath.Join(rootfs, "etc/rkt-resolv.conf"), []byte(content), 0644); err != nil {
-		log.Fatalf("error writing /etc/rkt-resolv.conf: %v\n", err)
+	_, err = f.WriteString(common.MakeResolvConf(cfg.DNSConfig, "Generated by rkt run"))
+	if err != nil {
+		log.Fatalf("error writing etc/rkt-resolv.conf: %v\n", err)
+	}
+}
+
+// writeEtcHosts writes the file /etc/rkt-hosts into the stage1 rootfs.
+// This will read defaults from <rootfs>/etc/hosts-fallback if it exists.
+// Therefore, this should be called after the stage1 is mounted
+func writeEtcHosts(cfg *RunConfig, rootfs string) {
+	if cfg.DNSConfMode.Hosts != "stage0" {
+		return
+	}
+
+	// Read <stage1>/rootfs/etc/hosts-fallback to get some sane defaults
+	hostsTextb, err := ioutil.ReadFile(filepath.Join(rootfs, "etc/hosts-fallback"))
+	if err != nil {
+		// fallback-fallback :-)
+		hostsTextb = []byte("#created by rkt stage0\n127.0.0.1 localhost localhost.localdomain\n")
+	}
+	hostsText := string(hostsTextb)
+
+	hostsText += "\n\n# Added by rkt run --hosts-entry\n"
+
+	for ip, hostnames := range cfg.HostsEntries {
+		hostsText = fmt.Sprintf("%s%s %s\n", hostsText, ip, strings.Join(hostnames, " "))
+	}
+
+	// Create /etc if it does not exist
+	etcPath := filepath.Join(rootfs, "etc")
+	if _, err := os.Stat(etcPath); err != nil && os.IsNotExist(err) {
+		err = os.Mkdir(etcPath, 0755)
+		if err != nil {
+			log.FatalE("failed to make stage1 etc directory", err)
+		}
+	} else if err != nil {
+		log.FatalE("Failed to stat stage1 etc", err)
+	}
+
+	hostsPath := filepath.Join(etcPath, "rkt-hosts")
+	err = ioutil.WriteFile(hostsPath, []byte(hostsText), 0644)
+	if err != nil {
+		log.FatalE("failed to write etc/rkt-hosts", err)
 	}
 }
 
 // Run mounts the right overlay filesystems and actually runs the prepared
 // pod by exec()ing the stage1 init inside the pod filesystem.
 func Run(cfg RunConfig, dir string, dataDir string) {
-	useOverlay, err := preparedWithOverlay(dir)
-	if err != nil {
-		log.FatalE("error preparing overlay", err)
-	}
-
 	privateUsers, err := preparedWithPrivateUsers(dir)
 	if err != nil {
 		log.FatalE("error preparing private users", err)
 	}
 
 	debug("Setting up stage1")
-	if err := setupStage1Image(cfg, dir, useOverlay); err != nil {
+	if err := setupStage1Image(cfg, dir, cfg.UseOverlay); err != nil {
 		log.FatalE("error setting up stage1", err)
 	}
 	debug("Wrote filesystem to %s\n", dir)
 
 	for _, app := range cfg.Apps {
-		if err := setupAppImage(cfg, app.Name, app.Image.ID, dir, useOverlay); err != nil {
+		if err := setupAppImage(cfg, app.Name, app.Image.ID, dir, cfg.UseOverlay); err != nil {
 			log.FatalE("error setting up app image", err)
 		}
 	}
 
 	destRootfs := common.Stage1RootfsPath(dir)
 
-	if len(cfg.DNS) > 0 || len(cfg.DNSSearch) > 0 || len(cfg.DNSOpt) > 0 {
-		addResolvConf(cfg, destRootfs)
-	}
+	writeDnsConfig(&cfg, destRootfs)
 
 	if err := os.Setenv(common.EnvLockFd, fmt.Sprintf("%v", cfg.LockFd)); err != nil {
 		log.FatalE("setting lock fd environment", err)
@@ -490,7 +640,7 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 	}
 
 	if err := os.Setenv(common.EnvSELinuxMountContext, fmt.Sprintf("%v", cfg.MountLabel)); err != nil {
-		log.FatalE("setting SELinux mount context enviroment", err)
+		log.FatalE("setting SELinux mount context environment", err)
 	}
 
 	debug("Pivoting to filesystem %s", dir)
@@ -503,7 +653,6 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 		log.FatalE("error determining 'run' entrypoint", err)
 	}
 	args := []string{filepath.Join(destRootfs, ep)}
-	debug("Execing %s", ep)
 
 	if cfg.Debug {
 		args = append(args, "--debug")
@@ -543,6 +692,39 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 		}
 	}
 
+	if cfg.DNSConfMode.Hosts != "default" || cfg.DNSConfMode.Resolv != "default" {
+		if interfaceVersionSupportsDNSConfMode(s1v) {
+			args = append(args, fmt.Sprintf("--dns-conf-mode=resolv=%s,hosts=%s", cfg.DNSConfMode.Resolv, cfg.DNSConfMode.Hosts))
+		} else {
+			log.Printf("warning: --dns-conf-mode option not supported by stage1")
+		}
+	}
+
+	if interfaceVersionSupportsInsecureOptions(s1v) {
+		if cfg.InsecureCapabilities {
+			args = append(args, "--disable-capabilities-restriction")
+		}
+		if cfg.InsecurePaths {
+			args = append(args, "--disable-paths")
+		}
+		if cfg.InsecureSeccomp {
+			args = append(args, "--disable-seccomp")
+		}
+	}
+
+	if cfg.Mutable {
+		mutable, err := supportsMutableEnvironment(dir)
+
+		switch {
+		case err != nil:
+			log.FatalE("error determining stage1 mutable support", err)
+		case !mutable:
+			log.Fatalln("stage1 does not support mutable pods")
+		}
+
+		args = append(args, "--mutable")
+	}
+
 	args = append(args, cfg.UUID.String())
 
 	// make sure the lock fd stays open across exec
@@ -556,6 +738,8 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 	// that enforces TPM behaviour, but we don't have any infrastructure
 	// around that yet.
 	_ = tpm.Extend(tpmEvent)
+
+	debug("Execing %s", args)
 	if err := syscall.Exec(args[0], args, os.Environ()); err != nil {
 		log.FatalE("error execing init", err)
 	}
@@ -563,89 +747,97 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 
 // prepareAppImage renders and verifies the tree cache of the app image that
 // corresponds to the given app name.
-// When useOverlay is false, it attempts to render and expand the app image
-func prepareAppImage(cfg PrepareConfig, appName types.ACName, img types.Hash, cdir string, useOverlay bool) error {
+// When useOverlay is false, it attempts to render and expand the app image.
+// It returns the tree store ID if overlay is being used.
+func prepareAppImage(cfg PrepareConfig, appName types.ACName, img types.Hash, cdir string, useOverlay bool) (string, error) {
 	debug("Loading image %s", img.String())
 
 	am, err := cfg.Store.GetImageManifest(img.String())
 	if err != nil {
-		return errwrap.Wrap(errors.New("error getting the manifest"), err)
+		return "", errwrap.Wrap(errors.New("error getting the manifest"), err)
 	}
 
 	if _, hasOS := am.Labels.Get("os"); !hasOS {
-		return fmt.Errorf("missing os label in the image manifest")
+		return "", fmt.Errorf("missing os label in the image manifest")
 	}
+
 	if _, hasArch := am.Labels.Get("arch"); !hasArch {
-		return fmt.Errorf("missing arch label in the image manifest")
+		return "", fmt.Errorf("missing arch label in the image manifest")
 	}
 
 	if err := types.IsValidOSArch(am.Labels.ToMap(), ValidOSArch); err != nil {
-		return err
+		return "", err
 	}
 
 	appInfoDir := common.AppInfoPath(cdir, appName)
-	if err := os.MkdirAll(appInfoDir, defaultRegularDirPerm); err != nil {
-		return errwrap.Wrap(errors.New("error creating apps info directory"), err)
+	if err := os.MkdirAll(appInfoDir, common.DefaultRegularDirPerm); err != nil {
+		return "", errwrap.Wrap(errors.New("error creating apps info directory"), err)
 	}
 
+	var treeStoreID string
 	if useOverlay {
 		if cfg.PrivateUsers.Shift > 0 {
-			return fmt.Errorf("cannot use both overlay and user namespace: not implemented yet. (Try --no-overlay)")
+			return "", fmt.Errorf("cannot use both overlay and user namespace: not implemented yet. (Try --no-overlay)")
 		}
-		treeStoreID, _, err := cfg.Store.RenderTreeStore(img.String(), false)
+
+		treeStoreID, _, err = cfg.TreeStore.Render(img.String(), false)
 		if err != nil {
-			return errwrap.Wrap(errors.New("error rendering tree image"), err)
+			return "", errwrap.Wrap(errors.New("error rendering tree image"), err)
 		}
 
 		if !cfg.SkipTreeStoreCheck {
-			hash, err := cfg.Store.CheckTreeStore(treeStoreID)
+			hash, err := cfg.TreeStore.Check(treeStoreID)
 			if err != nil {
 				log.PrintE("warning: tree cache is in a bad state.  Rebuilding...", err)
 				var err error
-				treeStoreID, hash, err = cfg.Store.RenderTreeStore(img.String(), true)
+				treeStoreID, hash, err = cfg.TreeStore.Render(img.String(), true)
 				if err != nil {
-					return errwrap.Wrap(errors.New("error rendering tree image"), err)
+					return "", errwrap.Wrap(errors.New("error rendering tree image"), err)
 				}
 			}
 			cfg.CommonConfig.RootHash = hash
 		}
 
-		if err := ioutil.WriteFile(common.AppTreeStoreIDPath(cdir, appName), []byte(treeStoreID), defaultRegularFilePerm); err != nil {
-			return errwrap.Wrap(errors.New("error writing app treeStoreID"), err)
+		if err := ioutil.WriteFile(common.AppTreeStoreIDPath(cdir, appName), []byte(treeStoreID), common.DefaultRegularFilePerm); err != nil {
+			return "", errwrap.Wrap(errors.New("error writing app treeStoreID"), err)
 		}
 	} else {
 		ad := common.AppPath(cdir, appName)
-		err := os.MkdirAll(ad, defaultRegularDirPerm)
+
+		err := os.MkdirAll(ad, common.DefaultRegularDirPerm)
 		if err != nil {
-			return errwrap.Wrap(errors.New("error creating image directory"), err)
+			return "", errwrap.Wrap(errors.New("error creating image directory"), err)
 		}
 
 		shiftedUid, shiftedGid, err := cfg.PrivateUsers.ShiftRange(uint32(os.Getuid()), uint32(os.Getgid()))
 		if err != nil {
-			return errwrap.Wrap(errors.New("error getting uid, gid"), err)
+			return "", errwrap.Wrap(errors.New("error getting uid, gid"), err)
 		}
 
 		if err := os.Chown(ad, int(shiftedUid), int(shiftedGid)); err != nil {
-			return errwrap.Wrap(fmt.Errorf("error shifting app %q's stage2 dir", appName), err)
+			return "", errwrap.Wrap(fmt.Errorf("error shifting app %q's stage2 dir", appName), err)
 		}
 
 		if err := aci.RenderACIWithImageID(img, ad, cfg.Store, cfg.PrivateUsers); err != nil {
-			return errwrap.Wrap(errors.New("error rendering ACI"), err)
+			return "", errwrap.Wrap(errors.New("error rendering ACI"), err)
 		}
 	}
+
 	if err := writeManifest(*cfg.CommonConfig, img, appInfoDir); err != nil {
-		return err
+		return "", errwrap.Wrap(errors.New("error writing manifest"), err)
 	}
-	return nil
+
+	return treeStoreID, nil
 }
 
 // setupAppImage mounts the overlay filesystem for the app image that
-// corresponds to the given hash. Then, it creates the tmp directory.
-// When useOverlay is false it just creates the tmp directory for this app.
+// corresponds to the given hash if useOverlay is true.
+// It also creates an mtab file in the application's rootfs if one is not
+// present.
 func setupAppImage(cfg RunConfig, appName types.ACName, img types.Hash, cdir string, useOverlay bool) error {
 	ad := common.AppPath(cdir, appName)
 	if useOverlay {
-		err := os.MkdirAll(ad, defaultRegularDirPerm)
+		err := os.MkdirAll(ad, common.DefaultRegularDirPerm)
 		if err != nil {
 			return errwrap.Wrap(errors.New("error creating image directory"), err)
 		}
@@ -660,30 +852,62 @@ func setupAppImage(cfg RunConfig, appName types.ACName, img types.Hash, cdir str
 			return errwrap.Wrap(errors.New("error rendering overlay filesystem"), err)
 		}
 	}
+	return ensureMtabExists(filepath.Join(ad, "rootfs"))
+}
 
+// ensureMtabExists creates a symlink from /etc/mtab -> /proc/self/mounts if
+// nothing exists at /etc/mtab.
+// Various tools, such as mount from util-linux 2.25, expect the mtab file to
+// be populated.
+func ensureMtabExists(rootfs string) error {
+	stat, err := os.Stat(filepath.Join(rootfs, "etc"))
+	if os.IsNotExist(err) {
+		// If your image has no /etc you don't get /etc/mtab either
+		return nil
+	}
+	if err != nil {
+		return errwrap.Wrap(errors.New("error determining if /etc existed in the image"), err)
+	}
+	if !stat.IsDir() {
+		return nil
+	}
+	mtabPath := filepath.Join(rootfs, "etc", "mtab")
+	if _, err = os.Lstat(mtabPath); err == nil {
+		// If the image already has an mtab, don't replace it
+		return nil
+	}
+	if !os.IsNotExist(err) {
+		return errwrap.Wrap(errors.New("error determining if /etc/mtab exists in the image"), err)
+	}
+
+	target := "../proc/self/mounts"
+	err = os.Symlink(target, mtabPath)
+	if err != nil {
+		return errwrap.Wrap(errors.New("error creating mtab symlink"), err)
+	}
 	return nil
 }
 
 // prepareStage1Image renders and verifies tree cache of the given hash
 // when using overlay.
 // When useOverlay is false, it attempts to render and expand the stage1.
-func prepareStage1Image(cfg PrepareConfig, img types.Hash, cdir string, useOverlay bool) error {
+func prepareStage1Image(cfg PrepareConfig, cdir string) error {
 	s1 := common.Stage1ImagePath(cdir)
-	if err := os.MkdirAll(s1, defaultRegularDirPerm); err != nil {
+	if err := os.MkdirAll(s1, common.DefaultRegularDirPerm); err != nil {
 		return errwrap.Wrap(errors.New("error creating stage1 directory"), err)
 	}
 
-	treeStoreID, _, err := cfg.Store.RenderTreeStore(img.String(), false)
+	treeStoreID, _, err := cfg.TreeStore.Render(cfg.Stage1Image.String(), false)
 	if err != nil {
 		return errwrap.Wrap(errors.New("error rendering tree image"), err)
 	}
 
 	if !cfg.SkipTreeStoreCheck {
-		hash, err := cfg.Store.CheckTreeStore(treeStoreID)
+		hash, err := cfg.TreeStore.Check(treeStoreID)
 		if err != nil {
 			log.Printf("warning: tree cache is in a bad state: %v. Rebuilding...", err)
 			var err error
-			treeStoreID, hash, err = cfg.Store.RenderTreeStore(img.String(), true)
+			treeStoreID, hash, err = cfg.TreeStore.Render(cfg.Stage1Image.String(), true)
 			if err != nil {
 				return errwrap.Wrap(errors.New("error rendering tree image"), err)
 			}
@@ -691,20 +915,20 @@ func prepareStage1Image(cfg PrepareConfig, img types.Hash, cdir string, useOverl
 		cfg.CommonConfig.RootHash = hash
 	}
 
-	if err := writeManifest(*cfg.CommonConfig, img, s1); err != nil {
+	if err := writeManifest(*cfg.CommonConfig, cfg.Stage1Image, s1); err != nil {
 		return errwrap.Wrap(errors.New("error writing manifest"), err)
 	}
 
-	if !useOverlay {
+	if !cfg.UseOverlay {
 		destRootfs := filepath.Join(s1, "rootfs")
-		cachedTreePath := cfg.Store.GetTreeStoreRootFS(treeStoreID)
+		cachedTreePath := cfg.TreeStore.GetRootFS(treeStoreID)
 		if err := fileutil.CopyTree(cachedTreePath, destRootfs, cfg.PrivateUsers); err != nil {
 			return errwrap.Wrap(errors.New("error rendering ACI"), err)
 		}
 	}
 
 	fn := path.Join(cdir, common.Stage1TreeStoreIDFilename)
-	if err := ioutil.WriteFile(fn, []byte(treeStoreID), defaultRegularFilePerm); err != nil {
+	if err := ioutil.WriteFile(fn, []byte(treeStoreID), common.DefaultRegularFilePerm); err != nil {
 		return errwrap.Wrap(errors.New("error writing stage1 treeStoreID"), err)
 	}
 	return nil
@@ -745,7 +969,7 @@ func writeManifest(cfg CommonConfig, img types.Hash, dest string) error {
 	}
 
 	debug("Writing image manifest")
-	if err := ioutil.WriteFile(filepath.Join(dest, "manifest"), mb, defaultRegularFilePerm); err != nil {
+	if err := ioutil.WriteFile(filepath.Join(dest, "manifest"), mb, common.DefaultRegularFilePerm); err != nil {
 		return errwrap.Wrap(errors.New("error writing image manifest"), err)
 	}
 
@@ -765,24 +989,40 @@ func copyAppManifest(cdir string, appName types.ACName, dest string) error {
 }
 
 // overlayRender renders the image that corresponds to the given hash using the
-// overlay filesystem.
-// It mounts an overlay filesystem from the cached tree of the image as rootfs.
+// overlay filesystem. It mounts an overlay filesystem from the cached tree of
+// the image as rootfs.
 func overlayRender(cfg RunConfig, treeStoreID string, cdir string, dest string, appName string) error {
-	cachedTreePath := cfg.Store.GetTreeStoreRootFS(treeStoreID)
-	fi, err := os.Stat(cachedTreePath)
+	cachedTreePath := cfg.TreeStore.GetRootFS(treeStoreID)
+	mc, err := prepareOverlay(cachedTreePath, treeStoreID, cdir, dest, appName, cfg.MountLabel,
+		cfg.RktGid, common.DefaultRegularDirPerm)
 	if err != nil {
-		return err
+		return errwrap.Wrap(errors.New("problem preparing overlay directories"), err)
+	}
+	if err = overlay.Mount(mc); err != nil {
+		return errwrap.Wrap(errors.New("problem mounting overlay filesystem"), err)
+	}
+
+	return nil
+}
+
+// prepateOverlay sets up the needed directories, files and permissions for the
+// overlay-rendered pods
+func prepareOverlay(lower, treeStoreID, cdir, dest, appName, lbl string,
+	gid int, fm os.FileMode) (*overlay.MountCfg, error) {
+	fi, err := os.Stat(lower)
+	if err != nil {
+		return nil, err
 	}
 	imgMode := fi.Mode()
 
-	destRootfs := path.Join(dest, "rootfs")
-	if err := os.MkdirAll(destRootfs, imgMode); err != nil {
-		return err
+	dst := path.Join(dest, "rootfs")
+	if err := os.MkdirAll(dst, imgMode); err != nil {
+		return nil, err
 	}
 
 	overlayDir := path.Join(cdir, "overlay")
-	if err := os.MkdirAll(overlayDir, defaultRegularDirPerm); err != nil {
-		return err
+	if err := os.MkdirAll(overlayDir, fm); err != nil {
+		return nil, err
 	}
 
 	// Since the parent directory (rkt/pods/$STATE/$POD_UUID) has the 'S_ISGID' bit, here
@@ -790,43 +1030,36 @@ func overlayRender(cfg RunConfig, treeStoreID string, cdir string, dest string,
 	// directory so that it won't inherit the bit. Otherwise the files
 	// created by users within the pod will inherit the 'S_ISGID' bit
 	// as well.
-	if err := os.Chmod(overlayDir, defaultRegularDirPerm); err != nil {
-		return err
+	if err := os.Chmod(overlayDir, fm); err != nil {
+		return nil, err
 	}
 
 	imgDir := path.Join(overlayDir, treeStoreID)
-	if err := os.MkdirAll(imgDir, defaultRegularDirPerm); err != nil {
-		return err
+	if err := os.MkdirAll(imgDir, fm); err != nil {
+		return nil, err
 	}
-
 	// Also make 'rkt/pods/$STATE/$POD_UUID/overlay/$IMAGE_ID' to be readable by 'rkt' group
 	// As 'rkt' status will read the 'rkt/pods/$STATE/$POD_UUID/overlay/$IMAGE_ID/upper/rkt/status/$APP'
-	// to get exit status.
-	if err := os.Chown(imgDir, -1, cfg.RktGid); err != nil {
-		return err
+	// to get exgid
+	if err := os.Chown(imgDir, -1, gid); err != nil {
+		return nil, err
 	}
 
-	upperDir := path.Join(imgDir, "upper", appName)
-	if err := os.MkdirAll(upperDir, imgMode); err != nil {
-		return err
+	upper := path.Join(imgDir, "upper", appName)
+	if err := os.MkdirAll(upper, imgMode); err != nil {
+		return nil, err
 	}
-	if err := label.SetFileLabel(upperDir, cfg.MountLabel); err != nil {
-		return err
+	if err := label.SetFileLabel(upper, lbl); err != nil {
+		return nil, err
 	}
 
-	workDir := path.Join(imgDir, "work", appName)
-	if err := os.MkdirAll(workDir, defaultRegularDirPerm); err != nil {
-		return err
+	work := path.Join(imgDir, "work", appName)
+	if err := os.MkdirAll(work, fm); err != nil {
+		return nil, err
 	}
-	if err := label.SetFileLabel(workDir, cfg.MountLabel); err != nil {
-		return err
+	if err := label.SetFileLabel(work, lbl); err != nil {
+		return nil, err
 	}
 
-	opts := fmt.Sprintf("lowerdir=%s,upperdir=%s,workdir=%s", cachedTreePath, upperDir, workDir)
-	opts = label.FormatMountLabel(opts, cfg.MountLabel)
-	if err := syscall.Mount("overlay", destRootfs, "overlay", 0, opts); err != nil {
-		return errwrap.Wrap(errors.New("error mounting"), err)
-	}
-
-	return nil
+	return &overlay.MountCfg{lower, upper, work, dst, lbl}, nil
 }
