@@ -17,7 +17,6 @@
 package common
 
 import (
-	"bytes"
 	"errors"
 	"fmt"
 	"io"
@@ -38,7 +37,6 @@ import (
 	"github.com/hashicorp/errwrap"
 
 	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/common/cgroup"
 	"github.com/coreos/rkt/pkg/fileutil"
 	"github.com/coreos/rkt/pkg/user"
 )
@@ -49,16 +47,6 @@ const (
 	SharedVolPerm = os.FileMode(0755)
 )
 
-var (
-	defaultEnv = map[string]string{
-		"PATH":    "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-		"SHELL":   "/bin/sh",
-		"USER":    "root",
-		"LOGNAME": "root",
-		"HOME":    "/root",
-	}
-)
-
 // execEscape uses Golang's string quoting for ", \, \n, and regex for special cases
 func execEscape(i int, str string) string {
 	escapeMap := map[string]string{
@@ -96,66 +84,7 @@ func quoteExec(exec []string) string {
 	return strings.Join(qexec, " ")
 }
 
-// WriteDefaultTarget writes the default.target unit file
-// which is responsible for bringing up the applications
-func WriteDefaultTarget(p *stage1commontypes.Pod) error {
-	opts := []*unit.UnitOption{
-		unit.NewUnitOption("Unit", "Description", "rkt apps target"),
-		unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
-	}
-
-	for i := range p.Manifest.Apps {
-		ra := &p.Manifest.Apps[i]
-		serviceName := ServiceUnitName(ra.Name)
-		opts = append(opts, unit.NewUnitOption("Unit", "After", serviceName))
-		opts = append(opts, unit.NewUnitOption("Unit", "Wants", serviceName))
-	}
-
-	unitsPath := filepath.Join(common.Stage1RootfsPath(p.Root), UnitsDir)
-	file, err := os.OpenFile(filepath.Join(unitsPath, "default.target"), os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0644)
-	if err != nil {
-		return err
-	}
-	defer file.Close()
-
-	if _, err = io.Copy(file, unit.Serialize(opts)); err != nil {
-		return err
-	}
-
-	return nil
-}
-
-// WritePrepareAppTemplate writes service unit files for preparing the pod's applications
-func WritePrepareAppTemplate(p *stage1commontypes.Pod) error {
-	opts := []*unit.UnitOption{
-		unit.NewUnitOption("Unit", "Description", "Prepare minimum environment for chrooted applications"),
-		unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
-		unit.NewUnitOption("Unit", "OnFailureJobMode", "fail"),
-		unit.NewUnitOption("Unit", "Requires", "systemd-journald.service"),
-		unit.NewUnitOption("Unit", "After", "systemd-journald.service"),
-		unit.NewUnitOption("Service", "Type", "oneshot"),
-		unit.NewUnitOption("Service", "Restart", "no"),
-		unit.NewUnitOption("Service", "ExecStart", "/prepare-app %I"),
-		unit.NewUnitOption("Service", "User", "0"),
-		unit.NewUnitOption("Service", "Group", "0"),
-		unit.NewUnitOption("Service", "CapabilityBoundingSet", "CAP_SYS_ADMIN CAP_DAC_OVERRIDE"),
-	}
-
-	unitsPath := filepath.Join(common.Stage1RootfsPath(p.Root), UnitsDir)
-	file, err := os.OpenFile(filepath.Join(unitsPath, "prepare-app@.service"), os.O_WRONLY|os.O_CREATE, 0644)
-	if err != nil {
-		return errwrap.Wrap(errors.New("failed to create service unit file"), err)
-	}
-	defer file.Close()
-
-	if _, err = io.Copy(file, unit.Serialize(opts)); err != nil {
-		return errwrap.Wrap(errors.New("failed to write service unit file"), err)
-	}
-
-	return nil
-}
-
-func writeAppReaper(p *stage1commontypes.Pod, appName string) error {
+func writeAppReaper(p *stage1commontypes.Pod, appName string, appRootDirectory string, binPath string) error {
 	opts := []*unit.UnitOption{
 		unit.NewUnitOption("Unit", "Description", fmt.Sprintf("%s Reaper", appName)),
 		unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
@@ -166,7 +95,7 @@ func writeAppReaper(p *stage1commontypes.Pod, appName string) error {
 		unit.NewUnitOption("Unit", "Conflicts", "halt.target"),
 		unit.NewUnitOption("Unit", "Conflicts", "poweroff.target"),
 		unit.NewUnitOption("Service", "RemainAfterExit", "yes"),
-		unit.NewUnitOption("Service", "ExecStop", fmt.Sprintf("/reaper.sh %s", appName)),
+		unit.NewUnitOption("Service", "ExecStop", fmt.Sprintf("/reaper.sh \"%s\" \"%s\" \"%s\"", appName, appRootDirectory, binPath)),
 	}
 
 	unitsPath := filepath.Join(common.Stage1RootfsPath(p.Root), UnitsDir)
@@ -280,12 +209,8 @@ func generateSysusers(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uid_ int,
 		return err
 	}
 
-	if uidRange.Shift != 0 && uidRange.Count != 0 {
-		for _, f := range toShift {
-			if err := os.Chown(f, int(uidRange.Shift), int(uidRange.Shift)); err != nil {
-				return err
-			}
-		}
+	if err := user.ShiftFiles(toShift, uidRange); err != nil {
+		return err
 	}
 
 	return nil
@@ -335,10 +260,16 @@ func appSearchPaths(p *stage1commontypes.Pod, workDir string, app types.App) []s
 	return []string{workDir, "/bin", "/usr/bin"}
 }
 
-// findBinPath takes a binary path and returns a the absolute path of the
+// FindBinPath takes a binary path and returns a the absolute path of the
 // binary relative to the app rootfs. This can be passed to ExecStart on the
 // app's systemd service file directly.
-func findBinPath(p *stage1commontypes.Pod, appName types.ACName, app types.App, workDir string, bin string) (string, error) {
+func FindBinPath(p *stage1commontypes.Pod, ra *schema.RuntimeApp) (string, error) {
+	if len(ra.App.Exec) == 0 {
+		return "", errors.New("app has no executable")
+	}
+
+	bin := ra.App.Exec[0]
+
 	var binPath string
 	switch {
 	// absolute path, just use it
@@ -346,18 +277,18 @@ func findBinPath(p *stage1commontypes.Pod, appName types.ACName, app types.App,
 		binPath = bin
 	// non-absolute path containing a slash, look in the working dir
 	case strings.Contains(bin, "/"):
-		binPath = filepath.Join(workDir, bin)
+		binPath = filepath.Join(ra.App.WorkingDirectory, bin)
 	// filename, search in the app's $PATH
 	default:
 		absRoot, err := filepath.Abs(p.Root)
 		if err != nil {
 			return "", errwrap.Wrap(errors.New("could not get pod's root absolute path"), err)
 		}
-		appRootfs := common.AppRootfsPath(absRoot, appName)
-		appPathDirs := appSearchPaths(p, workDir, app)
+		appRootfs := common.AppRootfsPath(absRoot, ra.Name)
+		appPathDirs := appSearchPaths(p, ra.App.WorkingDirectory, *ra.App)
 		appPath := strings.Join(appPathDirs, ":")
 
-		binPath, err = lookupPathInsideApp(bin, appPath, appRootfs, workDir)
+		binPath, err = lookupPathInsideApp(bin, appPath, appRootfs, ra.App.WorkingDirectory)
 		if err != nil {
 			return "", errwrap.Wrap(fmt.Errorf("error looking up %q", bin), err)
 		}
@@ -366,255 +297,64 @@ func findBinPath(p *stage1commontypes.Pod, appName types.ACName, app types.App,
 	return binPath, nil
 }
 
-// appToSystemd transforms the provided RuntimeApp+ImageManifest into systemd units
-func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive bool, flavor string, privateUsers string) error {
-	app := ra.App
-	appName := ra.Name
-	imgName := p.AppNameToImageName(appName)
+// generateDeviceAllows generates a DeviceAllow= line for an app.
+// To make it work, the path needs to start with "/dev" but the device won't
+// exist inside the container. So for a given mount, if the volume is a device
+// node, we create a symlink to its target in "/rkt/volumes". Later,
+// prepare-app will copy those to "/dev/.rkt/" so that's what we use in the
+// DeviceAllow= line.
+func generateDeviceAllows(root string, appName types.ACName, mountPoints []types.MountPoint, mounts []mountWrapper, uidRange *user.UidRange) ([]string, error) {
+	var devAllow []string
 
-	if len(app.Exec) == 0 {
-		return fmt.Errorf(`image %q has an empty "exec" (try --exec=BINARY)`, imgName)
+	rktVolumeLinksPath := filepath.Join(root, "rkt", "volumes")
+	if err := os.MkdirAll(rktVolumeLinksPath, 0600); err != nil {
+		return nil, err
+	}
+	if err := user.ShiftFiles([]string{rktVolumeLinksPath}, uidRange); err != nil {
+		return nil, err
 	}
 
-	workDir := "/"
-	if app.WorkingDirectory != "" {
-		workDir = app.WorkingDirectory
-	}
-
-	env := app.Environment
-
-	env.Set("AC_APP_NAME", appName.String())
-	if p.MetadataServiceURL != "" {
-		env.Set("AC_METADATA_URL", p.MetadataServiceURL)
-	}
-
-	// TODO(iaguis): make enterexec use the same format as systemd
-	envFilePathSystemd := EnvFilePathSystemd(p.Root, appName)
-	envFilePathEnterexec := EnvFilePathEnterexec(p.Root, appName)
-
-	uidRange := user.NewBlankUidRange()
-	if err := uidRange.Deserialize([]byte(privateUsers)); err != nil {
-		return err
-	}
-
-	if err := writeEnvFile(p, env, appName, uidRange, '\n', envFilePathSystemd); err != nil {
-		return errwrap.Wrap(errors.New("unable to write environment file for systemd"), err)
-	}
-
-	if err := writeEnvFile(p, env, appName, uidRange, '\000', envFilePathEnterexec); err != nil {
-		return errwrap.Wrap(errors.New("unable to write environment file for enterexec"), err)
-	}
-
-	u, g, err := parseUserGroup(p, ra, uidRange)
-	if err != nil {
-		return err
-	}
-
-	if err := generateSysusers(p, ra, u, g, uidRange); err != nil {
-		return errwrap.Wrap(errors.New("unable to generate sysusers"), err)
-	}
-
-	binPath, err := findBinPath(p, appName, *app, workDir, app.Exec[0])
-	if err != nil {
-		return err
-	}
-
-	var supplementaryGroups []string
-	for _, g := range app.SupplementaryGIDs {
-		supplementaryGroups = append(supplementaryGroups, strconv.Itoa(g))
-	}
-
-	capabilitiesStr, err := getAppCapabilities(app.Isolators)
-	if err != nil {
-		return err
-	}
-
-	noNewPrivileges := getAppNoNewPrivileges(app.Isolators)
-
-	execStart := append([]string{binPath}, app.Exec[1:]...)
-	execStartString := quoteExec(execStart)
-	opts := []*unit.UnitOption{
-		unit.NewUnitOption("Unit", "Description", fmt.Sprintf("Application=%v Image=%v", appName, imgName)),
-		unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
-		unit.NewUnitOption("Unit", "Wants", fmt.Sprintf("reaper-%s.service", appName)),
-		unit.NewUnitOption("Service", "Restart", "no"),
-		unit.NewUnitOption("Service", "ExecStart", execStartString),
-		unit.NewUnitOption("Service", "RootDirectory", common.RelAppRootfsPath(appName)),
-		// MountFlags=shared creates a new mount namespace and (as unintuitive
-		// as it might seem) makes sure the mount is slave+shared.
-		unit.NewUnitOption("Service", "MountFlags", "shared"),
-		unit.NewUnitOption("Service", "WorkingDirectory", workDir),
-		unit.NewUnitOption("Service", "EnvironmentFile", RelEnvFilePathSystemd(appName)),
-		unit.NewUnitOption("Service", "User", strconv.Itoa(u)),
-		unit.NewUnitOption("Service", "Group", strconv.Itoa(g)),
-		unit.NewUnitOption("Service", "SupplementaryGroups", strings.Join(supplementaryGroups, " ")),
-		unit.NewUnitOption("Service", "CapabilityBoundingSet", strings.Join(capabilitiesStr, " ")),
-		unit.NewUnitOption("Service", "NoNewPrivileges", strconv.FormatBool(noNewPrivileges)),
-		// This helps working around a race
-		// (https://github.com/systemd/systemd/issues/2913) that causes the
-		// systemd unit name not getting written to the journal if the unit is
-		// short-lived and runs as non-root.
-		unit.NewUnitOption("Service", "SyslogIdentifier", appName.String()),
-	}
-
-	// Restrict access to sensitive paths (eg. procfs)
-	opts = protectSystemFiles(opts, appName)
-
-	if ra.ReadOnlyRootFS {
-		opts = append(opts, unit.NewUnitOption("Service", "ReadOnlyDirectories", common.RelAppRootfsPath(appName)))
-	}
-
-	// TODO(tmrts): Extract this logic into a utility function.
-	vols := make(map[types.ACName]types.Volume)
-	for _, v := range p.Manifest.Volumes {
-		vols[v.Name] = v
-	}
-
-	absRoot, err := filepath.Abs(p.Root) // Absolute path to the pod's rootfs.
-	if err != nil {
-		return err
-	}
-	appRootfs := common.AppRootfsPath(absRoot, appName)
-
-	rwDirs := []string{}
-	imageManifest := p.Images[appName.String()]
-	for _, m := range GenerateMounts(ra, vols, imageManifest) {
-		mntPath, err := EvaluateSymlinksInsideApp(appRootfs, m.Path)
-		if err != nil {
-			return err
+	for _, m := range mounts {
+		if m.Volume.Kind != "host" {
+			continue
 		}
-
-		if !IsMountReadOnly(vols[m.Volume], app.MountPoints) {
-			rwDirs = append(rwDirs, filepath.Join(common.RelAppRootfsPath(appName), mntPath))
-		}
-	}
-
-	opts = append(opts, unit.NewUnitOption("Service", "ReadWriteDirectories", strings.Join(rwDirs, " ")))
-
-	if interactive {
-		opts = append(opts, unit.NewUnitOption("Service", "StandardInput", "tty"))
-		opts = append(opts, unit.NewUnitOption("Service", "StandardOutput", "tty"))
-		opts = append(opts, unit.NewUnitOption("Service", "StandardError", "tty"))
-	} else {
-		opts = append(opts, unit.NewUnitOption("Service", "StandardOutput", "journal+console"))
-		opts = append(opts, unit.NewUnitOption("Service", "StandardError", "journal+console"))
-	}
-
-	// When an app fails, we shut down the pod
-	opts = append(opts, unit.NewUnitOption("Unit", "OnFailure", "halt.target"))
-
-	for _, eh := range app.EventHandlers {
-		var typ string
-		switch eh.Name {
-		case "pre-start":
-			typ = "ExecStartPre"
-		case "post-stop":
-			typ = "ExecStopPost"
-		default:
-			return fmt.Errorf("unrecognized eventHandler: %v", eh.Name)
-		}
-		exec := quoteExec(eh.Exec)
-		opts = append(opts, unit.NewUnitOption("Service", typ, exec))
-	}
-
-	// Some pre-start jobs take a long time, set the timeout to 0
-	opts = append(opts, unit.NewUnitOption("Service", "TimeoutStartSec", "0"))
-
-	var saPorts []types.Port
-	for _, p := range app.Ports {
-		if p.SocketActivated {
-			saPorts = append(saPorts, p)
-		}
-	}
-
-	for _, i := range app.Isolators {
-		switch v := i.Value().(type) {
-		case *types.ResourceMemory:
-			opts, err = cgroup.MaybeAddIsolator(opts, "memory", v.Limit())
-			if err != nil {
-				return err
+		if fileutil.IsDeviceNode(m.Volume.Source) {
+			mode := "r"
+			if !m.ReadOnly {
+				mode += "w"
 			}
-		case *types.ResourceCPU:
-			opts, err = cgroup.MaybeAddIsolator(opts, "cpu", v.Limit())
-			if err != nil {
-				return err
+
+			tgt := filepath.Join(common.RelAppRootfsPath(appName), m.Mount.Path)
+			// the DeviceAllow= line needs the link path in /dev/.rkt/
+			linkRel := filepath.Join("/dev/.rkt", m.Volume.Name.String())
+			// the real link should be in /rkt/volumes for now
+			link := filepath.Join(rktVolumeLinksPath, m.Volume.Name.String())
+
+			err := os.Symlink(tgt, link)
+			// if the link already exists, we don't need to do anything
+			if err != nil && !os.IsExist(err) {
+				return nil, err
 			}
+
+			devAllow = append(devAllow, linkRel+" "+mode)
 		}
 	}
 
-	if len(saPorts) > 0 {
-		sockopts := []*unit.UnitOption{
-			unit.NewUnitOption("Unit", "Description", fmt.Sprintf("Application=%v Image=%v %s", appName, imgName, "socket-activated ports")),
-			unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
-			unit.NewUnitOption("Socket", "BindIPv6Only", "both"),
-			unit.NewUnitOption("Socket", "Service", ServiceUnitName(appName)),
-		}
+	return devAllow, nil
+}
 
-		for _, sap := range saPorts {
-			var proto string
-			switch sap.Protocol {
-			case "tcp":
-				proto = "ListenStream"
-			case "udp":
-				proto = "ListenDatagram"
-			default:
-				return fmt.Errorf("unrecognized protocol: %v", sap.Protocol)
-			}
-			// We find the host port for the pod's port and use that in the
-			// socket unit file.
-			// This is so because systemd inside the pod will match based on
-			// the socket port number, and since the socket was created on the
-			// host, it will have the host port number.
-			port := findHostPort(*p.Manifest, sap.Name)
-			if port == 0 {
-				log.Printf("warning: no --port option for socket-activated port %q, assuming port %d as specified in the manifest", sap.Name, sap.Port)
-				port = sap.Port
-			}
-			sockopts = append(sockopts, unit.NewUnitOption("Socket", proto, fmt.Sprintf("%v", port)))
-		}
-
-		file, err := os.OpenFile(SocketUnitPath(p.Root, appName), os.O_WRONLY|os.O_CREATE, 0644)
-		if err != nil {
-			return errwrap.Wrap(errors.New("failed to create socket file"), err)
-		}
-		defer file.Close()
-
-		if _, err = io.Copy(file, unit.Serialize(sockopts)); err != nil {
-			return errwrap.Wrap(errors.New("failed to write socket unit file"), err)
-		}
-
-		if err = os.Symlink(path.Join("..", SocketUnitName(appName)), SocketWantPath(p.Root, appName)); err != nil {
-			return errwrap.Wrap(errors.New("failed to link socket want"), err)
-		}
-
-		opts = append(opts, unit.NewUnitOption("Unit", "Requires", SocketUnitName(appName)))
+// supportsNotify returns true if in the image manifest appc.io/executor/supports-systemd-notify is set to true
+func supportsNotify(p *stage1commontypes.Pod, appName string) bool {
+	appImg := p.Images[appName]
+	if appImg == nil {
+		return false
 	}
-
-	opts = append(opts, unit.NewUnitOption("Unit", "Requires", InstantiatedPrepareAppUnitName(appName)))
-	opts = append(opts, unit.NewUnitOption("Unit", "After", InstantiatedPrepareAppUnitName(appName)))
-
-	opts = append(opts, unit.NewUnitOption("Unit", "Requires", "sysusers.service"))
-	opts = append(opts, unit.NewUnitOption("Unit", "After", "sysusers.service"))
-
-	file, err := os.OpenFile(ServiceUnitPath(p.Root, appName), os.O_WRONLY|os.O_CREATE, 0644)
-	if err != nil {
-		return errwrap.Wrap(errors.New("failed to create service unit file"), err)
+	supportNotifyAnnotation, ok := appImg.Annotations.Get("appc.io/executor/supports-systemd-notify")
+	supportNotify, err := strconv.ParseBool(supportNotifyAnnotation)
+	if ok && supportNotify && err == nil {
+		return true
 	}
-	defer file.Close()
-
-	if _, err = io.Copy(file, unit.Serialize(opts)); err != nil {
-		return errwrap.Wrap(errors.New("failed to write service unit file"), err)
-	}
-
-	if err = os.Symlink(path.Join("..", ServiceUnitName(appName)), ServiceWantPath(p.Root, appName)); err != nil {
-		return errwrap.Wrap(errors.New("failed to link service want"), err)
-	}
-
-	if err = writeAppReaper(p, appName.String()); err != nil {
-		return errwrap.Wrap(fmt.Errorf("failed to write app %q reaper service", appName), err)
-	}
-
-	return nil
+	return false
 }
 
 // parseUserGroup parses the User and Group fields of an App and returns its
@@ -625,7 +365,7 @@ func appToSystemd(p *stage1commontypes.Pod, ra *schema.RuntimeApp, interactive b
 //   3. a number
 //   4. a name in reference to /etc/{group,passwd} in the image
 // See https://github.com/appc/spec/blob/master/spec/aci.md#image-manifest-schema
-func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *user.UidRange) (int, int, error) {
+func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp) (int, int, error) {
 	var uidResolver, gidResolver user.Resolver
 	var uid, gid int
 	var err error
@@ -634,7 +374,7 @@ func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *u
 
 	uidResolver, err = user.NumericIDs(ra.App.User)
 	if err != nil {
-		uidResolver, err = user.IDsFromStat(root, ra.App.User, uidRange)
+		uidResolver, err = user.IDsFromStat(root, ra.App.User, &p.UidRange)
 	}
 
 	if err != nil {
@@ -651,7 +391,7 @@ func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *u
 
 	gidResolver, err = user.NumericIDs(ra.App.Group)
 	if err != nil {
-		gidResolver, err = user.IDsFromStat(root, ra.App.Group, uidRange)
+		gidResolver, err = user.IDsFromStat(root, ra.App.Group, &p.UidRange)
 	}
 
 	if err != nil {
@@ -669,99 +409,9 @@ func parseUserGroup(p *stage1commontypes.Pod, ra *schema.RuntimeApp, uidRange *u
 	return uid, gid, nil
 }
 
-func writeShutdownService(p *stage1commontypes.Pod) error {
-	flavor, systemdVersion, err := GetFlavor(p)
-	if err != nil {
-		return err
-	}
-
-	opts := []*unit.UnitOption{
-		unit.NewUnitOption("Unit", "Description", "Pod shutdown"),
-		unit.NewUnitOption("Unit", "AllowIsolate", "true"),
-		unit.NewUnitOption("Unit", "StopWhenUnneeded", "yes"),
-		unit.NewUnitOption("Unit", "DefaultDependencies", "false"),
-		unit.NewUnitOption("Service", "RemainAfterExit", "yes"),
-	}
-
-	shutdownVerb := "exit"
-	// systemd <v227 doesn't allow the "exit" verb when running as PID 1, so
-	// use "halt".
-	// If systemdVersion is 0 it means it couldn't be guessed, assume it's new
-	// enough for "systemctl exit".
-	// This can happen, for example, when building rkt with:
-	//
-	// ./configure --with-stage1-flavors=src --with-stage1-systemd-version=master
-	//
-	// The patches for the "exit" verb are backported to the "coreos" flavor, so
-	// don't rely on the systemd version on the "coreos" flavor.
-	if flavor != "coreos" && systemdVersion != 0 && systemdVersion < 227 {
-		shutdownVerb = "halt"
-	}
-
-	opts = append(opts, unit.NewUnitOption("Service", "ExecStop", fmt.Sprintf("/usr/bin/systemctl --force %s", shutdownVerb)))
-
-	unitsPath := filepath.Join(common.Stage1RootfsPath(p.Root), UnitsDir)
-	file, err := os.OpenFile(filepath.Join(unitsPath, "shutdown.service"), os.O_WRONLY|os.O_CREATE, 0644)
-	if err != nil {
-		return errwrap.Wrap(errors.New("failed to create unit file"), err)
-	}
-	defer file.Close()
-
-	if _, err = io.Copy(file, unit.Serialize(opts)); err != nil {
-		return errwrap.Wrap(errors.New("failed to write unit file"), err)
-	}
-
-	return nil
-}
-
-// writeEnvFile creates an environment file for given app name, the minimum
-// required environment variables by the appc spec will be set to sensible
-// defaults here if they're not provided by env.
-func writeEnvFile(p *stage1commontypes.Pod, env types.Environment, appName types.ACName, uidRange *user.UidRange, separator byte, envFilePath string) error {
-	ef := bytes.Buffer{}
-
-	for dk, dv := range defaultEnv {
-		if _, exists := env.Get(dk); !exists {
-			fmt.Fprintf(&ef, "%s=%s%c", dk, dv, separator)
-		}
-	}
-
-	for _, e := range env {
-		fmt.Fprintf(&ef, "%s=%s%c", e.Name, e.Value, separator)
-	}
-
-	if err := ioutil.WriteFile(envFilePath, ef.Bytes(), 0644); err != nil {
-		return err
-	}
-
-	if uidRange.Shift != 0 && uidRange.Count != 0 {
-		if err := os.Chown(envFilePath, int(uidRange.Shift), int(uidRange.Shift)); err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
-// PodToSystemd creates the appropriate systemd service unit files for
-// all the constituent apps of the Pod
-func PodToSystemd(p *stage1commontypes.Pod, interactive bool, flavor string, privateUsers string) error {
-	for i := range p.Manifest.Apps {
-		ra := &p.Manifest.Apps[i]
-		if err := appToSystemd(p, ra, interactive, flavor, privateUsers); err != nil {
-			return errwrap.Wrap(fmt.Errorf("failed to transform app %q into systemd service", ra.Name), err)
-		}
-	}
-
-	if err := writeShutdownService(p); err != nil {
-		return errwrap.Wrap(errors.New("failed to write shutdown service"), err)
-	}
-
-	return nil
-}
-
 // EvaluateSymlinksInsideApp tries to resolve symlinks within the path.
 // It returns the actual path relative to the app rootfs for the given path.
+// This is needed for absolute symlinks - we are in a different rootfs.
 func EvaluateSymlinksInsideApp(appRootfs, path string) (string, error) {
 	link := appRootfs
 
@@ -828,11 +478,13 @@ func appToNspawnArgs(p *stage1commontypes.Pod, ra *schema.RuntimeApp) ([]string,
 	}
 
 	imageManifest := p.Images[appName.String()]
-	mounts := GenerateMounts(ra, vols, imageManifest)
+	mounts, err := GenerateMounts(ra, p.Manifest.Volumes, ConvertedFromDocker(imageManifest))
+	if err != nil {
+		return nil, errwrap.Wrap(fmt.Errorf("could not generate app %q mounts", appName), err)
+	}
 	for _, m := range mounts {
-		vol := vols[m.Volume]
 
-		shPath := filepath.Join(sharedVolPath, vol.Name.String())
+		shPath := filepath.Join(sharedVolPath, m.Volume.Name.String())
 
 		absRoot, err := filepath.Abs(p.Root) // Absolute path to the pod's rootfs.
 		if err != nil {
@@ -841,46 +493,56 @@ func appToNspawnArgs(p *stage1commontypes.Pod, ra *schema.RuntimeApp) ([]string,
 
 		appRootfs := common.AppRootfsPath(absRoot, appName)
 
-		// TODO(yifan): This is a temporary fix for systemd-nspawn not handling symlink mounts well.
-		// Could be removed when https://github.com/systemd/systemd/issues/2860 is resolved, and systemd
-		// version is bumped.
-		mntPath, err := EvaluateSymlinksInsideApp(appRootfs, m.Path)
+		// Evaluate symlinks within the app's rootfs. This is needed because symlinks
+		// within the container can be absolute, which will, of course, be wrong in our ns.
+		// Systemd also gets this wrong, see https://github.com/systemd/systemd/issues/2860
+		// When the above issue is fixed, we can pass the un-evaluated path to --bind instead.
+		mntPath, err := EvaluateSymlinksInsideApp(appRootfs, m.Mount.Path)
 		if err != nil {
-			return nil, errwrap.Wrap(fmt.Errorf("could not evaluate path %v", m.Path), err)
+			return nil, errwrap.Wrap(fmt.Errorf("could not evaluate path %v", m.Mount.Path), err)
 		}
 		mntAbsPath := filepath.Join(appRootfs, mntPath)
 
-		if err := PrepareMountpoints(shPath, mntAbsPath, &vol, m.DockerImplicit); err != nil {
+		if err := PrepareMountpoints(shPath, mntAbsPath, &m.Volume, m.DockerImplicit); err != nil {
 			return nil, err
 		}
 
-		opt := make([]string, 4)
+		opt := make([]string, 6)
 
-		if IsMountReadOnly(vol, app.MountPoints) {
+		if m.ReadOnly {
 			opt[0] = "--bind-ro="
 		} else {
 			opt[0] = "--bind="
 		}
 
-		switch vol.Kind {
-		case "host":
-			opt[1] = vol.Source
-		case "empty":
-			opt[1] = filepath.Join(common.SharedVolumesPath(absRoot), vol.Name.String())
-		default:
-			return nil, fmt.Errorf(`invalid volume kind %q. Must be one of "host" or "empty"`, vol.Kind)
-		}
+		opt[1] = m.Source(absRoot)
 		opt[2] = ":"
 		opt[3] = filepath.Join(common.RelAppRootfsPath(appName), mntPath)
+		opt[4] = ":"
+
+		// If Recursive is not set, default to recursive.
+		recursive := true
+		if m.Volume.Recursive != nil {
+			recursive = *m.Volume.Recursive
+		}
+
+		// rbind/norbind options exist since systemd-nspawn v226
+		if recursive {
+			opt[5] = "rbind"
+		} else {
+			opt[5] = "norbind"
+		}
 		args = append(args, strings.Join(opt, ""))
 	}
 
-	capabilitiesStr, err := getAppCapabilities(app.Isolators)
-	if err != nil {
-		return nil, err
+	if !p.InsecureOptions.DisableCapabilities {
+		capabilitiesStr, err := getAppCapabilities(app.Isolators)
+		if err != nil {
+			return nil, err
+		}
+		capList := strings.Join(capabilitiesStr, ",")
+		args = append(args, "--capability="+capList)
 	}
-	capList := strings.Join(capabilitiesStr, ",")
-	args = append(args, "--capability="+capList)
 
 	return args, nil
 }
@@ -902,6 +564,10 @@ func PodToNspawnArgs(p *stage1commontypes.Pod) ([]string, error) {
 		args = append(args, aa...)
 	}
 
+	if p.InsecureOptions.DisableCapabilities {
+		args = append(args, "--capability=all")
+	}
+
 	return args, nil
 }
 
@@ -1065,9 +731,10 @@ func getAppNoNewPrivileges(isolators types.Isolators) bool {
 	return false
 }
 
-// restrictProcFS restricts access to some security-sensitive paths under
+// protectKernelTunables restricts access to some security-sensitive paths under
 // /proc and /sys. Entries are either hidden or just made read-only to app.
-func protectSystemFiles(opts []*unit.UnitOption, appName types.ACName) []*unit.UnitOption {
+// This protection is enabled by default.
+func protectKernelTunables(opts []*unit.UnitOption, appName types.ACName, systemdVersion int) []*unit.UnitOption {
 	roPaths := []string{
 		"/proc/bus/",
 		"/proc/sys/kernel/core_pattern",
@@ -1081,27 +748,38 @@ func protectSystemFiles(opts []*unit.UnitOption, appName types.ACName) []*unit.U
 		"/sys/devices/",
 		"/sys/kernel/",
 	}
-	hiddenPaths := []string{
-		// TODO(lucab): file-paths restrictions need support in systemd first
-		//"/proc/config.gz",
-		//"/proc/kallsyms",
-		//"/proc/sched_debug",
-		//"/proc/kcore",
-		//"/proc/kmem",
-		//"/proc/mem",
+	hiddenDirs := []string{
 		"/sys/firmware/",
 		"/sys/fs/",
 		"/sys/hypervisor/",
 		"/sys/module/",
 		"/sys/power/",
 	}
+	hiddenPaths := []string{
+		"/proc/config.gz",
+		"/proc/kallsyms",
+		"/proc/sched_debug",
+		"/proc/kcore",
+		"/proc/kmem",
+		"/proc/mem",
+	}
+
 	// Paths prefixed with "-" are ignored if they do not exist:
 	// https://www.freedesktop.org/software/systemd/man/systemd.exec.html#ReadWriteDirectories=
-	for _, p := range hiddenPaths {
-		opts = append(opts, unit.NewUnitOption("Service", "InaccessibleDirectories", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
-	}
 	for _, p := range roPaths {
 		opts = append(opts, unit.NewUnitOption("Service", "ReadOnlyDirectories", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
 	}
+	for _, p := range hiddenDirs {
+		opts = append(opts, unit.NewUnitOption("Service", "InaccessibleDirectories", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
+	}
+	if systemdVersion >= 231 {
+		for _, p := range hiddenPaths {
+			opts = append(opts, unit.NewUnitOption("Service", "InaccessiblePaths", fmt.Sprintf("-%s", filepath.Join(common.RelAppRootfsPath(appName), p))))
+		}
+	}
+	if systemdVersion >= 233 {
+		opts = append(opts, unit.NewUnitOption("Service", "ProtectKernelTunables", "true"))
+	}
+
 	return opts
 }
