@@ -28,18 +28,17 @@ import (
 	"github.com/hashicorp/errwrap"
 )
 
-// GetNetworkDescriptions explicitly convert slice of activeNets to slice of netDescribers
-// which is slice required by GetKVMNetArgs
-func GetNetworkDescriptions(n *networking.Networking) []netDescriber {
-	var nds []netDescriber
+// GetNetworkDescriptions converts activeNets to netDescribers
+func GetNetworkDescriptions(n *networking.Networking) []NetDescriber {
+	var nds []NetDescriber
 	for _, an := range n.GetActiveNetworks() {
 		nds = append(nds, an)
 	}
 	return nds
 }
 
-// netDescriber is something that describes network configuration
-type netDescriber interface {
+// NetDescriber is the interface that describes a network configuration
+type NetDescriber interface {
 	GuestIP() net.IP
 	Mask() net.IP
 	IfName() string
@@ -53,7 +52,7 @@ type netDescriber interface {
 // to lkvm tool to configure networks properly.
 // Logic is based on Network configuration extracted from Networking struct
 // and essentially from activeNets that expose netDescriber behavior
-func GetKVMNetArgs(nds []netDescriber) ([]string, error) {
+func GetKVMNetArgs(nds []NetDescriber) ([]string, error) {
 
 	var lkvmArgs []string
 
@@ -103,8 +102,7 @@ func upInterfaceCommand(ifName string) string {
 	return fmt.Sprintf("/bin/ip link set dev %s up", ifName)
 }
 
-func GenerateNetworkInterfaceUnits(unitsPath string, netDescriptions []netDescriber) error {
-
+func GenerateNetworkInterfaceUnits(unitsPath string, netDescriptions []NetDescriber) error {
 	for i, netDescription := range netDescriptions {
 		ifName := fmt.Sprintf(networking.IfNamePattern, i)
 		netAddress := net.IPNet{
