@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#define _GNU_SOURCE
 #include <errno.h>
 #include <string.h>
 #include <stdio.h>
@@ -24,6 +25,7 @@
 #include <sys/vfs.h>
 #include <dirent.h>
 #include <inttypes.h>
+#include <stdbool.h>
 
 #define err_out(_fmt, _args...)						\
 		fprintf(stderr, "Error: " _fmt "\n", ##_args);
@@ -70,64 +72,12 @@ typedef struct _mount_point_t {
 	const char	*type;
 	const char	*options;
 	unsigned long	flags;
+	const bool	skip_if_dst_exists; // Only respected for files_mount_table
 } mount_point;
 
 #define dir(_name, _mode) \
 	{ .name = _name, .mode = _mode }
 
-static int get_machine_name(char *out, int out_len) {
-	int	fd;
-	char	buf[MACHINE_ID_LEN + 1];
-
-	pgoto_if((fd = open("/etc/machine-id", O_RDONLY)) == -1,
-		_fail, "Error opening \"/etc/machine-id\"");
-	pgoto_if(read(fd, buf, MACHINE_ID_LEN) == -1,
-		_fail_fd, "Error reading \"/etc/machine-id\"");
-	pgoto_if(close(fd) != 0,
-		_fail, "Error closing \"/etc/machine-id\"");
-	goto_if(snprintf(out, out_len,
-			"rkt-%.8s-%.4s-%.4s-%.4s-%.12s",
-			buf, buf+8, buf+12, buf+16, buf+20) >= out_len,
-		_fail, "Error constructing machine name");
-
-	return 1;
-
-_fail_fd:
-	close(fd);
-_fail:
-	return 0;
-}
-
-static int ensure_etc_hosts_exists(const char *root, int rootfd) {
-	char	name[MACHINE_NAME_LEN + 1];
-	char	hosts[128];
-	int	fd, len;
-
-	if(faccessat(rootfd, "etc/hosts", F_OK, AT_EACCESS) == 0)
-		return 1;
-
-	goto_if(!get_machine_name(name, sizeof(name)),
-		_fail, "Failed to get machine name");
-	goto_if((len = snprintf(hosts, sizeof(hosts),
-			"%s\t%s\t%s\t%s\n",
-			"127.0.0.1", name,
-			"localhost", "localhost.localdomain")) >= sizeof(hosts),
-		_fail, "/etc/hosts line too long: \"%s\"", hosts);
-	pgoto_if((fd = openat(rootfd, "etc/hosts", O_WRONLY|O_CREAT, 0644)) == -1,
-		_fail, "Failed to create \"%s/etc/hosts\"", root);
-	pgoto_if(write(fd, hosts, len) != len,
-		_fail_fd, "Failed to write \"%s/etc/hosts\"", root);
-	pgoto_if(close(fd) != 0,
-		_fail, "Failed to close \"%s/etc/hosts\"", root);
-
-	return 1;
-
-_fail_fd:
-	close(fd);
-_fail:
-	return 0;
-}
-
 static void mount_at(const char *root, const mount_point *mnt)
 {
 	char to[4096];
@@ -138,11 +88,50 @@ static void mount_at(const char *root, const mount_point *mnt)
 		 "Mounting \"%s\" on \"%s\" failed", mnt->source, to);
 }
 
+static int mount_sys_required(const char *root)
+{
+	FILE *f;
+	char *line = NULL;
+	size_t len = 0;
+	ssize_t read;
+
+	pexit_if((f = fopen("/proc/self/mountinfo", "re")) == NULL,
+		 "Unable to open /proc/self/mountinfo");
+
+	while ((read = getline(&line, &len, f)) != -1) {
+		char *sys_dir;
+		char *sys_subdir;
+		char *mountpoint;
+
+		exit_if(asprintf(&sys_dir, "%s/sys", root) == -1,
+			"Calling asprintf failed");
+		exit_if(asprintf(&sys_subdir, "%s/sys/", root) == -1,
+			"Calling asprintf failed");
+		sscanf(line, "%*s %*s %*s %*s %ms", &mountpoint);
+
+		// The mount point is exactly $ROOTFS/sys
+		if (strcmp(sys_dir, mountpoint) == 0) {
+			free(mountpoint);
+			return 0;
+		}
+		// The mount point is a subdirectory of $ROOTFS/sys
+		if (strncmp(sys_subdir, mountpoint, strlen(sys_subdir)) == 0) {
+			free(mountpoint);
+			return 0;
+		}
+
+		free(mountpoint);
+	}
+
+	pexit_if(fclose(f) != 0, "Unable to close /proc/self/mountinfo");
+
+	return 1;
+}
 static void mount_sys(const char *root)
 {
 	struct statfs fs;
-	const mount_point sys_bind_rec = { "/sys", "sys", "bind", NULL, MS_BIND|MS_REC };
-	const mount_point sys_bind = { "/sys", "sys", "bind", NULL, MS_BIND };
+	const mount_point sys_bind_rec = { "/sys", "sys", "bind", NULL, MS_BIND|MS_REC, false };
+	const mount_point sys_bind = { "/sys", "sys", "bind", NULL, MS_BIND, false };
 
 	pexit_if(statfs("/sys/fs/cgroup", &fs) != 0,
 	         "Cannot statfs /sys/fs/cgroup");
@@ -190,6 +179,47 @@ static void mount_sys(const char *root)
 	mount_at(root, &sys_bind);
 }
 
+static void copy_volume_symlinks()
+{
+	DIR *volumes_dir;
+	struct dirent *de;
+	const char *rkt_volume_links_path = "/rkt/volumes";
+	const char *dev_rkt_path = "/dev/.rkt";
+
+	pexit_if(mkdir(dev_rkt_path, 0700) == -1 && errno != EEXIST,
+		"Failed to create directory \"%s\"", dev_rkt_path);
+
+	pexit_if((volumes_dir = opendir(rkt_volume_links_path)) == NULL && errno != ENOENT,
+                 "Failed to open directory \"%s\"", rkt_volume_links_path);
+	while (volumes_dir) {
+		errno = 0;
+		if ((de = readdir(volumes_dir)) != NULL) {
+			char *link_path;
+			char *new_link;
+			char target[4096] = {0,};
+
+			if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, ".."))
+			  continue;
+
+			exit_if(asprintf(&link_path, "%s/%s", rkt_volume_links_path, de->d_name) == -1,
+				"Calling asprintf failed");
+			exit_if(asprintf(&new_link, "%s/%s", dev_rkt_path, de->d_name) == -1,
+				"Calling asprintf failed");
+
+			pexit_if(readlink(link_path, target, sizeof(target)) == -1,
+				 "Error reading \"%s\" link", link_path);
+			pexit_if(symlink(target, new_link) == -1 && errno != EEXIST,
+				"Failed to create volume symlink \"%s\"", new_link);
+		} else {
+			pexit_if(errno != 0,
+				"Error reading \"%s\" directory", rkt_volume_links_path);
+			pexit_if(closedir(volumes_dir),
+				 "Error closing \"%s\" directory", rkt_volume_links_path);
+			return;
+		}
+	}
+}
+
 int main(int argc, char *argv[])
 {
 	static const char *unlink_paths[] = {
@@ -222,15 +252,19 @@ int main(int argc, char *argv[])
 		NULL
 	};
 	static const mount_point dirs_mount_table[] = {
-		{ "/proc", "/proc", "bind", NULL, MS_BIND|MS_REC },
-		{ "/dev/shm", "/dev/shm", "bind", NULL, MS_BIND },
-		{ "/dev/pts", "/dev/pts", "bind", NULL, MS_BIND },
-		{ "/run/systemd/journal", "/run/systemd/journal", "bind", NULL, MS_BIND },
+		{ "/proc", "/proc", "bind", NULL, MS_BIND|MS_REC, false },
+		{ "/dev/shm", "/dev/shm", "bind", NULL, MS_BIND, false },
+		{ "/dev/pts", "/dev/pts", "bind", NULL, MS_BIND, false },
+		{ "/run/systemd/journal", "/run/systemd/journal", "bind", NULL, MS_BIND, false },
 		/* /sys is handled separately */
 	};
 	static const mount_point files_mount_table[] = {
-		{ "/etc/rkt-resolv.conf", "/etc/resolv.conf", "bind", NULL, MS_BIND },
-		{ "/proc/sys/kernel/hostname", "/etc/hostname", "bind", NULL, MS_BIND },
+		{ "/etc/rkt-resolv.conf", "/etc/resolv.conf", "bind", NULL, MS_BIND, false },
+		{ "/etc/rkt-hosts", "/etc/hosts", "bind", NULL, MS_BIND, false },
+		{ "/etc/hosts-fallback", "/etc/hosts", "bind", NULL, MS_BIND, true }, // only create as fallback
+		{ "/proc/sys/kernel/hostname", "/etc/hostname", "bind", NULL, MS_BIND, false },
+		// TODO @alepuccetti this could be removed when https://github.com/systemd/systemd/issues/3544 is solved
+		{ "/run/systemd/notify", "/run/systemd/notify", "bind", NULL, MS_BIND, false },
 	};
 	const char *root;
 	int rootfd;
@@ -275,9 +309,6 @@ int main(int argc, char *argv[])
 			"Failed to create directory \"%s/%s\"", root, d->name);
 	}
 
-	exit_if(!ensure_etc_hosts_exists(root, rootfd),
-		"Failed to ensure \"%s/etc/hosts\" exists", root);
-
 	close(rootfd);
 
 	/* systemd-nspawn already creates few /dev entries in the container
@@ -327,9 +358,11 @@ int main(int argc, char *argv[])
 	}
 
 	/* Bind mount /sys: handled differently, depending on cgroups */
-	mount_sys(root);
+	if (mount_sys_required(root))
+		mount_sys(root);
 
-	/* Bind mount files, if the source exists */
+	/* Bind mount files, if the source exists.
+	 * By default, overwrite dst unless skip_if_dst_exists is true. */
 	for (i = 0; i < nelems(files_mount_table); i++) {
 		const mount_point *mnt = &files_mount_table[i];
 		int fd;
@@ -338,6 +371,8 @@ int main(int argc, char *argv[])
 			"Path too long: \"%s\"", to);
 		if (access(mnt->source, F_OK) != 0)
 			continue;
+		if( mnt->skip_if_dst_exists && access(to, F_OK) == 0)
+			continue;
 		if (access(to, F_OK) != 0) {
 			pexit_if((fd = creat(to, 0644)) == -1,
 				"Cannot create file: \"%s\"", to);
@@ -355,6 +390,11 @@ int main(int argc, char *argv[])
 	pexit_if(symlink("/dev/pts/ptmx", to) == -1 && errno != EEXIST,
 		"Failed to create /dev/ptmx symlink");
 
+	/* Copy symlinks to device node volumes to "/dev/.rkt" so they can be
+	 * used in the DeviceAllow= option of the app's unit file (systemd
+	 * needs the path to start with "/dev". */
+	copy_volume_symlinks();
+
 	/* /dev/log -> /run/systemd/journal/dev-log */
 	exit_if(snprintf(to, sizeof(to), "%s/dev/log", root) >= sizeof(to),
 		"Path too long: \"%s\"", to);
