@@ -19,21 +19,20 @@ import (
 	"io/ioutil"
 	"os"
 
+	"github.com/coreos/rkt/common"
 	rktlog "github.com/coreos/rkt/pkg/log"
 )
 
-const (
-	mountinfoPath = "/proc/self/mountinfo"
-)
-
 var (
 	debug bool
 
-	diag *rktlog.Logger
+	diag        *rktlog.Logger
+	localConfig string
 )
 
 func init() {
 	flag.BoolVar(&debug, "debug", false, "Run in debug mode")
+	flag.StringVar(&localConfig, "local-config", common.DefaultLocalConfigDir, "Local config path (ignored)")
 }
 
 func main() {
