@@ -15,7 +15,6 @@
 package main
 
 import (
-	"bufio"
 	"errors"
 	"flag"
 	"fmt"
@@ -31,7 +30,10 @@ import (
 	"github.com/hashicorp/errwrap"
 
 	"github.com/coreos/rkt/common"
+	"github.com/coreos/rkt/pkg/fileutil"
+	pkgflag "github.com/coreos/rkt/pkg/flag"
 	rktlog "github.com/coreos/rkt/pkg/log"
+	"github.com/coreos/rkt/pkg/mountinfo"
 	"github.com/coreos/rkt/pkg/sys"
 	"github.com/coreos/rkt/pkg/user"
 	stage1common "github.com/coreos/rkt/stage1/common"
@@ -66,59 +68,64 @@ var (
 	diag *rktlog.Logger
 )
 
-func getHostMounts() (map[string]struct{}, error) {
-	hostMounts := map[string]struct{}{}
+func parseFlags() *stage1commontypes.RuntimePod {
+	rp := stage1commontypes.RuntimePod{}
 
-	mi, err := os.Open("/proc/self/mountinfo")
-	if err != nil {
-		return nil, err
-	}
-	defer mi.Close()
-
-	sc := bufio.NewScanner(mi)
-	for sc.Scan() {
-		var (
-			discard    string
-			mountPoint string
-		)
-
-		_, err := fmt.Sscanf(sc.Text(),
-			"%s %s %s %s %s",
-			&discard, &discard, &discard, &discard, &mountPoint,
-		)
-		if err != nil {
-			return nil, err
-		}
-
-		hostMounts[mountPoint] = struct{}{}
-	}
-	if sc.Err() != nil {
-		return nil, errwrap.Wrap(errors.New("problem parsing mountinfo"), sc.Err())
-	}
-	return hostMounts, nil
-}
-
-func init() {
 	flag.BoolVar(&debug, "debug", false, "Run in debug mode")
 
 	// The following flags need to be supported by stage1 according to
 	// https://github.com/coreos/rkt/blob/master/Documentation/devel/stage1-implementors-guide.md
-	// TODO: either implement functionality or give not implemented warnings
+	// Most of them are ignored
+	// These are ignored, but stage0 always passes them
 	flag.Var(&discardNetlist, "net", "Setup networking")
-	flag.BoolVar(&discardBool, "interactive", true, "The pod is interactive")
-	flag.StringVar(&discardString, "mds-token", "", "MDS auth token")
 	flag.StringVar(&discardString, "local-config", common.DefaultLocalConfigDir, "Local config path")
+
+	// These are discarded with a warning
+	// TODO either implement these, or stop passing them
+	flag.Bool("interactive", true, "The pod is interactive (ignored, always true)")
+	flag.Var(pkgflag.NewDiscardFlag("mds-token"), "mds-token", "MDS auth token (not implemented)")
+
+	flag.Var(pkgflag.NewDiscardFlag("hostname"), "hostname", "Set hostname (not implemented)")
+	flag.Bool("disable-capabilities-restriction", true, "ignored")
+	flag.Bool("disable-paths", true, "ignored")
+	flag.Bool("disable-seccomp", true, "ignored")
+
+	dnsConfMode := pkgflag.MustNewPairList(map[string][]string{
+		"resolv": {"host", "stage0", "none", "default"},
+		"hosts":  {"host", "stage0", "default"},
+	}, map[string]string{
+		"resolv": "default",
+		"hosts":  "default",
+	})
+	flag.Var(dnsConfMode, "dns-conf-mode", "DNS config file modes")
+
+	flag.Parse()
+
+	rp.Debug = debug
+	rp.ResolvConfMode = dnsConfMode.Pairs["resolv"]
+	rp.EtcHostsMode = dnsConfMode.Pairs["hosts"]
+
+	return &rp
+}
+
+func addMountPoints(namedVolumeMounts map[types.ACName]volumeMountTuple, mountpoints []types.MountPoint) error {
+	for _, mp := range mountpoints {
+		tuple, exists := namedVolumeMounts[mp.Name]
+		switch {
+		case exists && tuple.M.Path != mp.Path:
+			return fmt.Errorf("conflicting path information from mount and mountpoint %q", mp.Name)
+		case !exists:
+			namedVolumeMounts[mp.Name] = volumeMountTuple{M: schema.Mount{Volume: mp.Name, Path: mp.Path}}
+			diag.Printf("adding %+v", namedVolumeMounts[mp.Name])
+		}
+	}
+	return nil
 }
 
 func evaluateMounts(rfs string, app string, p *stage1commontypes.Pod) ([]flyMount, error) {
-	imApp := p.Images[app].App
 	namedVolumeMounts := map[types.ACName]volumeMountTuple{}
 
-	var manifestMPs []types.MountPoint
-	if imApp != nil {
-		manifestMPs = imApp.MountPoints
-	}
-
+	// Insert the PodManifest's first RuntimeApp's Mounts
 	for _, m := range p.Manifest.Apps[0].Mounts {
 		_, exists := namedVolumeMounts[m.Volume]
 		if exists {
@@ -129,17 +136,20 @@ func evaluateMounts(rfs string, app string, p *stage1commontypes.Pod) ([]flyMoun
 	}
 
 	// Merge command-line Mounts with ImageManifest's MountPoints
-	for _, mp := range manifestMPs {
-		tuple, exists := namedVolumeMounts[mp.Name]
-		switch {
-		case exists && tuple.M.Path != mp.Path:
-			return nil, fmt.Errorf("conflicting path information from mount and mountpoint %q", mp.Name)
-		case !exists:
-			namedVolumeMounts[mp.Name] = volumeMountTuple{M: schema.Mount{Volume: mp.Name, Path: mp.Path}}
-			diag.Printf("adding %+v", namedVolumeMounts[mp.Name])
+	var imAppManifestMPs []types.MountPoint
+	if imApp := p.Images[app].App; imApp != nil {
+		imAppManifestMPs = imApp.MountPoints
+		if err := addMountPoints(namedVolumeMounts, imAppManifestMPs); err != nil {
+			return nil, err
 		}
 	}
 
+	// Merge command-line Mounts with PodManifest's RuntimeApp's App's MountPoints
+	raApp := p.Manifest.Apps[0]
+	if err := addMountPoints(namedVolumeMounts, raApp.App.MountPoints); err != nil {
+		return nil, err
+	}
+
 	// Insert the command-line Volumes
 	for _, v := range p.Manifest.Volumes {
 		// Check if we have a mount for this volume
@@ -155,7 +165,7 @@ func evaluateMounts(rfs string, app string, p *stage1commontypes.Pod) ([]flyMoun
 	}
 
 	// Merge command-line Volumes with ImageManifest's MountPoints
-	for _, mp := range manifestMPs {
+	for _, mp := range imAppManifestMPs {
 		// Check if we have a volume for this mountpoint
 		tuple, exists := namedVolumeMounts[mp.Name]
 		if !exists || tuple.V.Name == "" {
@@ -172,13 +182,16 @@ func evaluateMounts(rfs string, app string, p *stage1commontypes.Pod) ([]flyMoun
 	}
 
 	// Gather host mounts which we make MS_SHARED if passed as a volume source
-	hostMounts, err := getHostMounts()
+	hostMounts := map[string]struct{}{}
+	mnts, err := mountinfo.ParseMounts(0)
 	if err != nil {
 		return nil, errwrap.Wrap(errors.New("can't gather host mounts"), err)
 	}
+	for _, m := range mnts {
+		hostMounts[m.MountPoint] = struct{}{}
+	}
 
 	argFlyMounts := []flyMount{}
-	var flags uintptr = syscall.MS_BIND // TODO: allow optional | syscall.MS_REC
 	for _, tuple := range namedVolumeMounts {
 		if _, isHostMount := hostMounts[tuple.V.Source]; isHostMount {
 			// Mark the host mount as SHARED so the container's changes to the mount are propagated to the host
@@ -186,37 +199,69 @@ func evaluateMounts(rfs string, app string, p *stage1commontypes.Pod) ([]flyMoun
 				flyMount{"", "", tuple.V.Source, "none", syscall.MS_REC | syscall.MS_SHARED},
 			)
 		}
+
+		var (
+			flags     uintptr = syscall.MS_BIND
+			recursive         = tuple.V.Recursive != nil && *tuple.V.Recursive
+			ro                = tuple.V.ReadOnly != nil && *tuple.V.ReadOnly
+		)
+
+		// If Recursive is not set, default to non-recursive.
+		if recursive {
+			flags |= syscall.MS_REC
+		}
+
 		argFlyMounts = append(argFlyMounts,
 			flyMount{tuple.V.Source, rfs, tuple.M.Path, "none", flags},
 		)
 
-		if tuple.V.ReadOnly != nil && *tuple.V.ReadOnly {
+		if ro {
 			argFlyMounts = append(argFlyMounts,
 				flyMount{"", rfs, tuple.M.Path, "none", flags | syscall.MS_REMOUNT | syscall.MS_RDONLY},
 			)
+
+			if recursive {
+				// Every sub-mount needs to be remounted read-only separately
+				mnts, err := mountinfo.ParseMounts(0)
+				if err != nil {
+					return nil, errwrap.Wrap(fmt.Errorf("error getting mounts under %q from mountinfo", tuple.V.Source), err)
+				}
+				mnts = mnts.Filter(mountinfo.HasPrefix(tuple.V.Source + "/"))
+
+				for _, mnt := range mnts {
+					innerRelPath := tuple.M.Path + strings.Replace(mnt.MountPoint, tuple.V.Source, "", -1)
+					argFlyMounts = append(argFlyMounts,
+						flyMount{"", rfs, innerRelPath, "none", flags | syscall.MS_REMOUNT | syscall.MS_RDONLY},
+					)
+				}
+			}
 		}
 	}
 	return argFlyMounts, nil
 }
 
-func stage1() int {
+func stage1(rp *stage1commontypes.RuntimePod) int {
 	uuid, err := types.NewUUID(flag.Arg(0))
 	if err != nil {
 		log.Print("UUID is missing or malformed\n")
-		return 1
+		return 254
 	}
 
 	root := "."
-	p, err := stage1commontypes.LoadPod(root, uuid)
+	p, err := stage1commontypes.LoadPod(root, uuid, rp)
 	if err != nil {
 		log.PrintE("can't load pod", err)
-		return 1
+		return 254
+	}
+
+	if err := p.SaveRuntime(); err != nil {
+		log.FatalE("failed to save runtime parameters", err)
 	}
 
 	// Sanity checks
 	if len(p.Manifest.Apps) != 1 {
 		log.Printf("flavor %q only supports 1 application per Pod for now", flavor)
-		return 1
+		return 254
 	}
 
 	ra := p.Manifest.Apps[0]
@@ -225,19 +270,19 @@ func stage1() int {
 	args := ra.App.Exec
 	if len(args) == 0 {
 		log.Printf(`image %q has an empty "exec" (try --exec=BINARY)`, imgName)
-		return 1
+		return 254
 	}
 
 	lfd, err := common.GetRktLockFD()
 	if err != nil {
 		log.PrintE("can't get rkt lock fd", err)
-		return 1
+		return 254
 	}
 
 	// set close-on-exec flag on RKT_LOCK_FD so it gets correctly closed after execution is finished
 	if err := sys.CloseOnExec(lfd, true); err != nil {
 		log.PrintE("can't set FD_CLOEXEC on rkt lock", err)
-		return 1
+		return 254
 	}
 
 	workDir := "/"
@@ -255,7 +300,7 @@ func stage1() int {
 	argFlyMounts, err := evaluateMounts(rfs, string(ra.Name), p)
 	if err != nil {
 		log.PrintE("can't evaluate mounts", err)
-		return 1
+		return 254
 	}
 
 	effectiveMounts := append(
@@ -274,7 +319,54 @@ func stage1() int {
 		argFlyMounts...,
 	)
 
+	/* Process DNS config files
+	 *
+	 * /etc/resolv.conf: four modes
+	 * 'host' - bind-mount host's file
+	 * 'stage0' - bind-mount the file created by stage0
+	 * 'default' - do nothing (we would respect CNI if fly had networking)
+	 * 'none' - do nothing
+	 */
+	switch p.ResolvConfMode {
+	case "host":
+		effectiveMounts = append(effectiveMounts,
+			flyMount{"/etc/resolv.conf", rfs, "/etc/resolv.conf", "none", syscall.MS_BIND | syscall.MS_RDONLY})
+	case "stage0":
+		if err := copyResolv(p); err != nil {
+			log.PrintE("can't copy /etc/resolv.conf", err)
+			return 254
+		}
+	}
+
+	/*
+	 * /etc/hosts: three modes:
+	 * 'host' - bind-mount hosts's file
+	 * 'stage0' - bind mount the file created by stage1
+	 * 'default' - create a stub /etc/hosts if needed
+	 */
+
+	switch p.EtcHostsMode {
+	case "host":
+		effectiveMounts = append(effectiveMounts,
+			flyMount{"/etc/hosts", rfs, "/etc/hosts", "none", syscall.MS_BIND | syscall.MS_RDONLY})
+	case "stage0":
+		effectiveMounts = append(effectiveMounts, flyMount{
+			filepath.Join(common.Stage1RootfsPath(p.Root), "etc", "rkt-hosts"),
+			rfs,
+			"/etc/hosts",
+			"none",
+			syscall.MS_BIND | syscall.MS_RDONLY})
+	case "default":
+		stage2HostsPath := filepath.Join(common.AppRootfsPath(p.Root, ra.Name), "etc", "hosts")
+		if _, err := os.Stat(stage2HostsPath); err != nil && os.IsNotExist(err) {
+			fallbackHosts := []byte("127.0.0.1 localhost localdomain\n")
+			ioutil.WriteFile(stage2HostsPath, fallbackHosts, 0644)
+		}
+	}
+
 	for _, mount := range effectiveMounts {
+		diag.Printf("Processing %+v", mount)
+
 		var (
 			err            error
 			hostPathInfo   os.FileInfo
@@ -284,7 +376,7 @@ func stage1() int {
 		if strings.HasPrefix(mount.HostPath, "/") {
 			if hostPathInfo, err = os.Stat(mount.HostPath); err != nil {
 				log.PrintE(fmt.Sprintf("stat of host path %s", mount.HostPath), err)
-				return 1
+				return 254
 			}
 		} else {
 			hostPathInfo = nil
@@ -293,27 +385,31 @@ func stage1() int {
 		absTargetPath := filepath.Join(mount.TargetPrefixPath, mount.RelTargetPath)
 		if targetPathInfo, err = os.Stat(absTargetPath); err != nil && !os.IsNotExist(err) {
 			log.PrintE(fmt.Sprintf("stat of target path %s", absTargetPath), err)
-			return 1
+			return 254
 		}
 
 		switch {
+		case (mount.Flags & syscall.MS_REMOUNT) != 0:
+			{
+				diag.Printf("don't attempt to create files for remount of %q", absTargetPath)
+			}
 		case targetPathInfo == nil:
 			absTargetPathParent, _ := filepath.Split(absTargetPath)
 			if err := os.MkdirAll(absTargetPathParent, 0755); err != nil {
 				log.PrintE(fmt.Sprintf("can't create directory %q", absTargetPath), err)
-				return 1
+				return 254
 			}
 			switch {
 			case hostPathInfo == nil || hostPathInfo.IsDir():
 				if err := os.Mkdir(absTargetPath, 0755); err != nil {
 					log.PrintE(fmt.Sprintf("can't create directory %q", absTargetPath), err)
-					return 1
+					return 254
 				}
 			case !hostPathInfo.IsDir():
 				file, err := os.OpenFile(absTargetPath, os.O_CREATE, 0700)
 				if err != nil {
 					log.PrintE(fmt.Sprintf("can't create file %q", absTargetPath), err)
-					return 1
+					return 254
 				}
 				file.Close()
 			}
@@ -321,22 +417,22 @@ func stage1() int {
 			switch {
 			case hostPathInfo.IsDir() && !targetPathInfo.IsDir():
 				log.Printf("can't mount because %q is a directory while %q is not", mount.HostPath, absTargetPath)
-				return 1
+				return 254
 			case !hostPathInfo.IsDir() && targetPathInfo.IsDir():
 				log.Printf("can't mount because %q is not a directory while %q is", mount.HostPath, absTargetPath)
-				return 1
+				return 254
 			}
 		}
 
 		if err := syscall.Mount(mount.HostPath, absTargetPath, mount.Fs, mount.Flags, ""); err != nil {
 			log.PrintE(fmt.Sprintf("can't mount %q on %q with flags %v", mount.HostPath, absTargetPath, mount.Flags), err)
-			return 1
+			return 254
 		}
 	}
 
 	if err = stage1common.WritePid(os.Getpid(), "pid"); err != nil {
 		log.Error(err)
-		return 1
+		return 254
 	}
 
 	var uidResolver, gidResolver user.Resolver
@@ -349,12 +445,12 @@ func stage1() int {
 
 	if err != nil { // give up
 		log.PrintE(fmt.Sprintf("invalid user %q", ra.App.User), err)
-		return 1
+		return 254
 	}
 
 	if uid, _, err = uidResolver.IDs(); err != nil {
 		log.PrintE(fmt.Sprintf("failed to configure user %q", ra.App.User), err)
-		return 1
+		return 254
 	}
 
 	gidResolver, err = user.NumericIDs(ra.App.Group)
@@ -364,23 +460,23 @@ func stage1() int {
 
 	if err != nil { // give up
 		log.PrintE(fmt.Sprintf("invalid group %q", ra.App.Group), err)
-		return 1
+		return 254
 	}
 
 	if _, gid, err = gidResolver.IDs(); err != nil {
 		log.PrintE(fmt.Sprintf("failed to configure group %q", ra.App.Group), err)
-		return 1
+		return 254
 	}
 
 	diag.Printf("chroot to %q", rfs)
 	if err := syscall.Chroot(rfs); err != nil {
 		log.PrintE("can't chroot", err)
-		return 1
+		return 254
 	}
 
 	if err := os.Chdir(workDir); err != nil {
 		log.PrintE(fmt.Sprintf("can't change to working directory %q", workDir), err)
-		return 1
+		return 254
 	}
 
 	// lock the current goroutine to its current OS thread.
@@ -392,12 +488,12 @@ func stage1() int {
 
 	if err := syscall.Setresgid(gid, gid, gid); err != nil {
 		log.PrintE(fmt.Sprintf("can't set gid %d", gid), err)
-		return 1
+		return 254
 	}
 
 	if err := syscall.Setresuid(uid, uid, uid); err != nil {
 		log.PrintE(fmt.Sprintf("can't set uid %d", uid), err)
-		return 1
+		return 254
 	}
 
 	diag.Printf("execing %q in %q", args, rfs)
@@ -406,14 +502,40 @@ func stage1() int {
 	})
 	if err != nil {
 		log.PrintE(fmt.Sprintf("can't execute %q", args[0]), err)
-		return 1
+		return 254
 	}
 
 	return 0
 }
 
+func copyResolv(p *stage1commontypes.Pod) error {
+	ra := p.Manifest.Apps[0]
+
+	stage1Rootfs := common.Stage1RootfsPath(p.Root)
+	resolvPath := filepath.Join(stage1Rootfs, "etc", "rkt-resolv.conf")
+
+	appRootfs := common.AppRootfsPath(p.Root, ra.Name)
+	targetEtc := filepath.Join(appRootfs, "etc")
+	targetResolvPath := filepath.Join(targetEtc, "resolv.conf")
+
+	_, err := os.Stat(resolvPath)
+	switch {
+	case os.IsNotExist(err):
+		return nil
+	case err != nil:
+		return err
+	}
+
+	_, err = os.Stat(targetResolvPath)
+	if err != nil && !os.IsNotExist(err) {
+		return err
+	}
+
+	return fileutil.CopyRegularFile(resolvPath, targetResolvPath)
+}
+
 func main() {
-	flag.Parse()
+	rp := parseFlags()
 
 	log, diag, _ = rktlog.NewLogSet("run", debug)
 	if !debug {
@@ -421,5 +543,5 @@ func main() {
 	}
 
 	// move code into stage1() helper so defered fns get run
-	os.Exit(stage1())
+	os.Exit(stage1(rp))
 }
