@@ -17,6 +17,7 @@ package main
 import (
 	"bufio"
 	"bytes"
+	"crypto/sha1"
 	"flag"
 	"fmt"
 	"io/ioutil"
@@ -32,7 +33,10 @@ import (
 
 	"golang.org/x/sys/unix"
 
+	"github.com/appc/spec/pkg/device"
 	"github.com/coreos/rkt/common/cgroup"
+	"github.com/coreos/rkt/common/cgroup/v1"
+	"github.com/coreos/rkt/common/cgroup/v2"
 	"github.com/coreos/rkt/tests/testutils"
 	"github.com/syndtr/gocapability/capability"
 )
@@ -42,6 +46,7 @@ var (
 	globalFlags   = struct {
 		ReadStdin          bool
 		CheckTty           bool
+		CheckPath          bool
 		PrintExec          bool
 		PrintMsg           string
 		SuffixMsg          string
@@ -54,10 +59,13 @@ var (
 		ReadFile           bool
 		WriteFile          bool
 		StatFile           bool
+		HashFile           bool
+		FileSymlinkTarget  bool
 		Sleep              int
 		PreSleep           int
 		PrintMemoryLimit   bool
 		PrintCPUQuota      bool
+		PrintCPUShares     bool
 		FileName           string
 		Content            string
 		CheckCgroupMounts  bool
@@ -77,12 +85,14 @@ var (
 		SilentSigterm      bool
 		CheckMountNS       bool
 		PrintNoNewPrivs    bool
+		CheckMknod         string
 	}{}
 )
 
 func init() {
 	globalFlagset.BoolVar(&globalFlags.ReadStdin, "read-stdin", false, "Read a line from stdin")
 	globalFlagset.BoolVar(&globalFlags.CheckTty, "check-tty", false, "Check if stdin is a terminal")
+	globalFlagset.BoolVar(&globalFlags.CheckPath, "check-path", false, "Check if environment variable PATH does not contain \\n")
 	globalFlagset.BoolVar(&globalFlags.PrintExec, "print-exec", false, "Print the command we were execed as (i.e. argv[0])")
 	globalFlagset.StringVar(&globalFlags.PrintMsg, "print-msg", "", "Print the message given as parameter")
 	globalFlagset.StringVar(&globalFlags.SuffixMsg, "suffix-msg", "", "Print this suffix after some commands")
@@ -95,10 +105,13 @@ func init() {
 	globalFlagset.BoolVar(&globalFlags.ReadFile, "read-file", false, "Print the content of the file $FILE")
 	globalFlagset.BoolVar(&globalFlags.WriteFile, "write-file", false, "Write $CONTENT in the file $FILE")
 	globalFlagset.BoolVar(&globalFlags.StatFile, "stat-file", false, "Print the ownership and mode of the file $FILE")
+	globalFlagset.BoolVar(&globalFlags.HashFile, "hash-file", false, "Print the SHA1SUM of the file $FILE")
+	globalFlagset.BoolVar(&globalFlags.FileSymlinkTarget, "file-symlink-target", false, "Print the symlink target of $FILE")
 	globalFlagset.IntVar(&globalFlags.Sleep, "sleep", -1, "Sleep before exiting (in seconds)")
 	globalFlagset.IntVar(&globalFlags.PreSleep, "pre-sleep", -1, "Sleep before executing (in seconds)")
 	globalFlagset.BoolVar(&globalFlags.PrintMemoryLimit, "print-memorylimit", false, "Print cgroup memory limit")
 	globalFlagset.BoolVar(&globalFlags.PrintCPUQuota, "print-cpuquota", false, "Print cgroup cpu quota in milli-cores")
+	globalFlagset.BoolVar(&globalFlags.PrintCPUShares, "print-cpushares", false, "Print cgroup cpu shares")
 	globalFlagset.StringVar(&globalFlags.FileName, "file-name", "", "The file to read/write, $FILE will be ignored if this is specified")
 	globalFlagset.StringVar(&globalFlags.Content, "content", "", "The content to write, $CONTENT will be ignored if this is specified")
 	globalFlagset.BoolVar(&globalFlags.CheckCgroupMounts, "check-cgroups", false, "Try to write to the cgroup filesystem. Everything should be RO except some well-known files")
@@ -118,6 +131,7 @@ func init() {
 	globalFlagset.BoolVar(&globalFlags.SilentSigterm, "silent-sigterm", false, "Exit with a success exit status if we receive SIGTERM")
 	globalFlagset.BoolVar(&globalFlags.CheckMountNS, "check-mountns", false, "Check if app's mount ns is different than stage1's. Requires CAP_SYS_PTRACE")
 	globalFlagset.BoolVar(&globalFlags.PrintNoNewPrivs, "print-no-new-privs", false, "print the prctl PR_GET_NO_NEW_PRIVS value")
+	globalFlagset.StringVar(&globalFlags.CheckMknod, "check-mknod", "", "check whether mknod on restricted devices is allowed")
 }
 
 func in(list []int, el int) bool {
@@ -134,17 +148,7 @@ func main() {
 	args := globalFlagset.Args()
 	if len(args) > 0 {
 		fmt.Fprintln(os.Stderr, "Wrong parameters")
-		os.Exit(1)
-	}
-
-	if globalFlags.PrintNoNewPrivs {
-		r1, _, err := syscall.Syscall(
-			syscall.SYS_PRCTL,
-			uintptr(unix.PR_GET_NO_NEW_PRIVS),
-			uintptr(0), uintptr(0),
-		)
-
-		fmt.Printf("no_new_privs: %v err: %v\n", r1, err)
+		os.Exit(254)
 	}
 
 	if globalFlags.SilentSigterm {
@@ -167,6 +171,57 @@ func main() {
 		fmt.Printf("Received text: %s\n", text)
 	}
 
+	if globalFlags.PrintNoNewPrivs {
+		r1, _, err := syscall.Syscall(
+			syscall.SYS_PRCTL,
+			uintptr(unix.PR_GET_NO_NEW_PRIVS),
+			uintptr(0), uintptr(0),
+		)
+
+		fmt.Printf("no_new_privs: %v err: %v\n", r1, err)
+	}
+
+	if globalFlags.CheckMknod != "" {
+		/* format: c:5:2:name */
+		dev := strings.SplitN(globalFlags.CheckMknod, ":", 4)
+		if len(dev) < 4 {
+			fmt.Fprintln(os.Stderr, "Not enough parameters for mknod")
+			os.Exit(254)
+		}
+		typ := dev[0]
+		major, err := strconv.Atoi(dev[1])
+		if err != nil {
+			fmt.Fprintln(os.Stderr, "Wrong major")
+			os.Exit(254)
+		}
+		minor, err := strconv.Atoi(dev[2])
+		if err != nil {
+			fmt.Fprintln(os.Stderr, "Wrong minor")
+			os.Exit(254)
+		}
+		nodeName := dev[3]
+
+		majorMinor := device.Makedev(uint(major), uint(minor))
+		mode := uint32(0777)
+		switch typ {
+		case "c":
+			mode |= syscall.S_IFCHR
+		case "b":
+			mode |= syscall.S_IFBLK
+		default:
+			fmt.Fprintln(os.Stderr, "Wrong device node type")
+			os.Exit(254)
+		}
+
+		if err := syscall.Mknod(nodeName, mode, int(majorMinor)); err != nil {
+			fmt.Fprintf(os.Stderr, "mknod %s: fail: %v\n", nodeName, err)
+			os.Exit(254)
+		} else {
+			fmt.Printf("mknod %s: succeed\n", nodeName)
+			os.Exit(0)
+		}
+	}
+
 	if globalFlags.CheckTty {
 		fd := int(os.Stdin.Fd())
 		var termios syscall.Termios
@@ -178,6 +233,32 @@ func main() {
 		}
 	}
 
+	if globalFlags.CheckPath {
+		envBytes, err := ioutil.ReadFile("/proc/self/environ")
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Error reading environment from \"/proc/self/environ\": %v\n", err)
+			os.Exit(254)
+		}
+		for _, v := range bytes.Split(envBytes, []byte{0}) {
+			if len(v) == 0 {
+				continue
+			}
+			if strings.HasPrefix(string(v), "PATH=") {
+				if strings.Contains(string(v), "\n") {
+					fmt.Fprintf(os.Stderr, "Malformed PATH: found new line")
+					os.Exit(254)
+				} else {
+					fmt.Printf("PATH is good\n")
+					os.Exit(0)
+				}
+			} else {
+				continue
+			}
+		}
+		fmt.Fprintf(os.Stderr, "PATH not found")
+		os.Exit(254)
+	}
+
 	if globalFlags.PrintExec {
 		fmt.Fprintf(os.Stdout, "inspect execed as: %s\n", os.Args[0])
 	}
@@ -202,7 +283,7 @@ func main() {
 		caps, err := capability.NewPid(globalFlags.PrintCapsPid)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Cannot get caps: %v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("Capability set: effective: %s (%s)\n", caps.StringCap(capability.EFFECTIVE), globalFlags.SuffixMsg)
 		fmt.Printf("Capability set: permitted: %s (%s)\n", caps.StringCap(capability.PERMITTED), globalFlags.SuffixMsg)
@@ -213,7 +294,7 @@ func main() {
 			capInt, err := strconv.Atoi(capStr)
 			if err != nil {
 				fmt.Fprintf(os.Stderr, "Environment variable $CAPABILITY is not a valid capability number: %v\n", err)
-				os.Exit(1)
+				os.Exit(254)
 			}
 			c := capability.Cap(capInt)
 			if caps.Get(capability.BOUNDING, c) {
@@ -232,7 +313,7 @@ func main() {
 		gids, err := os.Getgroups()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Error getting groups: %v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		// getgroups(2): It is unspecified whether the effective group ID of
 		// the calling process is included in the returned list. (Thus, an
@@ -263,7 +344,7 @@ func main() {
 		err := ioutil.WriteFile(fileName, []byte(content), 0600)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Cannot write to file %q: %v\n", fileName, err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 	}
 
@@ -276,7 +357,7 @@ func main() {
 		dat, err := ioutil.ReadFile(fileName)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Cannot read file %q: %v\n", fileName, err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Print("<<<")
 		fmt.Print(string(dat))
@@ -292,49 +373,92 @@ func main() {
 		fi, err := os.Stat(fileName)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Cannot stat file %q: %v\n", fileName, err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("%s: mode: %s\n", fileName, fi.Mode().String())
 		fmt.Printf("%s: user: %v\n", fileName, fi.Sys().(*syscall.Stat_t).Uid)
 		fmt.Printf("%s: group: %v\n", fileName, fi.Sys().(*syscall.Stat_t).Gid)
 	}
 
+	if globalFlags.HashFile {
+		fileName := os.Getenv("FILE")
+		if globalFlags.FileName != "" {
+			fileName = globalFlags.FileName
+		}
+
+		dat, err := ioutil.ReadFile(fileName)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Cannot read file %q: %v\n", fileName, err)
+			os.Exit(254)
+		}
+
+		fmt.Printf("sha1sum: %x\n", sha1.Sum(dat))
+	}
+
+	if globalFlags.FileSymlinkTarget {
+		fileName := os.Getenv("FILE")
+		if globalFlags.FileName != "" {
+			fileName = globalFlags.FileName
+		}
+
+		dst, err := os.Readlink(fileName)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Cannot read file target %q: %v\n", fileName, err)
+			os.Exit(1)
+		}
+
+		fmt.Printf("symlink: %q -> %q\n", fileName, dst)
+	}
+
 	if globalFlags.PrintCwd {
 		wd, err := os.Getwd()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Cannot get working directory: %v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("cwd: %s\n", wd)
 	}
 
-	if globalFlags.Sleep >= 0 {
-		time.Sleep(time.Duration(globalFlags.Sleep) * time.Second)
-	}
-
 	if globalFlags.PrintMemoryLimit {
-		memCgroupPath, err := cgroup.GetOwnCgroupPath("memory")
+		// we use /proc/1/root to escape the chroot we're in and read the file
+		isUnified, err := cgroup.IsCgroupUnified("/proc/1/root/")
 		if err != nil {
-			fmt.Fprintf(os.Stderr, "Error getting own memory cgroup path: %v\n", err)
-			os.Exit(1)
+			fmt.Fprintf(os.Stderr, "Error getting cgroup type: %v\n", err)
+			os.Exit(254)
+		}
+
+		var limitPath string
+		if isUnified {
+			cgroupPath, err := v2.GetOwnCgroupPath()
+			if err != nil {
+				fmt.Fprintf(os.Stderr, "Error getting own memory cgroup path: %v\n", err)
+				os.Exit(254)
+			}
+			limitPath = filepath.Join("/proc/1/root/sys/fs/cgroup/", cgroupPath, "memory.max")
+			fmt.Fprintln(os.Stderr, "limitPath:", limitPath)
+		} else {
+			memCgroupPath, err := v1.GetOwnCgroupPath("memory")
+			if err != nil {
+				fmt.Fprintf(os.Stderr, "Error getting own memory cgroup path: %v\n", err)
+				os.Exit(254)
+			}
+			limitPath = filepath.Join("/proc/1/root/sys/fs/cgroup/memory", memCgroupPath, "memory.limit_in_bytes")
+			fmt.Fprintln(os.Stderr, limitPath)
 		}
-		// we use /proc/1/root to escape the chroot we're in and read our
-		// memory limit
-		limitPath := filepath.Join("/proc/1/root/sys/fs/cgroup/memory", memCgroupPath, "memory.limit_in_bytes")
 		limit, err := ioutil.ReadFile(limitPath)
 		if err != nil {
-			fmt.Fprintf(os.Stderr, "Can't read memory.limit_in_bytes\n")
-			os.Exit(1)
+			fmt.Fprintf(os.Stderr, "Can't read %s\n", limitPath)
+			os.Exit(254)
 		}
 
 		fmt.Printf("Memory Limit: %s\n", string(limit))
 	}
 
 	if globalFlags.PrintCPUQuota {
-		cpuCgroupPath, err := cgroup.GetOwnCgroupPath("cpu")
+		cpuCgroupPath, err := v1.GetOwnCgroupPath("cpu")
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Error getting own cpu cgroup path: %v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		// we use /proc/1/root to escape the chroot we're in and read our
 		// cpu quota
@@ -342,30 +466,48 @@ func main() {
 		periodBytes, err := ioutil.ReadFile(periodPath)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Can't read cpu.cpu_period_us\n")
-			os.Exit(1)
+			os.Exit(254)
 		}
 		quotaPath := filepath.Join("/proc/1/root/sys/fs/cgroup/cpu", cpuCgroupPath, "cpu.cfs_quota_us")
 		quotaBytes, err := ioutil.ReadFile(quotaPath)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "Can't read cpu.cpu_quota_us\n")
-			os.Exit(1)
+			os.Exit(254)
 		}
 
 		period, err := strconv.Atoi(strings.Trim(string(periodBytes), "\n"))
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		quota, err := strconv.Atoi(strings.Trim(string(quotaBytes), "\n"))
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 
 		quotaMilliCores := quota * 1000 / period
 		fmt.Printf("CPU Quota: %s\n", strconv.Itoa(quotaMilliCores))
 	}
 
+	if globalFlags.PrintCPUShares {
+		cpuCgroupPath, err := v1.GetOwnCgroupPath("cpu")
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Error getting own cpu cgroup path: %v\n", err)
+			os.Exit(1)
+		}
+		// we use /proc/1/root to escape the chroot we're in and read our
+		// cpu quota
+		sharesPath := filepath.Join("/proc/1/root/sys/fs/cgroup/cpu", cpuCgroupPath, "cpu.shares")
+		sharesBytes, err := ioutil.ReadFile(sharesPath)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Can't read cpu.shares\n")
+			os.Exit(1)
+		}
+
+		fmt.Printf("CPU Shares: %s", string(sharesBytes))
+	}
+
 	if globalFlags.CheckCgroupMounts {
 		rootCgroupPath := "/proc/1/root/sys/fs/cgroup"
 		testPaths := []string{rootCgroupPath}
@@ -381,7 +523,7 @@ func main() {
 		for _, p := range testPaths {
 			if err := syscall.Mkdir(filepath.Join(p, "test"), 0600); err == nil || err != syscall.EROFS {
 				fmt.Fprintf(os.Stderr, "check-cgroups: FAIL (%v)", err)
-				os.Exit(1)
+				os.Exit(254)
 			}
 		}
 
@@ -402,11 +544,11 @@ func main() {
 		ips, err := testutils.GetIPsv4(iface)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		if len(ips) == 0 {
 			fmt.Fprintf(os.Stderr, "No IPv4 found for interface %+v:\n", iface)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("%v IPv4: %s\n", iface, ips[0])
 	}
@@ -415,7 +557,7 @@ func main() {
 		gw, err := testutils.GetDefaultGWv4()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("DefaultGWv4: %s\n", gw)
 	}
@@ -424,7 +566,7 @@ func main() {
 		gw, err := testutils.GetDefaultGWv6()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("DefaultGWv6: %s\n", gw)
 	}
@@ -435,7 +577,7 @@ func main() {
 		gw, err := testutils.GetGWv4(iface)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("%v GWv4: %s\n", iface, gw)
 	}
@@ -452,7 +594,7 @@ func main() {
 		hostname, err := os.Hostname()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("Hostname: %s\n", hostname)
 	}
@@ -461,7 +603,7 @@ func main() {
 		err := testutils.HTTPServe(globalFlags.ServeHTTP, globalFlags.ServeHTTPTimeout)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 	}
 
@@ -469,7 +611,7 @@ func main() {
 		body, err := testutils.HTTPGet(globalFlags.GetHTTP)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("HTTP-Get received: %s\n", body)
 	}
@@ -478,7 +620,7 @@ func main() {
 		ifaceCount, err := testutils.GetIfaceCount()
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("Interface count: %d\n", ifaceCount)
 	}
@@ -488,7 +630,7 @@ func main() {
 		body, err := testutils.HTTPGet(fmt.Sprintf("%s/acMetadata/v1/apps/%s/annotations/%s", mdsUrl, appName, globalFlags.PrintAppAnnotation))
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		fmt.Printf("Annotation %s=%s\n", globalFlags.PrintAppAnnotation, body)
 	}
@@ -497,20 +639,24 @@ func main() {
 		appMountNS, err := os.Readlink("/proc/self/ns/mnt")
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		s1MountNS, err := os.Readlink("/proc/1/ns/mnt")
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "%v\n", err)
-			os.Exit(1)
+			os.Exit(254)
 		}
 		if appMountNS != s1MountNS {
 			fmt.Println("check-mountns: DIFFERENT")
 		} else {
 			fmt.Println("check-mountns: IDENTICAL")
-			os.Exit(1)
+			os.Exit(254)
 		}
 	}
 
+	if globalFlags.Sleep >= 0 {
+		time.Sleep(time.Duration(globalFlags.Sleep) * time.Second)
+	}
+
 	os.Exit(globalFlags.ExitCode)
 }
