@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-// +build ignore
+// +build !fly,!kvm
 
 package main
 
@@ -24,6 +24,11 @@ import (
 	"github.com/coreos/rkt/tests/testutils"
 )
 
+// Spin up the appc ACE validator pod and return the result.
+
+// You'll find the actual application at vendor/github.com/appc/spec/ace/validator.go,
+// the two ACIs are built from that
+
 func TestAceValidator(t *testing.T) {
 	newStringSet := func(strs ...string) map[string]struct{} {
 		m := make(map[string]struct{}, len(strs))
@@ -35,9 +40,9 @@ func TestAceValidator(t *testing.T) {
 	expected := []map[string]struct{}{
 		newStringSet("prestart"),
 		newStringSet("main", "sidekick"),
-		newStringSet("poststop"),
+		// newStringSet("poststop"), // Disabled by caseyc for #2870
 	}
-	pattern := `ace-validator\[\d+\]: ([[:alpha:]]+) OK`
+	pattern := `ace-validator-(?:main|sidekick)\[\d+\]: ([[:alpha:]]+) OK`
 
 	ctx := testutils.NewRktRunCtx()
 	defer ctx.Cleanup()
