@@ -103,7 +103,7 @@ func checkPodState(t *testing.T, rawState string, apiState v1alpha.PodState) {
 		if apiState == v1alpha.PodState_POD_STATE_EXITED {
 			return
 		}
-	case "garbage":
+	case "garbage", "exited garbage":
 		if apiState == v1alpha.PodState_POD_STATE_GARBAGE {
 			return
 		}
@@ -346,7 +346,10 @@ func NewAPIServiceListInspectPodsTest() testutils.Test {
 		}
 
 		patches := []string{"--exec=/inspect --print-msg=HELLO_API --exit-code=0"}
-		imageHash := patchImportAndFetchHash("rkt-inspect-print.aci", patches, t, ctx)
+		imageHash, err := patchImportAndFetchHash("rkt-inspect-print.aci", patches, t, ctx)
+		if err != nil {
+			t.Fatalf("%v", err)
+		}
 		imgID, err := types.NewHash(imageHash)
 		if err != nil {
 			t.Fatalf("Cannot generate types.Hash from %v: %v", imageHash, err)
@@ -394,8 +397,8 @@ func NewAPIServiceListInspectPodsTest() testutils.Test {
 						},
 					},
 				},
-				"non-existed-network",
-				1,
+				"non-existent-network",
+				254,
 			},
 		}
 
@@ -503,7 +506,10 @@ func TestAPIServiceListInspectImages(t *testing.T) {
 		t.Errorf("Unexpected result: %v, should see zero images", resp.Images)
 	}
 
-	patchImportAndFetchHash("rkt-inspect-sleep.aci", []string{"--exec=/inspect"}, t, ctx)
+	_, err = patchImportAndFetchHash("rkt-inspect-sleep.aci", []string{"--exec=/inspect"}, t, ctx)
+	if err != nil {
+		t.Fatalf("%v", err)
+	}
 
 	// ListImages(detail=false).
 	resp, err = c.ListImages(context.Background(), &v1alpha.ListImagesRequest{})
@@ -611,6 +617,7 @@ func NewAPIServiceCgroupTest() testutils.Test {
 		}
 
 		// ListPods(detail=true). Filter according to the cgroup.
+		t.Logf("Calling ListPods with cgroup filter %v", cgroups)
 		resp, err = c.ListPods(context.Background(), &v1alpha.ListPodsRequest{
 			Detail:  true,
 			Filters: []*v1alpha.PodFilter{{Cgroups: cgroups}},
@@ -627,6 +634,7 @@ func NewAPIServiceCgroupTest() testutils.Test {
 			checkPodDetails(t, ctx, p)
 		}
 
+		t.Logf("Calling ListPods with subcgroup filter %v", subcgroups)
 		resp, err = c.ListPods(context.Background(), &v1alpha.ListPodsRequest{
 			Detail:  true,
 			Filters: []*v1alpha.PodFilter{{PodSubCgroups: subcgroups}},
