@@ -21,7 +21,6 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
-	"syscall"
 	"testing"
 
 	"github.com/coreos/rkt/common"
@@ -41,7 +40,7 @@ func TestGC(t *testing.T) {
 
 	// Abort prepare.
 	cmd := fmt.Sprintf("%s --insecure-options=image prepare %s %s", ctx.Cmd(), imagePath, imagePath)
-	spawnAndWaitOrFail(t, cmd, 1)
+	spawnAndWaitOrFail(t, cmd, 254)
 
 	gcCmd := fmt.Sprintf("%s gc --mark-only=true --expire-prepared=0 --grace-period=0", ctx.Cmd())
 	spawnAndWaitOrFail(t, gcCmd, 0)
@@ -81,8 +80,8 @@ func podsRemaining(t *testing.T, ctx *testutils.RktRunCtx) []os.FileInfo {
 }
 
 func TestGCAfterUnmount(t *testing.T) {
-	if !common.SupportsOverlay() {
-		t.Skip("Overlay fs not supported.")
+	if err := common.SupportsOverlay(); err != nil {
+		t.Skipf("Overlay fs not supported: %v", err)
 	}
 
 	ctx := testutils.NewRktRunCtx()
@@ -91,46 +90,24 @@ func TestGCAfterUnmount(t *testing.T) {
 	imagePath := getInspectImagePath()
 
 	for _, rmNetns := range []bool{false, true} {
-
-		importImageAndFetchHash(t, ctx, "", imagePath)
+		_, err := importImageAndFetchHash(t, ctx, "", imagePath)
+		if err != nil {
+			t.Fatalf("%v", err)
+		}
 		cmd := fmt.Sprintf("%s --insecure-options=image prepare %s", ctx.Cmd(), imagePath)
 		uuid := runRktAndGetUUID(t, cmd)
 
 		cmd = fmt.Sprintf("%s run-prepared %s", ctx.Cmd(), uuid)
 		runRktAndCheckOutput(t, cmd, "", false)
 
-		podDir := filepath.Join(ctx.DataDir(), "pods", "run", uuid)
-		stage1MntPath := filepath.Join(podDir, "stage1", "rootfs")
-		stage2MntPath := filepath.Join(stage1MntPath, "opt", "stage2", "rkt-inspect", "rootfs")
-		netnsPath := filepath.Join(podDir, "netns")
-		podNetNSPathBytes, err := ioutil.ReadFile(netnsPath)
-		if err != nil {
-			t.Fatalf(`cannot read "netns" stage1: %v`, err)
-		}
-		podNetNSPath := string(podNetNSPathBytes)
-
-		if err := syscall.Unmount(stage2MntPath, 0); err != nil {
-			t.Fatalf("cannot umount stage2: %v", err)
-		}
-
-		if err := syscall.Unmount(stage1MntPath, 0); err != nil {
-			t.Fatalf("cannot umount stage1: %v", err)
-		}
-
-		if err := syscall.Unmount(podNetNSPath, 0); err != nil {
-			t.Fatalf("cannot umount pod netns: %v", err)
-		}
-
-		if rmNetns {
-			_ = os.RemoveAll(podNetNSPath)
-		}
+		unmountPod(t, ctx, uuid, rmNetns)
 
 		pods := podsRemaining(t, ctx)
 		if len(pods) == 0 {
 			t.Fatalf("pods should still be present in rkt's data directory")
 		}
 
-		gcCmd := fmt.Sprintf("%s gc --mark-only=false --expire-prepared=0 --grace-period=0", ctx.Cmd())
+		gcCmd := fmt.Sprintf("%s gc --debug --mark-only=false --expire-prepared=0 --grace-period=0", ctx.Cmd())
 		// check we don't get any output (an error) after "executing net-plugin..."
 		runRktAndCheckRegexOutput(t, gcCmd, `executing net-plugin .*\n\z`)
 
