@@ -17,13 +17,9 @@
 package main
 
 import (
-	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"os"
-	"path"
 	"strconv"
-	"strings"
 	"testing"
 
 	"github.com/appc/spec/schema"
@@ -34,8 +30,6 @@ import (
 	"github.com/coreos/rkt/common/cgroup"
 )
 
-const baseAppName = "rkt-inspect"
-
 func intP(i int) *int {
 	return &i
 }
@@ -43,40 +37,12 @@ func stringP(s string) *string {
 	return &s
 }
 
-func verifyHostFile(t *testing.T, tmpdir, filename string, i int, expectedResult string) {
-	filePath := path.Join(tmpdir, filename)
-	defer os.Remove(filePath)
-
-	// Verify the file is written to host.
-	if strings.Contains(expectedResult, "host:") {
-		data, err := ioutil.ReadFile(filePath)
-		if err != nil {
-			t.Fatalf("%d: Cannot read the host file: %v", i, err)
-		}
-		if string(data) != expectedResult {
-			t.Fatalf("%d: Expecting %q in the host file, but saw %q", i, expectedResult, data)
-		}
+func mustNewIsolator(body string) (i types.Isolator) {
+	err := i.UnmarshalJSON([]byte(body))
+	if err != nil {
+		panic(err)
 	}
-}
-
-func rawValue(value string) *json.RawMessage {
-	msg := json.RawMessage(value)
-	return &msg
-}
-
-func rawRequestLimit(request, limit string) *json.RawMessage {
-	if request == "" {
-		return rawValue(fmt.Sprintf(`{"limit":%q}`, limit))
-	}
-	if limit == "" {
-		return rawValue(fmt.Sprintf(`{"request":%q}`, request))
-	}
-	return rawValue(fmt.Sprintf(`{"request":%q,"limit":%q}`, request, limit))
-}
-
-type imagePatch struct {
-	name    string
-	patches []string
+	return
 }
 
 // Test running pod manifests that contains just one app.
@@ -208,7 +174,11 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "empty:foo"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
@@ -245,7 +215,11 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "empty:foo"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
@@ -282,7 +256,11 @@ func TestPodManifest(t *testing.T) {
 								{"FILE", "/dir1"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
@@ -318,7 +296,11 @@ func TestPodManifest(t *testing.T) {
 								{"FILE", "/dir1"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
@@ -356,13 +338,26 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "host:foo"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -387,14 +382,27 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "host:foo"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -419,16 +427,29 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", true},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: true,
+								},
 							},
 						},
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -450,17 +471,30 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", true},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: true,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -484,16 +518,29 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  &boolTrue,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -517,17 +564,30 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir1", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir1",
+									ReadOnly: false,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  &boolTrue,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -550,13 +610,26 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "host:bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir2", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir2",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -582,14 +655,27 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "host:bar"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir1", "/dir2", false},
+								{
+									Name:     "dir1",
+									Path:     "/dir2",
+									ReadOnly: false,
+								},
 							},
 						},
 						ReadOnlyRootFS: true,
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -612,7 +698,16 @@ func TestPodManifest(t *testing.T) {
 					{Name: baseAppName},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -638,7 +733,16 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -648,7 +752,7 @@ func TestPodManifest(t *testing.T) {
 		{
 			// Simple read after write with volume mounted, no apps in pod manifest.
 			// This should succeed even the mount point in image manifest is readOnly,
-			// because it is overriden by the volume's readOnly.
+			// because it is overridden by the volume's readOnly.
 			[]imagePatch{
 				{
 					"rkt-test-run-pod-manifest-vol-ro-no-app.aci",
@@ -663,7 +767,16 @@ func TestPodManifest(t *testing.T) {
 					{Name: baseAppName},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, &boolFalse, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  &boolFalse,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -687,10 +800,19 @@ func TestPodManifest(t *testing.T) {
 					{Name: baseAppName},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  &boolTrue,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -714,10 +836,19 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir1", "host", tmpdir, &boolTrue, nil, nil, nil},
+					{
+						Name:      "dir1",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  &boolTrue,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
-			1,
+			254,
 			`Cannot write to file "/dir1/file": open /dir1/file: read-only file system`,
 			"",
 		},
@@ -735,14 +866,14 @@ func TestPodManifest(t *testing.T) {
 							User:  "0",
 							Group: "0",
 							Isolators: []types.Isolator{
-								{
-									Name:     "resource/cpu",
-									ValueRaw: rawRequestLimit("100", "100"),
-								},
-								{
-									Name:     "os/linux/capabilities-retain-set",
-									ValueRaw: rawValue(`{"set":["CAP_SYS_PTRACE"]}`),
-								},
+								mustNewIsolator(`{
+									"name":     "resource/cpu",
+									"value":    { "request": "100m", "limit": "100m"}
+								}`),
+								mustNewIsolator(`{
+									"name":     "os/linux/capabilities-retain-set",
+									"value":    { "set": ["CAP_SYS_PTRACE"] }
+								}`),
 							},
 						},
 					},
@@ -766,15 +897,15 @@ func TestPodManifest(t *testing.T) {
 							User:  "0",
 							Group: "0",
 							Isolators: []types.Isolator{
-								{
-									Name: "resource/memory",
-									// 4MB.
-									ValueRaw: rawRequestLimit("4194304", "4194304"),
-								},
-								{
-									Name:     "os/linux/capabilities-retain-set",
-									ValueRaw: rawValue(`{"set":["CAP_SYS_PTRACE"]}`),
-								},
+								// 4MB.
+								mustNewIsolator(`{
+									"name":     "resource/memory",
+									"value":    { "request": "4194304", "limit": "4194304"}
+								}`),
+								mustNewIsolator(`{
+									"name":     "os/linux/capabilities-retain-set",
+									"value":    { "set": ["CAP_SYS_PTRACE"] }
+								}`),
 							},
 						},
 					},
@@ -803,7 +934,11 @@ func TestPodManifest(t *testing.T) {
 								{"FILE", "/dir/file"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir", "/dir", false},
+								{
+									Name:     "dir",
+									Path:     "/dir",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
@@ -818,13 +953,26 @@ func TestPodManifest(t *testing.T) {
 								{"CONTENT", "host:foo"},
 							},
 							MountPoints: []types.MountPoint{
-								{"dir", "/dir", false},
+								{
+									Name:     "dir",
+									Path:     "/dir",
+									ReadOnly: false,
+								},
 							},
 						},
 					},
 				},
 				Volumes: []types.Volume{
-					{"dir", "host", tmpdir, nil, nil, nil, nil},
+					{
+						Name:      "dir",
+						Kind:      "host",
+						Source:    tmpdir,
+						ReadOnly:  nil,
+						Recursive: nil,
+						Mode:      nil,
+						UID:       nil,
+						GID:       nil,
+					},
 				},
 			},
 			0,
@@ -872,10 +1020,10 @@ func TestPodManifest(t *testing.T) {
 								{"CAPABILITY", strconv.Itoa(int(capability.CAP_NET_ADMIN))},
 							},
 							Isolators: []types.Isolator{
-								{
-									Name:     "os/linux/capabilities-retain-set",
-									ValueRaw: rawValue(fmt.Sprintf(`{"set":["CAP_NET_ADMIN"]}`)),
-								},
+								mustNewIsolator(`{
+									"name":     "os/linux/capabilities-retain-set",
+									"value":    { "set": ["CAP_NET_ADMIN"] }
+								}`),
 							},
 						},
 					},
@@ -966,7 +1114,7 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 			},
-			1,
+			254,
 			`"user2" user not found`,
 			"",
 		},
@@ -987,7 +1135,7 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 			},
-			1,
+			254,
 			`"group2" group not found`,
 			"",
 		},
@@ -1029,7 +1177,7 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 			},
-			1,
+			254,
 			`no such file or directory`,
 			"",
 		},
@@ -1050,21 +1198,30 @@ func TestPodManifest(t *testing.T) {
 					},
 				},
 			},
-			1,
+			254,
 			`no such file or directory`,
 			"",
 		},
 	}
 
 	for i, tt := range tests {
-		if tt.cgroup != "" && !cgroup.IsIsolatorSupported(tt.cgroup) {
-			t.Logf("Skip test #%v: cgroup %s not supported", i, tt.cgroup)
-			continue
+		if tt.cgroup != "" {
+			ok, err := cgroup.IsIsolatorSupported(tt.cgroup)
+			if err != nil {
+				t.Fatalf("Error checking memory isolator support: %v", err)
+			}
+			if !ok {
+				t.Logf("Skip test #%v: cgroup %s not supported", i, tt.cgroup)
+				continue
+			}
 		}
 
 		var hashesToRemove []string
 		for j, v := range tt.images {
-			hash := patchImportAndFetchHash(v.name, v.patches, t, ctx)
+			hash, err := patchImportAndFetchHash(v.name, v.patches, t, ctx)
+			if err != nil {
+				t.Fatalf("%v", err)
+			}
 			hashesToRemove = append(hashesToRemove, hash)
 			imgName := types.MustACIdentifier(v.name)
 			imgID, err := types.NewHash(hash)
@@ -1124,3 +1281,24 @@ func TestPodManifest(t *testing.T) {
 		}
 	}
 }
+
+func TestPodManifestWithEmptyApps(t *testing.T) {
+	ctx := testutils.NewRktRunCtx()
+	defer ctx.Cleanup()
+
+	tmpdir := createTempDirOrPanic("rkt-tests.")
+	defer os.RemoveAll(tmpdir)
+
+	manifest := &schema.PodManifest{
+		Apps:      []schema.RuntimeApp{},
+		ACKind:    schema.PodManifestKind,
+		ACVersion: schema.AppContainerVersion,
+	}
+
+	manifestFile := generatePodManifestFile(t, manifest)
+	defer os.Remove(manifestFile)
+
+	runCmd := fmt.Sprintf("%s run --mds-register=false --pod-manifest=%s", ctx.Cmd(), manifestFile)
+	runEmptyAppsMsg := "pod must contain at least one application"
+	runRktAndCheckOutput(t, runCmd, runEmptyAppsMsg, true)
+}
