@@ -232,7 +232,7 @@ func TestStage1LoadingFromConfigRelativePathFail(t *testing.T) {
 	setup.generateStage1Config(cfg)
 	cmd := fmt.Sprintf("%s --insecure-options=image,tls --debug run %s", setup.ctx.Cmd(), getInspectImagePath())
 	child := spawnOrFail(setup.t, cmd)
-	defer waitOrFail(setup.t, child, 1)
+	defer waitOrFail(setup.t, child, 254)
 	expectedLine := "default stage1 image location is either a relative path or a URL without scheme"
 	setup.getExpectedOrFail(child, expectedLine)
 }
@@ -308,8 +308,8 @@ func TestStage1LoadingFromFlags(t *testing.T) {
 		// --stage1-path with a relative path
 		{
 			flag:              fmt.Sprintf("--stage1-path=%q", setup.getLocation(stubStage1PathRel)),
-			expectedFirstRun:  fmt.Sprintf("image: using image from file %s", setup.getLocation(stubStage1PathRel)),
-			expectedSecondRun: fmt.Sprintf("image: using image from file %s", setup.getLocation(stubStage1PathRel)),
+			expectedFirstRun:  fmt.Sprintf("image: using image from file %s", setup.getLocation(stubStage1PathAbs)),
+			expectedSecondRun: fmt.Sprintf("image: using image from file %s", setup.getLocation(stubStage1PathAbs)),
 		},
 
 		// --stage1-path with an absolute path
@@ -400,7 +400,10 @@ func TestStage1LoadingFromFlagsHash(t *testing.T) {
 	setup := newStubStage1Setup(t, nil)
 	defer setup.cleanup()
 
-	stubHash := importImageAndFetchHash(setup.t, setup.ctx, "", setup.getLocation(stubStage1PathAbs))
+	stubHash, err := importImageAndFetchHash(setup.t, setup.ctx, "", setup.getLocation(stubStage1PathAbs))
+	if err != nil {
+		t.Fatalf("%v", err)
+	}
 	cmd := fmt.Sprintf("%s --insecure-options=image,tls --debug run --stage1-hash=%s %s", setup.ctx.Cmd(), stubHash, getInspectImagePath())
 	child := spawnOrFail(setup.t, cmd)
 	defer waitOrFail(setup.t, child, 0)
