@@ -39,8 +39,10 @@ import (
 	"github.com/appc/spec/schema/types"
 	"github.com/coreos/gexpect"
 	"github.com/coreos/rkt/api/v1alpha"
+	"github.com/coreos/rkt/common"
 	"github.com/coreos/rkt/tests/testutils"
 	taas "github.com/coreos/rkt/tests/testutils/aci-server"
+	shellquote "github.com/kballard/go-shellquote"
 	"google.golang.org/grpc"
 )
 
@@ -48,6 +50,8 @@ const (
 	defaultTimeLayout = "2006-01-02 15:04:05.999 -0700 MST"
 	nobodyUid         = 65534
 	nobodyGid         = 65534
+
+	baseAppName = "rkt-inspect"
 )
 
 func expectCommon(p *gexpect.ExpectSubprocess, searchString string, timeout time.Duration) error {
@@ -120,25 +124,61 @@ func spawnOrFail(t *testing.T, cmd string) *gexpect.ExpectSubprocess {
 	return child
 }
 
-func getExitStatus(err error) int {
-	if err == nil {
-		return 0
+// waitOrFail waits for the child to exit, draining all its output.
+// If a non-negative return value is provided, child exit status must match.
+func waitOrFail(t *testing.T, child *gexpect.ExpectSubprocess, expectedStatus int) {
+	bufOut := []string{}
+	// TODO(lucab): gexpect should accept those channels from the caller
+	ttyIn, ttyOut := child.AsyncInteractChannels()
+	close(ttyIn)
+	// drain output till gexpect closes the channel (on EOF or error)
+	for line := range ttyOut {
+		bufOut = append(bufOut, line)
 	}
-	if exiterr, ok := err.(*exec.ExitError); ok {
-		// the program has exited with an exit code != 0
-		if status, ok := exiterr.Sys().(syscall.WaitStatus); ok {
-			return status.ExitStatus()
-		}
+	err := child.Wait()
+	status, _ := common.GetExitStatus(err)
+	if expectedStatus >= 0 && status != expectedStatus {
+		t.Fatalf("rkt terminated with unexpected status %d, expected %d\nOutput:\n%s", status, expectedStatus, bufOut)
 	}
-	return -1
 }
 
-func waitOrFail(t *testing.T, child *gexpect.ExpectSubprocess, expectedStatus int) {
-	err := child.Wait()
-	status := getExitStatus(err)
-	if status != expectedStatus {
-		t.Fatalf("rkt terminated with unexpected status %d, expected %d\nOutput:\n%s", status, expectedStatus, child.Collect())
+// waitPodReady waits for the pod supervisor to get ready, busy-looping until `timeout`
+// while waiting for it. It returns the pod UUID or an error on failure.
+func waitPodReady(ctx *testutils.RktRunCtx, uuidFile string, timeout time.Duration) (string, error) {
+	var podUUID []byte
+	var err error
+	interval := 500 * time.Millisecond
+	elapsed := time.Duration(0)
+
+	for elapsed < timeout {
+		time.Sleep(interval)
+		elapsed += interval
+		podUUID, err = ioutil.ReadFile(uuidFile)
+		if err == nil {
+			break
+		}
 	}
+	if err != nil {
+		return "", fmt.Errorf("Can't read pod UUID: %v", err)
+	}
+
+	// wait for the pod supervisor to be ready
+	podReadyFile := filepath.Join(ctx.DataDir(), "pods", "run", string(podUUID), "stage1/rootfs/rkt/supervisor-status")
+	target := ""
+	elapsed = time.Duration(0)
+	for elapsed < timeout {
+		time.Sleep(interval)
+		elapsed += interval
+		target, err = os.Readlink(podReadyFile)
+		if err == nil && target == "ready" {
+			break
+		}
+	}
+	if err != nil || target != "ready" {
+		return "", fmt.Errorf("Pod failed to become ready while checking %q", podReadyFile)
+	}
+
+	return string(podUUID), nil
 }
 
 func spawnAndWaitOrFail(t *testing.T, cmd string, expectedStatus int) {
@@ -186,7 +226,7 @@ func createTempDirOrPanic(dirName string) string {
 	return tmpDir
 }
 
-func importImageAndFetchHashAsGid(t *testing.T, ctx *testutils.RktRunCtx, img string, fetchArgs string, gid int) string {
+func importImageAndFetchHashAsGid(t *testing.T, ctx *testutils.RktRunCtx, img string, fetchArgs string, gid int) (string, error) {
 	// Import the test image into store manually.
 	cmd := fmt.Sprintf("%s --insecure-options=image,tls fetch %s %s", ctx.Cmd(), fetchArgs, img)
 
@@ -212,16 +252,18 @@ func importImageAndFetchHashAsGid(t *testing.T, ctx *testutils.RktRunCtx, img st
 
 	// Read out the image hash.
 	result, out, err := expectRegexWithOutput(child, "sha512-[0-9a-f]{32,64}")
+	if exitErr := checkExitStatus(child); exitErr != nil {
+		t.Logf("%v", exitErr)
+		return "", fmt.Errorf("fetching of %q failed", img)
+	}
 	if err != nil || len(result) != 1 {
 		t.Fatalf("Error: %v\nOutput: %v", err, out)
 	}
 
-	waitOrFail(t, child, 0)
-
-	return result[0]
+	return result[0], nil
 }
 
-func importImageAndFetchHash(t *testing.T, ctx *testutils.RktRunCtx, fetchArgs string, img string) string {
+func importImageAndFetchHash(t *testing.T, ctx *testutils.RktRunCtx, fetchArgs string, img string) (string, error) {
 	return importImageAndFetchHashAsGid(t, ctx, fetchArgs, img, 0)
 }
 
@@ -235,7 +277,7 @@ func importImageAndPrepare(imagePath string, t *testing.T, ctx *testutils.RktRun
 	spawnAndWaitOrFail(t, cmd, 0)
 }
 
-func patchImportAndFetchHash(image string, patches []string, t *testing.T, ctx *testutils.RktRunCtx) string {
+func patchImportAndFetchHash(image string, patches []string, t *testing.T, ctx *testutils.RktRunCtx) (string, error) {
 	imagePath := patchTestACI(image, patches...)
 	defer os.Remove(imagePath)
 
@@ -277,7 +319,7 @@ func runRktAndGetUUID(t *testing.T, rktCmd string) string {
 	child := spawnOrFail(t, rktCmd)
 	defer waitOrFail(t, child, 0)
 
-	result, out, err := expectRegexWithOutput(child, "\n[0-9a-f-]{36}")
+	result, out, err := expectRegexWithOutput(child, "[0-9a-f-]{36}")
 	if err != nil || len(result) != 1 {
 		t.Fatalf("Error: %v\nOutput: %v", err, out)
 	}
@@ -311,7 +353,7 @@ func runRktAsUidGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, exp
 	}
 	expectedStatus := 0
 	if expectError {
-		expectedStatus = 1
+		expectedStatus = 254
 	}
 	defer waitOrFail(t, child, expectedStatus)
 
@@ -344,7 +386,8 @@ func runRkt(t *testing.T, rktCmd string, uid, gid int) (string, int) {
 		buf.WriteString(line + "\n") // reappend newline
 	}
 
-	return buf.String(), getExitStatus(child.Wait())
+	status, _ := common.GetExitStatus(child.Wait())
+	return buf.String(), status
 }
 
 func startRktAsGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, gid int) *gexpect.ExpectSubprocess {
@@ -392,14 +435,28 @@ func startRktAsUidGidAndCheckOutput(t *testing.T, rktCmd, expectedLine string, e
 	return child
 }
 
-func runRktAndCheckRegexOutput(t *testing.T, rktCmd, match string) {
-	child := spawnOrFail(t, rktCmd)
-	defer child.Wait()
-
-	result, out, err := expectRegexWithOutput(child, match)
-	if err != nil || len(result) != 1 {
-		t.Fatalf("%q regex must be found one time, Error: %v\nOutput: %v", match, err, out)
+func runRktAndCheckRegexOutput(t *testing.T, rktCmd, match string) error {
+	re, err := regexp.Compile(match)
+	if err != nil {
+		t.Fatalf("error compiling regex %q: %v", match, err)
 	}
+
+	args, err := shellquote.Split(rktCmd)
+	if err != nil {
+		t.Fatalf("error splitting cmd %q: %v", rktCmd, err)
+	}
+
+	path, err := exec.LookPath(args[0])
+	cmd := exec.Command(path, args[1:]...)
+
+	out, err := cmd.CombinedOutput()
+
+	result := re.MatchString(string(out))
+	if !result {
+		t.Fatalf("%q regex must be found\nOutput: %q", match, string(out))
+	}
+
+	return err
 }
 
 func runRktAndCheckOutput(t *testing.T, rktCmd, expectedLine string, expectError bool) {
@@ -480,6 +537,11 @@ type podInfo struct {
 	startedAt int64
 }
 
+type imagePatch struct {
+	name    string
+	patches []string
+}
+
 // parsePodInfo parses the 'rkt status $UUID' result into podInfo struct.
 // For example, the 'result' can be:
 // state=running
@@ -618,18 +680,18 @@ func getPodInfo(t *testing.T, ctx *testutils.RktRunCtx, podID string) *podInfo {
 // For example, the 'result' can be:
 // 'sha512-e9b77714dbbfda12cb9e136318b103a6f0ce082004d09d0224a620d2bbf38133 nginx:latest 2015-10-16 17:42:57.741 -0700 PDT true'
 func parseImageInfoOutput(t *testing.T, result string) *imageInfo {
-	fields := regexp.MustCompile("\t+").Split(result, 6)
+	fields := regexp.MustCompile("\t+").Split(result, -1)
 	nameVersion := strings.Split(fields[1], ":")
 	if len(nameVersion) != 2 {
 		t.Fatalf("Failed to parse name version string: %q", fields[1])
 	}
-	importTime, err := time.Parse(defaultTimeLayout, fields[2])
+	importTime, err := time.Parse(defaultTimeLayout, fields[3])
 	if err != nil {
-		t.Fatalf("Failed to parse time string: %q", fields[2])
+		t.Fatalf("Failed to parse time string: %q", fields[3])
 	}
-	size, err := strconv.Atoi(fields[4])
+	size, err := strconv.Atoi(fields[2])
 	if err != nil {
-		t.Fatalf("Failed to parse image size string: %q", fields[4])
+		t.Fatalf("Failed to parse image size string: %q", fields[2])
 	}
 
 	return &imageInfo{
@@ -815,3 +877,71 @@ func writeConfig(t *testing.T, dir, filename, contents string) {
 		t.Fatalf("Failed to write file %q: %v", path, err)
 	}
 }
+
+func verifyHostFile(t *testing.T, tmpdir, filename string, i int, expectedResult string) {
+	filePath := path.Join(tmpdir, filename)
+	defer os.Remove(filePath)
+
+	// Verify the file is written to host.
+	if strings.Contains(expectedResult, "host:") {
+		data, err := ioutil.ReadFile(filePath)
+		if err != nil {
+			t.Fatalf("%d: Cannot read the host file: %v", i, err)
+		}
+		if string(data) != expectedResult {
+			t.Fatalf("%d: Expecting %q in the host file, but saw %q", i, expectedResult, data)
+		}
+	}
+}
+
+func executeFuncsReverse(funcs []func()) {
+	n := len(funcs)
+	for i := n - 1; i >= 0; i-- {
+		funcs[i]()
+	}
+}
+
+func unmountPod(t *testing.T, ctx *testutils.RktRunCtx, uuid string, rmNetns bool) {
+	podDir := filepath.Join(ctx.DataDir(), "pods", "run", uuid)
+	stage1MntPath := filepath.Join(podDir, "stage1", "rootfs")
+	stage2MntPath := filepath.Join(stage1MntPath, "opt", "stage2", "rkt-inspect", "rootfs")
+
+	netnsPath := filepath.Join(podDir, "netns")
+	podNetNSPathBytes, err := ioutil.ReadFile(netnsPath)
+	// There may be no netns, e.g. kvm or --net=host
+	if err != nil {
+		if !os.IsNotExist(err) {
+			t.Fatalf(`cannot read "netns" stage1: %v`, err)
+		} else {
+			rmNetns = false
+		}
+	}
+
+	if err := syscall.Unmount(stage2MntPath, 0); err != nil {
+		t.Fatalf("cannot umount stage2: %v", err)
+	}
+
+	if err := syscall.Unmount(stage1MntPath, 0); err != nil {
+		t.Fatalf("cannot umount stage1: %v", err)
+	}
+
+	if rmNetns {
+		podNetNSPath := string(podNetNSPathBytes)
+
+		if err := syscall.Unmount(podNetNSPath, 0); err != nil {
+			t.Fatalf("cannot umount pod netns: %v", err)
+		}
+
+		_ = os.RemoveAll(podNetNSPath)
+	}
+}
+
+func checkExitStatus(child *gexpect.ExpectSubprocess) error {
+	err := child.Wait()
+	status, _ := common.GetExitStatus(err)
+	if status != 0 {
+		return fmt.Errorf("rkt terminated with unexpected status %d, expected %d\nOutput:\n%s", status, 0, child.Collect())
+	}
+
+	return nil
+}
